@@ -1,17 +1,17 @@
 // File_Aac - Info for AAC (Raw) files
-// Copyright (C) 2008-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2008-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -35,6 +35,7 @@
 
 //---------------------------------------------------------------------------
 #include "MediaInfo/Audio/File_Aac.h"
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -59,6 +60,7 @@ File_Aac::File_Aac()
     MustSynchronize=true;
     Buffer_TotalBytes_FirstSynched_Max=64*1024;
     PTS_DTS_Needed=true;
+    IsRawStream=true;
 
     //In
     Frame_Count_Valid=MediaInfoLib::Config.ParseSpeed_Get()>=0.5?128:(MediaInfoLib::Config.ParseSpeed_Get()>=0.3?32:2);
@@ -76,6 +78,8 @@ File_Aac::File_Aac()
     aacSpectralDataResilienceFlag=false;
     aacSectionDataResilienceFlag=false;
     aacScalefactorDataResilienceFlag=false;
+    FrameSize_Min=(int64u)-1;
+    FrameSize_Max=0;
 
     //Temp - Main
     muxConfigPresent=true;
@@ -105,14 +109,14 @@ void File_Aac::Streams_Fill()
 {
     switch(Mode)
     {
-        case Mode_LATM                : Fill(Stream_General, 0, General_Format, "LATM"); if (IsSub) Fill(Stream_Audio, 0, Audio_MuxingMode, "LATM"); break;
-        default                       : ;
+        case Mode_LATM : Fill(Stream_General, 0, General_Format, "LATM"); if (IsSub) Fill(Stream_Audio, 0, Audio_MuxingMode, "LATM"); break;
+        default : ;
     }
 
-    for (std::map<std::string, Ztring>::iterator Info=Infos_General.begin(); Info!=Infos_General.end(); Info++)
+    for (std::map<std::string, Ztring>::iterator Info=Infos_General.begin(); Info!=Infos_General.end(); ++Info)
         Fill(Stream_General, 0, Info->first.c_str(), Info->second);
     File__Tags_Helper::Stream_Prepare(Stream_Audio);
-    for (std::map<std::string, Ztring>::iterator Info=Infos.begin(); Info!=Infos.end(); Info++)
+    for (std::map<std::string, Ztring>::iterator Info=Infos.begin(); Info!=Infos.end(); ++Info)
         Fill(Stream_Audio, StreamPos_Last, Info->first.c_str(), Info->second);
 
     switch(Mode)
@@ -131,6 +135,23 @@ void File_Aac::Streams_Finish()
         case Mode_ADTS    : File__Tags_Helper::Streams_Finish(); break;
         default           : ;
     }
+
+    if (FrameSize_Min!=(int32u)-1 && FrameSize_Max)
+    {
+        if (FrameSize_Max>FrameSize_Min*1.02)
+        {
+            Fill(Stream_Audio, 0, Audio_BitRate_Mode, "VBR", Unlimited, true, true);
+            if (Config->ParseSpeed>=1.0)
+            {
+                Fill(Stream_Audio, 0, Audio_BitRate_Minimum, ((float64)FrameSize_Min)/1024*48000*8, 0);
+                Fill(Stream_Audio, 0, Audio_BitRate_Maximum, ((float64)FrameSize_Max)/1024*48000*8, 0);
+            }
+        }
+        else if (Config->ParseSpeed>=1.0)
+        {
+            Fill(Stream_Audio, 0, Audio_BitRate_Mode, "CBR");
+        }
+    }
 }
 
 //***************************************************************************
@@ -206,7 +227,10 @@ void File_Aac::Read_Buffer_Continue()
 {
     if (Element_Size==0)
         return;
-        
+
+    if (Frame_Count==0)
+        PTS_Begin=FrameInfo.PTS;
+
     switch(Mode)
     {
         case Mode_AudioSpecificConfig : Read_Buffer_Continue_AudioSpecificConfig(); break;
@@ -247,7 +271,9 @@ void File_Aac::Read_Buffer_Continue_raw_data_block()
     FILLING_BEGIN();
         //Counting
         Frame_Count++;
-        Element_Info(Ztring::ToZtring(Frame_Count));
+        if (Frame_Count_NotParsedIncluded!=(int64u)-1)
+            Frame_Count_NotParsedIncluded++;
+        Element_Info1(Ztring::ToZtring(Frame_Count));
 
         //Filling
         if (!Status[IsAccepted])
@@ -301,6 +327,10 @@ bool File_Aac::Synchronize_ADTS()
                 break;
             if (File_Offset+Buffer_Offset+aac_frame_length!=File_Size-File_EndTagSize)
             {
+                //Padding
+                while (Buffer_Offset+aac_frame_length+2<=Buffer_Size && Buffer[Buffer_Offset+aac_frame_length]==0x00)
+                    aac_frame_length++;
+
                 if (Buffer_Offset+aac_frame_length+2>Buffer_Size)
                     return false; //Need more data
 
@@ -313,6 +343,10 @@ bool File_Aac::Synchronize_ADTS()
                     int16u aac_frame_length2=(CC3(Buffer+Buffer_Offset+aac_frame_length+3)>>5)&0x1FFF;
                     if (File_Offset+Buffer_Offset+aac_frame_length+aac_frame_length2!=File_Size-File_EndTagSize)
                     {
+                        //Padding
+                        while (Buffer_Offset+aac_frame_length+aac_frame_length2+2<=Buffer_Size && Buffer[Buffer_Offset+aac_frame_length+aac_frame_length2]==0x00)
+                            aac_frame_length2++;
+
                         if (Buffer_Offset+aac_frame_length+aac_frame_length2+2>Buffer_Size)
                             return false; //Need more data
 
@@ -325,6 +359,10 @@ bool File_Aac::Synchronize_ADTS()
                             int16u aac_frame_length3=(CC3(Buffer+Buffer_Offset+aac_frame_length+aac_frame_length2+3)>>5)&0x1FFF;
                             if (File_Offset+Buffer_Offset+aac_frame_length+aac_frame_length2+aac_frame_length3!=File_Size-File_EndTagSize)
                             {
+                                //Padding
+                                while (Buffer_Offset+aac_frame_length+aac_frame_length2+aac_frame_length3+2<=Buffer_Size && Buffer[Buffer_Offset+aac_frame_length+aac_frame_length2+aac_frame_length3]==0x00)
+                                    aac_frame_length3++;
+
                                 if (Buffer_Offset+aac_frame_length+aac_frame_length2+aac_frame_length3+2>Buffer_Size)
                                     return false; //Need more data
 
@@ -449,6 +487,10 @@ bool File_Aac::Synched_Test_ADTS()
     if (!File__Tags_Helper::Synched_Test())
         return false;
 
+    //Null padding
+    while (Buffer_Offset+2<=Buffer_Size && Buffer[Buffer_Offset]==0x00)
+        Buffer_Offset++;
+
     //Must have enough buffer for having header
     if (Buffer_Offset+2>Buffer_Size)
         return false;
@@ -590,6 +632,11 @@ void File_Aac::Header_Parse_LATM()
 //---------------------------------------------------------------------------
 void File_Aac::Data_Parse()
 {
+    if (FrameSize_Min>Header_Size+Element_Size)
+        FrameSize_Min=Header_Size+Element_Size;
+    if (FrameSize_Max<Header_Size+Element_Size)
+        FrameSize_Max=Header_Size+Element_Size;
+
     if (Frame_Count>Frame_Count_Valid || CanFill)
     {
         Skip_XX(Element_Size,                                   "Data");
@@ -610,16 +657,21 @@ void File_Aac::Data_Parse()
         if (File_Offset+Buffer_Offset+Element_Size==File_Size)
             Frame_Count_Valid=Frame_Count; //Finish frames in case of there are less than Frame_Count_Valid frames
         Frame_Count++;
-        Element_Info(Ztring::ToZtring(Frame_Count));
+        if (Frame_Count_NotParsedIncluded!=(int64u)-1)
+            Frame_Count_NotParsedIncluded++;
+        Element_Info1(Ztring::ToZtring(Frame_Count));
 
         //Filling
-        if (Frame_Count>=Frame_Count_Valid || CanFill)
+        if ((Frame_Count>=Frame_Count_Valid || CanFill) && Config->ParseSpeed<1.0)
         {
             //No more need data
             switch (Mode)
             {
-                case Mode_ADTS        : File__Tags_Helper::Finish(); break;
-                case Mode_LATM        : File__Analyze::Finish(); break;
+                case Mode_ADTS        :
+                case Mode_LATM        :
+                                        Fill();
+                                        if (!IsSub)
+                                            File__Tags_Helper::Finish(); break;
                 default               : ; //No header
             }
 
