@@ -1,17 +1,17 @@
 // File_Mpega - Info for MPEG Audio files
-// Copyright (C) 2002-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2002-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -254,7 +254,7 @@ const int8u Mpega_SlotSize[4]= //A frame is coposed of slots
     1, // Layer3
     1, // Layer2
     4, // Layer1
-};                       
+};
 
 //---------------------------------------------------------------------------
 const int16u Mpega_CRC12_Table[]=
@@ -309,6 +309,7 @@ File_Mpega::File_Mpega()
     Buffer_TotalBytes_FirstSynched_Max=64*1024;
     PTS_DTS_Needed=true;
     IsRawStream=true;
+    Frame_Count_NotParsedIncluded=0;
 
     //In
     Frame_Count_Valid=MediaInfoLib::Config.ParseSpeed_Get()>=0.5?128:(MediaInfoLib::Config.ParseSpeed_Get()>=0.3?32:4);
@@ -356,7 +357,7 @@ void File_Mpega::Streams_Fill()
     {
         //How much kinds of bitrates?
         if (BitRate_Count.size()>1)
-            BitRate_Mode=_T("VBR");
+            BitRate_Mode=__T("VBR");
     }
 
     File__Tags_Helper::Stream_Prepare(Stream_Audio);
@@ -388,9 +389,9 @@ void File_Mpega::Streams_Fill()
     }
 
     //Bitrate, if CBR
-    if (VBR_Frames==0 && BitRate_Mode!=_T("VBR"))
+    if (VBR_Frames==0 && BitRate_Mode!=__T("VBR"))
     {
-        BitRate_Mode=_T("CBR");
+        BitRate_Mode=__T("CBR");
         BitRate=Mpega_BitRate[ID][layer][bitrate_index]*1000;
         Fill(Stream_General, 0, General_OverallBitRate, BitRate);
         Fill(Stream_Audio, 0, Audio_BitRate, BitRate);
@@ -418,7 +419,7 @@ void File_Mpega::Streams_Finish()
     //Fill("Reservoir_Max", Reservoir_Max);
     //size_t Granules=(Mpeg==3?2:1);
     //size_t Ch=Mpega_Channels[Channels];
-    //Fill("Scalefactors", Ztring::ToZtring(Scalefac*100/(Granules*Ch*Frame_Count))+_T('%'));
+    //Fill("Scalefactors", Ztring::ToZtring(Scalefac*100/(Granules*Ch*Frame_Count))+__T('%'));
 
     //VBR_FileSize calculation
     if (!IsSub && (File_Size!=(int64u)-1 || LastSync_Offset!=(int64u)-1) && VBR_FileSize==0)
@@ -454,15 +455,15 @@ void File_Mpega::Streams_Finish()
             Divider=1152/8;
         if (ID<4 && sampling_frequency<4)
             BitRate=(int32u)(FrameLength*Mpega_SamplingRate[ID][sampling_frequency]/Divider);
-        BitRate_Mode=_T("VBR");
+        BitRate_Mode=__T("VBR");
     }
     //if (BitRate_Count.size()>1)
     //{
     //    Ztring BitRate_VBR;
     //    if (!BitRate_VBR.empty())
-    //        BitRate_VBR+=_T(' ');
+    //        BitRate_VBR+=__T(' ');
     //    BitRate_VBR+=Ztring::ToZtring(8);
-    //    BitRate_VBR+=_T(':');
+    //    BitRate_VBR+=__T(':');
     //    BitRate_VBR+=Ztring::ToZtring(BitRate_Count[8]);
     //    Fill("BitRate_VBR", Ztring::ToZtring(BitRate_Count[8]));
     //}
@@ -500,7 +501,7 @@ void File_Mpega::Streams_Finish()
     if (FrameInfo.PTS!=(int64u)-1 && FrameInfo.PTS>PTS_Begin)
     {
         Fill(Stream_Audio, 0, Audio_Duration, float64_int64s(((float64)(FrameInfo.PTS-PTS_Begin))/1000000));
-        if (Retrieve(Stream_Audio, 0, Audio_BitRate_Mode)==_T("CBR"))
+        if (Retrieve(Stream_Audio, 0, Audio_BitRate_Mode)==__T("CBR"))
         {
             int16u Samples;
             if (ID==3 && layer==3) //MPEG 1 layer 1
@@ -515,7 +516,7 @@ void File_Mpega::Streams_Finish()
         }
     }
 
-    if (FrameCount==0 && VBR_FileSize && Retrieve(Stream_Audio, 0, Audio_BitRate_Mode)==_T("CBR") && Mpega_SamplingRate[ID][sampling_frequency])
+    if (FrameCount==0 && VBR_FileSize && Retrieve(Stream_Audio, 0, Audio_BitRate_Mode)==__T("CBR") && Mpega_SamplingRate[ID][sampling_frequency])
     {
         size_t Size=(Mpega_Coefficient[ID][layer]*Mpega_BitRate[ID][layer][bitrate_index]*1000/Mpega_SamplingRate[ID][sampling_frequency])*Mpega_SlotSize[layer];
         if (Size)
@@ -632,6 +633,10 @@ bool File_Mpega::Synchronize()
                     break;
                 if (File_Offset+Buffer_Offset+Size0!=File_Size-File_EndTagSize)
                 {
+                    //Padding
+                    while (Buffer_Offset+Size0+4<=Buffer_Size && Buffer[Buffer_Offset+Size0]==0x00)
+                        Size0++;
+
                     if (Buffer_Offset+Size0+4>Buffer_Size)
                         return false; //Need more data
 
@@ -685,6 +690,10 @@ bool File_Mpega::Synchronize()
                                 break;
                             if (File_Offset+Buffer_Offset+Size0+Size1!=File_Size-File_EndTagSize)
                             {
+                                //Padding
+                                while (Buffer_Offset+Size0+Size1+4<=Buffer_Size && Buffer[Buffer_Offset+Size0+Size1]==0x00)
+                                    Size0++;
+
                                 if (Buffer_Offset+Size0+Size1+4>Buffer_Size)
                                     return false; //Need more data
 
@@ -719,6 +728,10 @@ bool File_Mpega::Synchronize()
                                             break;
                                         if (File_Offset+Buffer_Offset+Size0+Size1+Size2!=File_Size-File_EndTagSize)
                                         {
+                                            //Padding
+                                            while (Buffer_Offset+Size0+Size1+Size2+4<=Buffer_Size && Buffer[Buffer_Offset+Size0+Size1+Size2]==0x00)
+                                                Size0++;
+
                                             if (Buffer_Offset+Size0+Size1+Size2+4>Buffer_Size)
                                             {
                                                 if (IsSub || File_Offset+Buffer_Offset+Size0+Size1+Size2<File_Size)
@@ -782,6 +795,10 @@ bool File_Mpega::Synched_Test()
     if (!File__Tags_Helper::Synched_Test())
         return false;
 
+    //Padding
+    while (Buffer_Offset<Buffer_Size && Buffer[Buffer_Offset]==0x00)
+        Buffer_Offset++;
+
     //Must have enough buffer for having header
     if (Buffer_Offset+3>Buffer_Size)
         return false;
@@ -846,18 +863,18 @@ void File_Mpega::Header_Parse()
     //Parsing
     BS_Begin();
     Skip_S2(11,                                                 "syncword");
-    Get_S1 (2, ID,                                              "ID"); Param_Info(Mpega_Version[ID]);
-    Get_S1 (2, layer,                                           "layer"); Param_Info(Mpega_Layer[layer]);
+    Get_S1 (2, ID,                                              "ID"); Param_Info1(Mpega_Version[ID]);
+    Get_S1 (2, layer,                                           "layer"); Param_Info1(Mpega_Layer[layer]);
     Get_SB (   protection_bit,                                  "protection_bit");
-    Get_S1 (4, bitrate_index,                                   "bitrate_index"); Param_Info(Mpega_BitRate[ID][layer][bitrate_index], " Kbps");
-    Get_S1 (2, sampling_frequency,                              "sampling_frequency"); Param_Info(Mpega_SamplingRate[ID][sampling_frequency], " Hz");
+    Get_S1 (4, bitrate_index,                                   "bitrate_index"); Param_Info2(Mpega_BitRate[ID][layer][bitrate_index], " Kbps");
+    Get_S1 (2, sampling_frequency,                              "sampling_frequency"); Param_Info2(Mpega_SamplingRate[ID][sampling_frequency], " Hz");
     Get_SB (   padding_bit,                                     "padding_bit");
     Skip_SB(                                                    "private_bit");
-    Get_S1 (2, mode,                                            "mode"); Param_Info(Mpega_Channels[mode], " channels"); Param_Info(Mpega_Codec_Profile[mode]);
-    Get_S1 (2, mode_extension,                                  "mode_extension"); Param_Info(Mpega_Codec_Profile_Extension[mode_extension]);
+    Get_S1 (2, mode,                                            "mode"); Param_Info2(Mpega_Channels[mode], " channels"); Param_Info1(Mpega_Codec_Profile[mode]);
+    Get_S1 (2, mode_extension,                                  "mode_extension"); Param_Info1(Mpega_Codec_Profile_Extension[mode_extension]);
     Get_SB (   copyright,                                       "copyright");
     Get_SB (   original_home,                                   "original_home");
-    Get_S1 (2, emphasis,                                        "emphasis"); Param_Info(Mpega_Emphasis[emphasis]);
+    Get_S1 (2, emphasis,                                        "emphasis"); Param_Info1(Mpega_Emphasis[emphasis]);
     BS_End();
 
     //Coherancy
@@ -898,7 +915,7 @@ void File_Mpega::Data_Parse()
     }
 
     //Partial frame
-    if (Header_Size+Element_Size<(Mpega_Coefficient[ID][layer]*Mpega_BitRate[ID][layer][bitrate_index]*1000/Mpega_SamplingRate[ID][sampling_frequency]+(padding_bit?1:0))*Mpega_SlotSize[layer])
+    if (Header_Size+Element_Size<(int64u)((Mpega_Coefficient[ID][layer]*Mpega_BitRate[ID][layer][bitrate_index]*1000/Mpega_SamplingRate[ID][sampling_frequency]+(padding_bit?1:0))*Mpega_SlotSize[layer]))
     {
         Element_Name("Partial frame");
         Skip_XX(Element_Size,                                   "Data");
@@ -906,11 +923,10 @@ void File_Mpega::Data_Parse()
     }
 
     //PTS
-    if (FrameInfo.PTS!=(int64u)-1)
-        Element_Info(_T("PTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.PTS)/1000000)));
+    Element_Info1C((FrameInfo.PTS!=(int64u)-1), __T("PTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.PTS)/1000000)));
 
     //Name
-    Element_Info(_T("Frame ")+Ztring::ToZtring(Frame_Count));
+    Element_Info1(__T("Frame ")+Ztring::ToZtring(Frame_Count));
 
     //VBR and library headers
     if (Frame_Count<3) //No need to do it too much
@@ -926,6 +942,8 @@ void File_Mpega::Data_Parse()
         PTS_Begin=FrameInfo.PTS;
     Frame_Count++;
     Frame_Count_InThisBlock++;
+    if (Frame_Count_NotParsedIncluded!=(int64u)-1)
+        Frame_Count_NotParsedIncluded++;
     LastSync_Offset=File_Offset+Buffer_Offset+Element_Size;
     {
         int16u Samples;
@@ -986,7 +1004,7 @@ void File_Mpega::Data_Parse()
     }
     if (ID==3) //MPEG-1
     {
-    Element_Begin("scfsi");
+    Element_Begin1("scfsi");
         for(int8u ch=0; ch<Mpega_Channels[mode]; ch++)
             for(int8u scfsi_band=0; scfsi_band<4; scfsi_band++)
             {
@@ -995,16 +1013,16 @@ void File_Mpega::Data_Parse()
                 if (scfsi)
                     Scfsi++;
             }
-        Element_End();
+        Element_End0();
     }
     for(int8u gr=0; gr<(ID==3?2:1); gr++)
     {
-        Element_Begin("granule");
+        Element_Begin1("granule");
         if (mode>=4)
             return;
         for(int8u ch=0; ch<Mpega_Channels[mode]; ch++)
         {
-            Element_Begin("channel");
+            Element_Begin1("channel");
             Skip_S2(12,                                         "part2_3_length");
             Skip_S2(9,                                          "big_values");
             Skip_S1(8,                                          "global_gain");
@@ -1028,18 +1046,18 @@ void File_Mpega::Data_Parse()
                 {
                     if (mixed_block_flag==1)
                     {
-                        Param_Info("Mixed");
+                        Param_Info1("Mixed");
                         Block_Count[2]++; //Mixed
                     }
                     else
                     {
-                        Param_Info("Short");
+                        Param_Info1("Short");
                         Block_Count[1]++; //Short
                     }
                 }
                 else
                 {
-                    Param_Info("Long");
+                    Param_Info1("Long");
                     Block_Count[0]++; //Long
                 }
             }
@@ -1049,7 +1067,7 @@ void File_Mpega::Data_Parse()
                     Skip_S1(5,                                  "table_select");
                 Skip_S1(4,                                      "region0_count");
                 Skip_S1(3,                                      "region1_count");
-                Param_Info("Long");
+                Param_Info1("Long");
                 Block_Count[0]++; //Long
             }
             if (ID==3) //MPEG-1
@@ -1059,9 +1077,9 @@ void File_Mpega::Data_Parse()
             if (scalefac)
                 Scalefac++;
             Skip_SB(                                            "count1table_select");
-            Element_End();
+            Element_End0();
         } //channels
-        Element_End();
+        Element_End0();
     } //granules
     BS_End();
 
@@ -1091,13 +1109,13 @@ void File_Mpega::Data_Parse()
             //Parsing
             Skip_XX(Element_Offset_S-Element_Offset,            "data");
             BS_Begin();
-            Element_Begin("Surround");
+            Element_Begin1("Surround");
             Skip_S2(12,                                         "Sync");
             Skip_S1( 8,                                         "Size");
             Skip_S2(12,                                         "CRC12");
             BS_End();
             Skip_XX(Surround_Size-4,                            "data");
-            Element_End();
+            Element_End0();
 
             //Filling
             Surround_Frames++;
@@ -1158,11 +1176,11 @@ bool File_Mpega::Header_Xing()
         if (CC4(Xing_Header)==CC4("Xing") || CC4(Xing_Header)==CC4("Info"))
         {
             //This is a "tag"
-            Element_Info("Tag (Xing)");
+            Element_Info1("Tag (Xing)");
 
             //Parsing
-            Element_Begin("Xing");
-            Element_Begin("Xing header");
+            Element_Begin1("Xing");
+            Element_Begin1("Xing header");
             Skip_XX(Xing_Header_Offset,                         "Junk");
             int32u Flags;
             bool FrameCount, FileSize, TOC, Scale, Lame;
@@ -1179,7 +1197,7 @@ bool File_Mpega::Header_Xing()
                                    +(TOC?       100:0)    //TOC
                                    +(Scale?       4:0)    //Scale
                                    +(Lame?      348:0);   //Lame
-            Element_End();
+            Element_End0();
             //Element size
             if (Xing_Header_Size>Element_Size-Xing_Header_Offset)
                 return false; //Error tag size
@@ -1199,9 +1217,9 @@ bool File_Mpega::Header_Xing()
             if (Scale)
                 Get_B4 (Xing_Scale,                             "Scale");
             Ztring Lib;
-            Element_End();
+            Element_End0();
             Peek_Local(4, Lib);
-            if (Lame || Lib==_T("LAME") || Lib==_T("GOGO"))
+            if (Lame || Lib==__T("LAME") || Lib==__T("GOGO") || Lib==__T("L3.9"))
                 Header_Encoders_Lame();
 
             if (CC4(Xing_Header)==CC4("Info"))
@@ -1228,13 +1246,13 @@ bool File_Mpega::Header_VBRI()
         {
             //This is a "tag"
 
-            Element_Info("Tag (VBRI)");
+            Element_Info1("Tag (VBRI)");
 
             //Parsing
             int32u VBR_FileSize_Temp;
             int16u TableSize, TableScale, EntryBytes;
             Skip_XX(Fraunhofer_Header_Offset,                   "Junk");
-            Element_Begin("VBRI");
+            Element_Begin1("VBRI");
             Skip_C4(                                            "Sync");
             Skip_B2(                                            "Version");
             Skip_B2(                                            "Delay");
@@ -1245,19 +1263,19 @@ bool File_Mpega::Header_VBRI()
             Get_B2 (TableScale,                                 "TableScale");
             Get_B2 (EntryBytes,                                 "EntryBytes");
             Skip_B2(                                            "EntryFrames"); //Count of frames per entry
-            Element_Begin("Table");
+            Element_Begin1("Table");
                 for (int16u Pos=0; Pos<TableSize; Pos++)
                 {
                     switch (EntryBytes)
                     {
-                        case 1 : {Info_B1(Entry,                "Entry"); Param_Info (Entry*TableScale, " bytes");} break;
-                        case 2 : {Info_B2(Entry,                "Entry"); Param_Info (Entry*TableScale, " bytes");} break;
-                        case 4 : {Info_B4(Entry,                "Entry"); Param_Info (Entry*TableScale, " bytes");} break;
+                        case 1 : {Info_B1(Entry,                "Entry"); Param_Info2 (Entry*TableScale, " bytes");} break;
+                        case 2 : {Info_B2(Entry,                "Entry"); Param_Info2 (Entry*TableScale, " bytes");} break;
+                        case 4 : {Info_B4(Entry,                "Entry"); Param_Info2 (Entry*TableScale, " bytes");} break;
                         default: Skip_XX(EntryBytes,            "Entry");
                     }
                 }
-            Element_End();
-            Element_End();
+            Element_End0();
+            Element_End0();
             VBR_FileSize=VBR_FileSize_Temp;
 
             //Clearing Error detection
@@ -1280,15 +1298,15 @@ bool File_Mpega::Header_Encoders()
     Buffer_Pos=BufferS.find("LAME");
     if (Buffer_Pos!=std::string::npos && Buffer_Pos<=Element_Size-8)
     {
-        Element_Info("With tag (Lame)");
+        Element_Info1("With tag (Lame)");
         Element_Offset=Buffer_Pos;
         if (Element_Offset+20<=Element_Size)
             Get_Local(20, Encoded_Library,                      "Encoded_Library");
         else
             Get_Local( 8, Encoded_Library,                      "Encoded_Library");
-        Encoded_Library.Trim(_T('A'));
-        Encoded_Library.Trim(_T('U'));
-        Encoded_Library.Trim(_T('\xAA'));
+        Encoded_Library.Trim(__T('A'));
+        Encoded_Library.Trim(__T('U'));
+        Encoded_Library.Trim(__T('\xAA'));
         Element_Offset=0; //Reseting it
         return true;
     }
@@ -1297,7 +1315,7 @@ bool File_Mpega::Header_Encoders()
     Buffer_Pos=BufferS.find("RCA mp3PRO Encoder");
     if (Buffer_Pos!=std::string::npos && Buffer_Pos<Element_Size-23)
     {
-        Element_Info("With tag (RCA)");
+        Element_Info1("With tag (RCA)");
         Encoded_Library="RCA ";
         Encoded_Library+=Ztring((const char*)(Buffer+Buffer_Offset+18), 5);
         return true;
@@ -1307,7 +1325,7 @@ bool File_Mpega::Header_Encoders()
     Buffer_Pos=BufferS.find("THOMSON mp3PRO Encoder");
     if (Buffer_Pos!=std::string::npos && Buffer_Pos<Element_Size-29)
     {
-        Element_Info("With tag (Thomson)");
+        Element_Info1("With tag (Thomson)");
         Encoded_Library="Thomson ";
         Encoded_Library+=Ztring((const char*)(Buffer+Buffer_Offset+22), 6);
         return true;
@@ -1317,7 +1335,7 @@ bool File_Mpega::Header_Encoders()
     Buffer_Pos=BufferS.find("MPGE");
     if (Buffer_Pos!=std::string::npos)
     {
-        Element_Info("With tag (Gogo)");
+        Element_Info1("With tag (Gogo)");
         Encoded_Library="Gogo <3.0";
         return true;
     }
@@ -1326,7 +1344,7 @@ bool File_Mpega::Header_Encoders()
     Buffer_Pos=BufferS.find("GOGO");
     if (Buffer_Pos!=std::string::npos)
     {
-        Element_Info("With tag (Gogo)");
+        Element_Info1("With tag (Gogo)");
         Encoded_Library="Gogo >=3.0";
         return true;
     }
@@ -1337,25 +1355,27 @@ bool File_Mpega::Header_Encoders()
 void File_Mpega::Header_Encoders_Lame()
 {
     Peek_Local(8, Encoded_Library);
-    if (Encoded_Library>=_T("LAME3.90") && Element_IsNotFinished())
+    if (Encoded_Library.find(__T("L3.99"))==0)
+        Encoded_Library.insert(1, __T("AME")); //Ugly version string in Lame 3.99.1 "L3.99r1\0"
+    if ((Encoded_Library>=__T("LAME3.90")) && Element_IsNotFinished())
     {
-        int8u Flags, EncodingFlags, BitRate, StereoMode;
-        Param_Info(Ztring(_T("V "))+Ztring::ToZtring((100-Xing_Scale)/10));
-        Param_Info(Ztring(_T("q "))+Ztring::ToZtring((100-Xing_Scale)%10));
+        int8u Flags, lowpass, EncodingFlags, BitRate, StereoMode;
+        Param_Info1(Ztring(__T("V "))+Ztring::ToZtring((100-Xing_Scale)/10));
+        Param_Info1(Ztring(__T("q "))+Ztring::ToZtring((100-Xing_Scale)%10));
         Get_Local(9, Encoded_Library,                           "Encoded_Library");
         Get_B1 (Flags,                                          "Flags");
         if ((Flags&0xF0)<=0x20) //Rev. 0 or 1, http://gabriel.mp3-tech.org/mp3infotag.html and Rev. 2 was seen.
         {
-            Param_Info(Lame_Method[Flags&0x0F]);
+            Param_Info1(Lame_Method[Flags&0x0F]);
             BitRate_Mode=Lame_BitRate_Mode[Flags&0x0F];
             if ((Flags&0x0F)==1 || (Flags&0x0F)==8) //2 possible values for CBR
                 VBR_Frames=0;
         }
-        Info_B1(lowpass,                                        "Lowpass filter value"); Param_Info(lowpass*100, " Hz");
+        Get_B1 (lowpass,                                        "Lowpass filter value"); Param_Info2(lowpass*100, " Hz");
         Skip_B4(                                                "Peak signal amplitude");
         Skip_B2(                                                "Radio Replay Gain");
         Skip_B2(                                                "Audiophile Replay Gain");
-        Get_B1 (EncodingFlags,                                  "Encoding Flags"); Param_Info(Ztring(_T("ATH Type="))+Ztring::ToZtring(Flags&0x0F));
+        Get_B1 (EncodingFlags,                                  "Encoding Flags"); Param_Info1(Ztring(__T("ATH Type="))+Ztring::ToZtring(Flags&0x0F));
             Skip_Flags(EncodingFlags, 4,                        "nspsytune");
             Skip_Flags(EncodingFlags, 5,                        "nssafejoint");
             Skip_Flags(EncodingFlags, 6,                        "nogap (after)");
@@ -1375,36 +1395,36 @@ void File_Mpega::Header_Encoders_Lame()
         Skip_B2(                                                "CRC-16 of Info Tag");
 
         FILLING_BEGIN();
-            Encoded_Library_Settings+=_T("-m ");
+            Encoded_Library_Settings+=__T("-m ");
             switch(StereoMode)
             {
-                case 0 : Encoded_Library_Settings+=_T("m"); break;
-                case 1 : Encoded_Library_Settings+=_T("s"); break;
-                case 2 : Encoded_Library_Settings+=_T("d"); break;
-                case 3 : Encoded_Library_Settings+=_T("j"); break;
-                case 4 : Encoded_Library_Settings+=_T("f"); break;
-                case 5 : Encoded_Library_Settings+=_T("a"); break;
-                case 6 : Encoded_Library_Settings+=_T("i"); break;
+                case 0 : Encoded_Library_Settings+=__T("m"); break;
+                case 1 : Encoded_Library_Settings+=__T("s"); break;
+                case 2 : Encoded_Library_Settings+=__T("d"); break;
+                case 3 : Encoded_Library_Settings+=__T("j"); break;
+                case 4 : Encoded_Library_Settings+=__T("f"); break;
+                case 5 : Encoded_Library_Settings+=__T("a"); break;
+                case 6 : Encoded_Library_Settings+=__T("i"); break;
                 default: ;
             }
             if (Xing_Scale<=100) //Xing_Scale is used for LAME quality
             {
-                Encoded_Library_Settings+=_T( " -V ")+Ztring::ToZtring((100-Xing_Scale)/10);
-                Encoded_Library_Settings+=_T( " -q ")+Ztring::ToZtring((100-Xing_Scale)%10);
+                Encoded_Library_Settings+=__T( " -V ")+Ztring::ToZtring((100-Xing_Scale)/10);
+                Encoded_Library_Settings+=__T( " -q ")+Ztring::ToZtring((100-Xing_Scale)%10);
             }
             if (lowpass)
-                Encoded_Library_Settings+=(Encoded_Library_Settings.empty()?_T("-lowpass "):_T(" -lowpass "))+((lowpass%10)?Ztring::ToZtring(((float)lowpass)/10, 1):Ztring::ToZtring(lowpass/10));
+                Encoded_Library_Settings+=(Encoded_Library_Settings.empty()?__T("-lowpass "):__T(" -lowpass "))+((lowpass%10)?Ztring::ToZtring(((float)lowpass)/10, 1):Ztring::ToZtring(lowpass/10));
             switch (Flags&0x0F)
             {
                 case  2 :
                 case  9 : //ABR
-                            Encoded_Library_Settings+=_T(" --abr"); break;
+                            Encoded_Library_Settings+=__T(" --abr"); break;
                 case  3 : //VBR (old/rh)
-                            Encoded_Library_Settings+=_T(" --vbr-old"); break;
+                            Encoded_Library_Settings+=__T(" --vbr-old"); break;
                 case  4 : //VBR (new/mtrh)
-                            Encoded_Library_Settings+=_T(" --vbr-new"); break;
+                            Encoded_Library_Settings+=__T(" --vbr-new"); break;
                 case  5 : //VBR (?/mt)
-                            Encoded_Library_Settings+=_T(" --vbr-mt"); break;
+                            Encoded_Library_Settings+=__T(" --vbr-mt"); break;
                 default : ;
             }
             if (BitRate!=0x00 && BitRate!=0xFF)
@@ -1413,18 +1433,18 @@ void File_Mpega::Header_Encoders_Lame()
                 {
                     case  1 :
                     case  8 : //CBR
-                        Encoded_Library_Settings+=_T(" -b ")+Ztring::ToZtring(BitRate);
+                        Encoded_Library_Settings+=__T(" -b ")+Ztring::ToZtring(BitRate);
                         break;
                     case  2 :
                     case  9 : //ABR
                         BitRate_Nominal.From_Number(BitRate*1000);
-                        Encoded_Library_Settings+=_T(" ")+Ztring::ToZtring(BitRate);
+                        Encoded_Library_Settings+=__T(" ")+Ztring::ToZtring(BitRate);
                         break;
                     case  3 : //VBR (old/rh)
                     case  4 : //VBR (new/mtrh)
                     case  5 : //VBR (?/mt)
                         BitRate_Minimum.From_Number(BitRate*1000);
-                        Encoded_Library_Settings+=_T(" -b ")+Ztring::ToZtring(BitRate);
+                        Encoded_Library_Settings+=__T(" -b ")+Ztring::ToZtring(BitRate);
                         break;
                     default : ;
                 }
@@ -1515,13 +1535,13 @@ void File_Mpega::Encoded_Library_Guess()
                         //10 last bytes
                         //int sum = get_final_sum(data);
                         //if (sum==0)
-                        //    return guess = _T("FhG (fastenc, low quality mode)");
+                        //    return guess = __T("FhG (fastenc, low quality mode)");
                         //else if (sum==10 * 0xFF)
-                        //    return guess = _T("FhG (l3enc)");
+                        //    return guess = __T("FhG (l3enc)");
                         //else if (sum==5 * 0x20)
-                        //    return guess = _T("FhG (fastenc, medium or high quality mode)");
+                        //    return guess = __T("FhG (fastenc, medium or high quality mode)");
                         //else
-                        //    return guess = _T("FhG (l3enc or fastenc)");
+                        //    return guess = __T("FhG (l3enc or fastenc)");
                     }
                     else
                     {
@@ -1533,7 +1553,7 @@ void File_Mpega::Encoded_Library_Guess()
                 }
                 else //No padding
                 {
-                    if (BitRate_Mode.find(_T("VBR"))==0) //VBR
+                    if (BitRate_Mode.find(__T("VBR"))==0) //VBR
                         Encoded_Library="FhG (fastenc)";
                     else
                         Encoded_Library="FhG (ACM or producer pro)";
@@ -1548,7 +1568,7 @@ void File_Mpega::Encoded_Library_Guess()
                 }
                 else //Joint Stereo not used
                 {
-                    if (BitRate_Mode.find(_T("VBR"))==0) //VBR
+                    if (BitRate_Mode.find(__T("VBR"))==0) //VBR
                         Encoded_Library="Lame (old)";
                     else //CBR
                     {
@@ -1565,12 +1585,12 @@ void File_Mpega::Encoded_Library_Guess()
                             //if (data.av_reservoir < 40 && !data.vbr) //ISO based encoders are unable to properly use bit reservoir... average reservoir usage is about 10
                             //{
                             //    if (data.padding)
-                            //        return guess = _T("Blade");
+                            //        return guess = __T("Blade");
                             //    else
-                            //        return guess = _T("dist10 encoder or other encoder");
+                            //        return guess = __T("dist10 encoder or other encoder");
                             //}
                             //else
-                            //    return guess = _T("Gogo (before 3.0)");
+                            //    return guess = __T("Gogo (before 3.0)");
                         }
                     }
                 }
