@@ -1,17 +1,17 @@
 // File__Duplicate_MpegTs - Duplication of some formats
-// Copyright (C) 2007-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2007-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -80,7 +80,7 @@ File__Duplicate_MpegTs::File__Duplicate_MpegTs (const Ztring &Target)
 bool File__Duplicate_MpegTs::Configure (const Ztring &Value, bool ToRemove)
 {
     //Form: "program_number"
-    if (Value.find(_T("program_number="))==0)
+    if (Value.find(__T("program_number="))==0)
     {
         int16u program_number=Ztring(Value.substr(15, std::string::npos)).To_int16u();
         if (ToRemove)
@@ -101,7 +101,7 @@ bool File__Duplicate_MpegTs::Configure (const Ztring &Value, bool ToRemove)
             PAT.begin()->second.ConfigurationHasChanged=true;
     }
     //Form: "program_map_PID"
-    else if (Value.find(_T("program_map_PID="))==0)
+    else if (Value.find(__T("program_map_PID="))==0)
     {
         int16u program_map_PID=Ztring(Value.substr(16, std::string::npos)).To_int16u();
         if (ToRemove)
@@ -122,7 +122,7 @@ bool File__Duplicate_MpegTs::Configure (const Ztring &Value, bool ToRemove)
             PMT[program_map_PID].ConfigurationHasChanged=true;
     }
     //Form: "elementary_PID"
-    else if (Value.find(_T("elementary_PID="))==0)
+    else if (Value.find(__T("elementary_PID="))==0)
     {
         int16u elementary_PID=Ztring(Value.substr(15, std::string::npos)).To_int16u();
         if (ToRemove)
@@ -226,7 +226,7 @@ bool File__Duplicate_MpegTs::Manage_PAT (const int8u* ToAdd, size_t ToAdd_Size)
 
     //Reseting
     std::vector<int16u> StreamID_List;
-    for (std::map<int16u, buffer>::iterator PAT_=PAT.begin(); PAT_!=PAT.end(); PAT_++)
+    for (std::map<int16u, buffer>::iterator PAT_=PAT.begin(); PAT_!=PAT.end(); ++PAT_)
         if (PAT_->first!=StreamID)
             StreamID_List.push_back(PAT_->first);
     for (size_t Pos=0; Pos<StreamID_List.size(); Pos++)
@@ -244,7 +244,7 @@ bool File__Duplicate_MpegTs::Manage_PMT (const int8u* ToAdd, size_t ToAdd_Size)
     if (Wanted_program_numbers.find(StreamID)==Wanted_program_numbers.end()
      && Wanted_program_map_PIDs.find(elementary_PIDs_program_map_PIDs[StreamID])==Wanted_program_numbers.end())
     {
-        delete PMT[StreamID].Buffer; PMT[StreamID].Buffer=NULL;
+        delete[] PMT[StreamID].Buffer; PMT[StreamID].Buffer=NULL;
         return false;
     }
 
@@ -399,7 +399,7 @@ bool File__Duplicate_MpegTs::Parsing_Begin (const int8u* ToAdd, size_t ToAdd_Siz
 
     //Copying
     if (ToModify.Buffer!=NULL && ToModify.Size<FromTS.Size)
-        {delete ToModify.Buffer; ToModify.Buffer=NULL;}
+        {delete[] ToModify.Buffer; ToModify.Buffer=NULL;}
     if (ToModify.Buffer==NULL)
         ToModify.Buffer=new int8u[FromTS.Size+(FromTS.Size/188)*4];
     std::memcpy(ToModify.Buffer, FromTS.Buffer, FromTS.Begin+8); //Only up to last_section_number included
