@@ -1,17 +1,17 @@
 // File__MultipleParsing - Info for MultipleParsing files
-// Copyright (C) 2007-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2007-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -57,6 +57,9 @@
 #if defined(MEDIAINFO_GXF_YES)
     #include "MediaInfo/Multiple/File_Gxf.h"
 #endif
+#if defined(MEDIAINFO_HLS_YES)
+    #include "MediaInfo/Multiple/File_Hls.h"
+#endif
 #if defined(MEDIAINFO_ISM_YES)
     #include "MediaInfo/Multiple/File_Ism.h"
 #endif
@@ -96,6 +99,9 @@
 #if defined(MEDIAINFO_RM_YES)
     #include "MediaInfo/Multiple/File_Rm.h"
 #endif
+#if defined(MEDIAINFO_SEQUENCEINFO_YES)
+    #include "MediaInfo/Multiple/File_SequenceInfo.h"
+#endif
 #if defined(MEDIAINFO_SKM_YES)
     #include "MediaInfo/Multiple/File_Skm.h"
 #endif
@@ -123,6 +129,9 @@
 #if defined(MEDIAINFO_FLIC_YES)
     #include "MediaInfo/Video/File_Flic.h"
 #endif
+#if defined(MEDIAINFO_H263_YES)
+    #include "MediaInfo/Video/File_H263.h"
+#endif
 #if defined(MEDIAINFO_MPEG4V_YES)
     #include "MediaInfo/Video/File_Mpeg4v.h"
 #endif
@@ -147,8 +156,8 @@
 #if defined(MEDIAINFO_AC3_YES)
     #include "MediaInfo/Audio/File_Ac3.h"
 #endif
-#if defined(MEDIAINFO_AES3_YES)
-    #include "MediaInfo/Audio/File_Aes3.h"
+#if defined(MEDIAINFO_SMPTEST0337_YES)
+    #include "MediaInfo/Audio/File_SmpteSt0337.h"
 #endif
 #if defined(MEDIAINFO_ALS_YES)
     #include "MediaInfo/Audio/File_Als.h"
@@ -240,9 +249,15 @@
 #if defined(MEDIAINFO_JPEG_YES)
     #include "MediaInfo/Image/File_Jpeg.h"
 #endif
+#if defined(MEDIAINFO_PCX_YES)
+    #include "MediaInfo/Image/File_Pcx.h"
+#endif
 #if defined(MEDIAINFO_PNG_YES)
     #include "MediaInfo/Image/File_Png.h"
 #endif
+#if defined(MEDIAINFO_PSD_YES)
+    #include "MediaInfo/Image/File_Psd.h"
+#endif
 #if defined(MEDIAINFO_TIFF_YES)
     #include "MediaInfo/Image/File_Tiff.h"
 #endif
@@ -352,6 +367,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_GXF_YES)
         Temp=new File_Gxf(); Parser.push_back(Temp);
     #endif
+    #if defined(MEDIAINFO_HLS_YES)
+        Temp=new File_Hls(); Parser.push_back(Temp);
+    #endif
     #if defined(MEDIAINFO_ISM_YES)
         Temp=new File_Ism(); Parser.push_back(Temp);
     #endif
@@ -394,6 +412,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_RM_YES)
         Temp=new File_Rm(); Parser.push_back(Temp);
     #endif
+    #if defined(MEDIAINFO_SEQUENCEINFO_YES)
+        Temp=new File_SequenceInfo(); Parser.push_back(Temp);
+    #endif
     #if defined(MEDIAINFO_SKM_YES)
         Temp=new File_Skm(); Parser.push_back(Temp);
     #endif
@@ -420,6 +441,9 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_FLIC_YES)
         Temp=new File_Flic(); Parser.push_back(Temp);
     #endif
+    #if defined(MEDIAINFO_H263_YES)
+        Temp=new File_H263(); Parser.push_back(Temp);
+    #endif
     #if defined(MEDIAINFO_MPEG4V_YES)
         Temp=new File_Mpeg4v(); Parser.push_back(Temp);
     #endif
@@ -443,8 +467,8 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_AC3_YES)
         Temp=new File_Ac3(); Parser.push_back(Temp);
     #endif
-    #if defined(MEDIAINFO_AES3_YES)
-        Temp=new File_Aes3(); Parser.push_back(Temp);
+    #if defined(MEDIAINFO_SMPTEST0337_YES)
+        Temp=new File_SmpteSt0337(); Parser.push_back(Temp);
     #endif
     #if defined(MEDIAINFO_ALS_YES)
         Temp=new File_Als(); Parser.push_back(Temp);
@@ -531,9 +555,15 @@ File__MultipleParsing::File__MultipleParsing()
     #if defined(MEDIAINFO_JPEG_YES)
         Temp=new File_Jpeg(); Parser.push_back(Temp);
     #endif
+    #if defined(MEDIAINFO_PCX_YES)
+        Temp=new File_Pcx(); Parser.push_back(Temp);
+    #endif
     #if defined(MEDIAINFO_PNG_YES)
         Temp=new File_Png(); Parser.push_back(Temp);
     #endif
+    #if defined(MEDIAINFO_PSD_YES)
+        Temp=new File_Psd(); Parser.push_back(Temp);
+    #endif
     #if defined(MEDIAINFO_TIFF_YES)
         Temp=new File_Tiff(); Parser.push_back(Temp);
     #endif
@@ -689,4 +719,3 @@ void File__MultipleParsing::Read_Buffer_Continue()
 }
 
 } //NameSpace
-
