@@ -1,17 +1,17 @@
-// File_AncillaryDataPacket - Info for SCTE 20 streams
-// Copyright (C) 2010-2011 MediaArea.net SARL, Info@MediaArea.net
+// File_Ancillary - Info for Ancillary (SMPTE ST291) streams
+// Copyright (C) 2010-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -35,18 +35,14 @@
 
 //---------------------------------------------------------------------------
 #include "MediaInfo/Multiple/File_Ancillary.h"
-#if defined(MEDIAINFO_EIA608_YES)
-    #include "MediaInfo/Text/File_Eia608.h"
-#endif
-#if defined(MEDIAINFO_EIA708_YES)
-    #include "MediaInfo/Text/File_Eia708.h"
-#endif
 #if defined(MEDIAINFO_CDP_YES)
     #include "MediaInfo/Text/File_Cdp.h"
 #endif
 #if MEDIAINFO_EVENTS
     #include "MediaInfo/MediaInfo_Events.h"
 #endif //MEDIAINFO_EVENTS
+#include "MediaInfo/Multiple/File_Gxf_TimeCode.h"
+#include "MediaInfo/MediaInfo_Config_MediaInfo.h"
 #include <cstring>
 //---------------------------------------------------------------------------
 
@@ -100,13 +96,13 @@ const char* Ancillary_DataID(int8u DataID, int8u SecondaryDataID)
     else if (DataID<=0x5F)
         return "Reserved";
     else if (DataID==0x60)
-        return "Ancillary time code (Internationally registered)";
+        return "Ancillary time code"; //RP188
     else if (DataID==0x61)
     {
         switch (SecondaryDataID)
         {
-            case 0x01 : return "CEA-708 (CDP)";
-            case 0x02 : return "CEA-608";
+            case 0x01 : return "EIA-708 (CDP)";
+            case 0x02 : return "EIA-608";
             default   : return "S334-1-2007 Defined data services?";
         }
     }
@@ -138,8 +134,6 @@ const char* Ancillary_DataID(int8u DataID, int8u SecondaryDataID)
         return "Internationally registered";
     else if (DataID<=0xCF)
         return "User application";
-    else if (DataID<=0xDF)
-        return "Internationally registered";
     else
         return "Internationally registered";
 }
@@ -153,16 +147,21 @@ File_Ancillary::File_Ancillary()
 :File__Analyze()
 {
     //Configuration
-    ParserName=_T("Ancillary");
+    ParserName=__T("Ancillary");
+    PTS_DTS_Needed=true;
 
     //In
     WithTenBit=false;
     WithChecksum=false;
+    HasBFrames=false;
+    InDecodingOrder=false;
+    LineNumber_IsSecondField=false;
     AspectRatio=0;
     FrameRate=0;
-
-    //Temp
-    Cdp_Parser=NULL;
+    LineNumber=(int32u)-1;
+    #if defined(MEDIAINFO_CDP_YES)
+        Cdp_Parser=NULL;
+    #endif //defined(MEDIAINFO_CDP_YES)
 }
 
 //---------------------------------------------------------------------------
@@ -186,12 +185,53 @@ void File_Ancillary::Streams_Finish()
             {
                 Merge(*Cdp_Parser, Stream_Text, StreamPos, StreamPos);
                 Ztring MuxingMode=Cdp_Parser->Retrieve(Stream_Text, StreamPos, "MuxingMode");
-                Fill(Stream_Text, StreamPos, "MuxingMode", _T("Ancillary data / ")+MuxingMode, true);
+                Fill(Stream_Text, StreamPos, "MuxingMode", __T("Ancillary data / ")+MuxingMode, true);
             }
         }
     #endif //defined(MEDIAINFO_CDP_YES)
 }
 
+//***************************************************************************
+// Buffer - Synchro
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+bool File_Ancillary::Synchronize()
+{
+    //Synchronizing
+    while (Buffer_Offset+6<=Buffer_Size && ( Buffer[Buffer_Offset  ]!=0x00
+                                         ||  Buffer[Buffer_Offset+ 1]!=0xFF
+                                         ||  Buffer[Buffer_Offset+ 2]!=0xFF))
+        Buffer_Offset++;
+
+    if (!Status[IsAccepted])
+    {
+        Accept();
+
+        Fill(Stream_General, 0, General_Format, "Ancillary");
+    }
+
+    //Synched is OK
+    return true;
+}
+
+//---------------------------------------------------------------------------
+bool File_Ancillary::Synched_Test()
+{
+    Synchronize(); //This is not always in synch directly, there may be some garbage
+
+    //Must have enough buffer for having header
+    if (Buffer_Offset+6>Buffer_Size)
+        return false;
+
+    //Quick test of synchro
+    if (CC3(Buffer+Buffer_Offset)!=0x00FFFF)
+        Synched=false;
+
+    //We continue
+    return true;
+}
+
 //***************************************************************************
 // Buffer - Global
 //***************************************************************************
@@ -223,21 +263,86 @@ void File_Ancillary::Read_Buffer_Continue()
         return;
     }
 
-    if (!Status[IsAccepted])
+    if (!Status[IsAccepted] && !MustSynchronize)
         Accept();
+}
 
+//---------------------------------------------------------------------------
+void File_Ancillary::Read_Buffer_AfterParsing()
+{
+    Buffer_Offset=Buffer_Size; //This is per frame
+
+    Frame_Count++;
+    Frame_Count_InThisBlock++;
+    if (Frame_Count_NotParsedIncluded!=(int64u)-1)
+        Frame_Count_NotParsedIncluded++;
+}
+
+//---------------------------------------------------------------------------
+void File_Ancillary::Read_Buffer_Unsynched()
+{
+    #if defined(MEDIAINFO_CDP_YES)
+        for (size_t Pos=0; Pos<Cdp_Data.size(); Pos++)
+            delete Cdp_Data[Pos]; //Cdp_Data[Pos]=NULL;
+        Cdp_Data.clear();
+        if (Cdp_Parser)
+            Cdp_Parser->Open_Buffer_Unsynch();
+    #endif //defined(MEDIAINFO_CDP_YES)
+    #if defined(MEDIAINFO_AFDBARDATA_YES)
+        for (size_t Pos=0; Pos<AfdBarData_Data.size(); Pos++)
+            delete AfdBarData_Data[Pos]; //AfdBarData_Data[Pos]=NULL;
+        AfdBarData_Data.clear();
+    #endif //defined(MEDIAINFO_AFDBARDATA_YES)
+    AspectRatio=0;
+}
+//***************************************************************************
+// Buffer - Per element
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+void File_Ancillary::Header_Parse()
+{
     //Parsing
-    int8u DataID, SecondaryDataID, DataCount;
+    if (MustSynchronize)
+    {
+        if (WithTenBit)
+        {
+            Skip_L2(                                            "Ancillary data flag");
+            Skip_L2(                                            "Ancillary data flag");
+            Skip_L2(                                            "Ancillary data flag");
+        }
+        else
+        {
+            Skip_L1(                                            "Ancillary data flag");
+            Skip_L1(                                            "Ancillary data flag");
+            Skip_L1(                                            "Ancillary data flag");
+        }
+    }
     Get_L1 (DataID,                                             "Data ID");
     if (WithTenBit)
         Skip_L1(                                                "Parity+Unused"); //even:1, odd:2
-    Get_L1 (SecondaryDataID,                                    "Secondary Data ID"); Param_Info(Ancillary_DataID(DataID, SecondaryDataID));
+    Get_L1 (SecondaryDataID,                                    "Secondary Data ID"); Param_Info1(Ancillary_DataID(DataID, SecondaryDataID));
     if (WithTenBit)
         Skip_L1(                                                "Parity+Unused"); //even:1, odd:2
     Get_L1 (DataCount,                                          "Data count");
     if (WithTenBit)
         Skip_L1(                                                "Parity+Unused"); //even:1, odd:2
 
+    //Test (in some container formats, Cheksum is present sometimes)
+    bool WithChecksum_Temp=WithChecksum;
+    if (!MustSynchronize && !WithChecksum && (size_t)((3+DataCount+1)*(WithTenBit?2:1))==Buffer_Size)
+        WithChecksum_Temp=true;
+
+    //Filling
+    Header_Fill_Code((((int16u)DataID)<<8)|SecondaryDataID, Ztring().From_CC1(DataID)+__T('-')+Ztring().From_CC1(SecondaryDataID));
+    Header_Fill_Size(((MustSynchronize?3:0)+3+DataCount+(WithChecksum_Temp?1:0))*(WithTenBit?2:1));
+}
+
+//---------------------------------------------------------------------------
+void File_Ancillary::Data_Parse()
+{
+    Element_Info1(Ancillary_DataID(DataID, SecondaryDataID));
+
     //Buffer
     int8u* Payload=new int8u[DataCount];
     for(int8u Pos=0; Pos<DataCount; Pos++)
@@ -252,11 +357,20 @@ void File_Ancillary::Read_Buffer_Continue()
         Skip_L1(                                                "Checksum");
     if (WithTenBit)
         Skip_L1(                                                "Parity+Unused"); //even:1, odd:2
-    Element_End();
 
     FILLING_BEGIN();
         switch (DataID)
         {
+            case 0x40 :
+                        switch (SecondaryDataID)
+                        {
+                            case 0x01 : // (from SMPTE ST 305)
+                                        // SDTI
+                                        break;
+                            default   : ;
+                            ;
+                        }
+                        break;
             case 0x41 : // (from SMPTE 2016-3)
                         switch (SecondaryDataID)
                         {
@@ -292,10 +406,55 @@ void File_Ancillary::Read_Buffer_Continue()
                             ;
                         }
                         break;
-            case 0x61 : //Defined data services (from SMPTE 331-1)
+            case 0x60 :
                         switch (SecondaryDataID)
                         {
-                            case 0x01 : //CDP (from SMPTE 331-1)
+                            case 0x60 : // (from SMPTE RP 188 / SMPTE ST 12-2)
+                                        // Time code ATC
+                                        {
+                                        File_Gxf_TimeCode Parser;
+                                        Parser.IsAtc=true;
+                                        Open_Buffer_Init(&Parser);
+                                        Open_Buffer_Continue(&Parser, Payload, (size_t)DataCount);
+
+                                        bool Exists=false;
+                                        if (LineNumber!=(int32u)-1)
+                                            for (size_t Pos=0; Pos<Count_Get(Stream_Other); Pos++)
+                                            {
+                                                if (__T("Line")+Ztring::ToZtring(LineNumber)==Retrieve(Stream_Other, Pos, Other_ID)
+                                                 && ((!LineNumber_IsSecondField && Retrieve(Stream_Other, Pos, "IsSecondField").empty()) || (LineNumber_IsSecondField && !Retrieve(Stream_Other, Pos, "IsSecondField").empty()))
+                                                 && Parser.Settings==Retrieve(Stream_Other, Pos, Other_TimeCode_Settings).To_UTF8())
+                                                    Exists=true;
+                                            }
+                                        else
+                                            for (size_t Pos=0; Pos<Count_Get(Stream_Other); Pos++)
+                                            {
+                                                if (Parser.Settings==Retrieve(Stream_Other, Pos, Other_TimeCode_Settings).To_UTF8())
+                                                    Exists=true;
+                                            }
+                                        if (!Exists)
+                                        {
+                                            Stream_Prepare(Stream_Other);
+                                            Fill(Stream_Other, StreamPos_Last, Other_Type, "Time code");
+                                            Fill(Stream_Other, StreamPos_Last, Other_Format, "SMPTE ATC");
+                                            Fill(Stream_Other, StreamPos_Last, Other_TimeCode_FirstFrame, Parser.TimeCode_FirstFrame);
+                                            Fill(Stream_Other, StreamPos_Last, Other_TimeCode_Settings, Parser.Settings);
+                                            Fill(Stream_Other, StreamPos_Last, Other_MuxingMode, "Ancillary data / SMPTE RP 188");
+                                            if (LineNumber!=(int32u)-1)
+                                                Fill(Stream_Other, StreamPos_Last, Other_ID, __T("Line")+Ztring::ToZtring(LineNumber));
+                                            if (LineNumber_IsSecondField)
+                                                Fill(Stream_Other, StreamPos_Last, "IsSecondField", "Yes");
+                                        }
+                                        }
+                                        break;
+                            default   : ;
+                            ;
+                        }
+                        break;
+            case 0x61 : //Defined data services (from SMPTE 334-1)
+                        switch (SecondaryDataID)
+                        {
+                            case 0x01 : //CDP (from SMPTE 334-1)
                                         #if defined(MEDIAINFO_CDP_YES)
                                         {
                                             if (Cdp_Parser==NULL)
@@ -304,7 +463,7 @@ void File_Ancillary::Read_Buffer_Continue()
                                                 Open_Buffer_Init(Cdp_Parser);
                                             }
                                             Demux(Payload, (size_t)DataCount, ContentType_MainStream);
-                                            if (AspectRatio && FrameRate)
+                                            if (InDecodingOrder || (!HasBFrames && AspectRatio && FrameRate))
                                             {
                                                 if (!Cdp_Parser->Status[IsFinished])
                                                 {
@@ -326,7 +485,7 @@ void File_Ancillary::Read_Buffer_Continue()
                                         }
                                         #endif //MEDIAINFO_CDP_YES
                                         break;
-                            case 0x02 : //CEA-608 (from SMPTE 331-1)
+                            case 0x02 : //CEA-608 (from SMPTE 334-1)
                                         #if defined(MEDIAINFO_EIA608_YES)
                                         if (DataCount==3) //This must be 3-byte data
                                         {
@@ -338,14 +497,69 @@ void File_Ancillary::Read_Buffer_Continue()
                             ;
                         }
                         break;
-            case 0x62 : //Variable-format data services (from SMPTE 331-1)
+            case 0x62 : //Variable-format data services (from SMPTE 334-1)
                         switch (SecondaryDataID)
                         {
-                            case 0x01 : //Program description (from SMPTE 331-1),
+                            case 0x01 : //Program description (from SMPTE 334-1),
                                         break;
-                            case 0x02 : //Data broadcast (from SMPTE 331-1)
+                            case 0x02 : //Data broadcast (from SMPTE 334-1)
                                         break;
-                            case 0x03 : //VBI data (from SMPTE 331-1)
+                            case 0x03 : //VBI data (from SMPTE 334-1)
+                                        break;
+                            default   : ;
+                            ;
+                        }
+                        break;
+            case 0x64 :
+                        switch (SecondaryDataID)
+                        {
+                            case 0x64 : // (from SMPTE RP 196)
+                                        // LTC in HANC space
+                                        {
+                                        bool Exists=false;
+                                        if (LineNumber!=(int32u)-1)
+                                            for (size_t Pos=0; Pos<Count_Get(Stream_Other); Pos++)
+                                            {
+                                                if (__T("Line")+Ztring::ToZtring(LineNumber)==Retrieve(Stream_Other, Pos, Other_ID))
+                                                    Exists=true;
+                                            }
+                                        else
+                                            if (Count_Get(Stream_Other)!=0)
+                                                Exists=true;
+                                        if (!Exists)
+                                        {
+                                            Stream_Prepare(Stream_Other);
+                                            Fill(Stream_Other, StreamPos_Last, Other_Type, "Time code");
+                                            Fill(Stream_Other, StreamPos_Last, Other_Format, "LTC");
+                                            Fill(Stream_Other, StreamPos_Last, Other_MuxingMode, "Ancillary data / SMPTE RP 196");
+                                            if (LineNumber!=(int32u)-1)
+                                                Fill(Stream_Other, StreamPos_Last, Other_ID, __T("Line")+Ztring::ToZtring(LineNumber));
+                                        }
+                                        }
+                                        break;
+                            case 0x7F : // (from SMPTE RP 196)
+                                        // VITC in HANC space
+                                        {
+                                        bool Exists=false;
+                                        if (LineNumber!=(int32u)-1)
+                                            for (size_t Pos=0; Pos<Count_Get(Stream_Other); Pos++)
+                                            {
+                                                if (__T("Line")+Ztring::ToZtring(LineNumber)==Retrieve(Stream_Other, Pos, Other_ID))
+                                                    Exists=true;
+                                            }
+                                        else
+                                            if (Count_Get(Stream_Other)!=0)
+                                                Exists=true;
+                                        if (!Exists)
+                                        {
+                                            Stream_Prepare(Stream_Other);
+                                            Fill(Stream_Other, StreamPos_Last, Other_Type, "Time code");
+                                            Fill(Stream_Other, StreamPos_Last, Other_Format, "VITC");
+                                            Fill(Stream_Other, StreamPos_Last, Other_MuxingMode, "Ancillary data / SMPTE RP 196");
+                                            if (LineNumber!=(int32u)-1)
+                                                Fill(Stream_Other, StreamPos_Last, Other_ID, __T("Line")+Ztring::ToZtring(LineNumber));
+                                        }
+                                        }
                                         break;
                             default   : ;
                             ;
@@ -358,24 +572,6 @@ void File_Ancillary::Read_Buffer_Continue()
     delete[] Payload; //Payload=NULL
 }
 
-//---------------------------------------------------------------------------
-void File_Ancillary::Read_Buffer_Unsynched()
-{
-    #if defined(MEDIAINFO_CDP_YES)
-        for (size_t Pos=0; Pos<Cdp_Data.size(); Pos++)
-            delete Cdp_Data[Pos]; //Cdp_Data[Pos]=NULL;
-        Cdp_Data.clear();
-        if (Cdp_Parser)
-            Cdp_Parser->Open_Buffer_Unsynch();
-    #endif //defined(MEDIAINFO_CDP_YES)
-    #if defined(MEDIAINFO_AFDBARDATA_YES)
-        for (size_t Pos=0; Pos<AfdBarData_Data.size(); Pos++)
-            delete AfdBarData_Data[Pos]; //AfdBarData_Data[Pos]=NULL;
-        AfdBarData_Data.clear();
-    #endif //defined(MEDIAINFO_AFDBARDATA_YES)
-    AspectRatio=0;
-}
-
 //***************************************************************************
 // C++
 //***************************************************************************
