@@ -1,17 +1,17 @@
 // File_DvDif - Info for DV-DIF files
-// Copyright (C) 2002-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2002-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -134,7 +134,7 @@ const int8u  Dv_ChannelsPerBlock[]=
 const char*  Dv_Pair[]=
 {
     "One pair of channels",
-    "Independant channels",
+    "Independent channels",
 };
 
 //---------------------------------------------------------------------------
@@ -250,7 +250,7 @@ File_DvDif::File_DvDif()
     //Temp
     FrameSize_Theory=0;
     Duration=0;
-    TimeCode_First=(int64u)-1;
+    TimeCode_FirstFrame_ms=(int64u)-1;
     Synched_Test_Reset();
     DSF_IsValid=false;
     APT=0xFF; //Impossible
@@ -323,9 +323,10 @@ void File_DvDif::Streams_Fill()
     Stream_Prepare(Stream_Video);
     Fill(Stream_Video, 0, Video_Format, "DV");
     Fill(Stream_Video, 0, Video_Codec, "DV");
-    Fill(Stream_Video, 0, Video_BitRate_Mode, "CBR");
     Fill(Stream_Video, 0, Video_Standard, system?"PAL":"NTSC");
     Fill(Stream_Video, 0, Video_BitDepth, 8);
+    bool IsHd=false;
+    float FrameRate_Multiplicator=1;
     switch (video_source_stype)
     {
         case 0x00 :
@@ -337,14 +338,17 @@ void File_DvDif::Streams_Fill()
         case 0x15 :
                     Fill(Stream_Video, 0, Video_Width, system?1440:1280);
                     Fill(Stream_Video, 0, Video_Height, video_source_stype==0x14?1080:1035);
+                    IsHd=true;
                     break;
         case 0x18 :
                     Fill(Stream_Video, 0, Video_Width, 960);
                     Fill(Stream_Video, 0, Video_Height, 720);
+                    FrameRate_Multiplicator=2;
+                    IsHd=true;
                     break;
         default   : ;
     }
-    Fill(Stream_Video, 0, Video_FrameRate, system?25.000:29.970);
+    Fill(Stream_Video, 0, Video_FrameRate, (system?25.000:29.970)*FrameRate_Multiplicator);
     Fill(Stream_Video, 0, Video_FrameRate_Mode, "CFR");
     if (video_sourcecontrol_IsParsed)
     {
@@ -355,6 +359,10 @@ void File_DvDif::Streams_Fill()
                 case 0x14 :
                 case 0x15 :
                             Fill(Stream_Video, 0, Video_ScanType, "Interlaced");
+                            if (FieldOrder_FF)
+                                Fill(Stream_Video, 0, Video_ScanOrder, FieldOrder_FS?"TFF":"BFF");
+                            else
+                                Fill(Stream_Video, 0, Video_ScanOrder, FieldOrder_FS?"Top field only":"Bottom field only");
                             Fill(Stream_Video, 0, Video_Interlacement, "Interlaced");
                             break;
                 case 0x18 :
@@ -364,6 +372,15 @@ void File_DvDif::Streams_Fill()
                 default   : ;
             }
         }
+        else if (Interlaced)
+        {
+            Fill(Stream_Video, 0, Video_ScanType, "Interlaced");
+            if (FieldOrder_FF)
+                Fill(Stream_Video, 0, Video_ScanOrder, FieldOrder_FS?"BFF":"TFF");
+            else
+                Fill(Stream_Video, 0, Video_ScanOrder, FieldOrder_FS?"Bottom field only":"Top field only");
+            Fill(Stream_Video, 0, Video_Interlacement, "Interlaced");
+        }
         else
         {
             Fill(Stream_Video, 0, Video_ScanType, Interlaced?"Interlaced":"Progressive");
@@ -411,10 +428,9 @@ void File_DvDif::Streams_Fill()
     else //DV 50 Mbps and 100 Mbps
         Fill(Stream_Video, 0, Video_Colorimetry, "4:2:2");
 
-    if (FrameSize_Theory)
+    if (FrameSize_Theory && !IsHd)
     {
-        float64 OverallBitRate=FrameSize_Theory*(DSF?25.000:29.970)*8;
-        if (OverallBitRate>27360000 && OverallBitRate<=30240000) OverallBitRate=DSF?28800000:28771229;
+        float64 OverallBitRate=FrameSize_Theory*(DSF?((float64)25.000):((float64)30000/1001))*8;
         if (FSC_WasSet)
         {
             if (FSP_WasNotSet)
@@ -424,7 +440,10 @@ void File_DvDif::Streams_Fill()
         }
         if (OverallBitRate)
         {
-            Fill(Stream_General, 0, General_OverallBitRate, OverallBitRate, 0);
+            if (IsSub)
+                Fill(Stream_Video, 0, Video_BitRate_Encoded, OverallBitRate, 0);
+            else
+                Fill(Stream_General, 0, General_OverallBitRate, OverallBitRate, 0);
             Fill(Stream_Video, 0, (FSC_WasSet && FSP_WasNotSet)?Video_BitRate_Maximum:Video_BitRate, OverallBitRate*134/150*76/80, 0); //134 Video DIF from 150 DIF, 76 bytes from 80 byte DIF
         }
     }
@@ -433,56 +452,87 @@ void File_DvDif::Streams_Fill()
         for (size_t Pos=0; Pos<Streams_Audio.size(); Pos++)
         {
             Stream_Prepare(Stream_Audio);
-            for (std::map<std::string, Ztring>::iterator Info=Streams_Audio[Pos]->Infos.begin(); Info!=Streams_Audio[Pos]->Infos.end(); Info++)
+            for (std::map<std::string, Ztring>::iterator Info=Streams_Audio[Pos]->Infos.begin(); Info!=Streams_Audio[Pos]->Infos.end(); ++Info)
                 Fill(Stream_Audio, StreamPos_Last, Info->first.c_str(), Info->second, true);
+            Fill(Stream_Audio, StreamPos_Last, Audio_BitRate_Encoded, 0);
         }
 
+    if (Stream_BitRateFromContainer && Retrieve(Stream_Video, 0, Video_BitRate).empty())
+    {
+        if (Stream_BitRateFromContainer>=28800000*0.98 && Stream_BitRateFromContainer<=28800000*1.02)
+        {
+            Fill(Stream_Video, 0, Video_BitRate, ((float64)28800000)*134/150*76/80, 0);
+            Fill(Stream_Video, 0, Video_BitRate_Encoded, 28800000);
+        }
+        if (Stream_BitRateFromContainer>=57600000*0.98 && Stream_BitRateFromContainer<=57600000*1.02)
+        {
+            Fill(Stream_Video, 0, Video_BitRate, ((float64)57600000)*134/150*76/80, 0);
+            Fill(Stream_Video, 0, Video_BitRate_Encoded, 57600000);
+        }
+        if (Stream_BitRateFromContainer>=115200000*0.98 && Stream_BitRateFromContainer<=115200000*1.02)
+        {
+            Fill(Stream_Video, 0, Video_BitRate, ((float64)115200000)*134/150*76/80, 0);
+            Fill(Stream_Video, 0, Video_BitRate_Encoded, 115200000);
+        }
+    }
+
     //Library settings
     Fill(Stream_Video, 0, Video_Encoded_Library_Settings, Encoded_Library_Settings);
 
     //Profile
-    if (FSC_WasSet)
+    if (FSC_WasSet || IsHd)
     {
-        if (FSP_WasNotSet)
+        if (FSP_WasNotSet || IsHd)
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, "DVCPRO HD");
             Fill(Stream_Video, 0, Video_Format_Commercial_IfAny, "DVCPRO HD");
             Fill(Stream_Video, 0, Video_BitDepth, 10, 10, true); //MXF files say that DVCPRO HD are 10 bits, exact?
-            Fill(Stream_Video, 0, Video_BitRate_Mode, "VBR", Unlimited, true, true);
         }
         else
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, "DVCPRO 50");
             Fill(Stream_Video, 0, Video_Format_Commercial_IfAny, "DVCPRO 50");
+            Fill(Stream_Video, 0, Video_BitRate_Mode, "CBR");
         }
     }
-    else if (audio_locked || (Retrieve(Stream_Video, 0, Video_Standard)==_T("PAL") && Retrieve(Stream_Video, 0, Video_Colorimetry)==_T("4:1:1")))
+    else if (audio_locked || (Retrieve(Stream_Video, 0, Video_Standard)==__T("PAL") && Retrieve(Stream_Video, 0, Video_Colorimetry)==__T("4:1:1")))
     {
         Fill(Stream_General, 0, General_Format_Commercial_IfAny, "DVCPRO");
         Fill(Stream_Video, 0, Video_Format_Commercial_IfAny, "DVCPRO");
+        Fill(Stream_Video, 0, Video_BitRate_Mode, "CBR");
     }
+    else
+        Fill(Stream_Video, 0, Video_BitRate_Mode, "CBR");
 
     //Delay
-    if (TimeCode_First!=(int64u)-1)
+    if (TimeCode_FirstFrame_ms!=(int64u)-1)
     {
-        Fill(Stream_Video, 0, Video_Delay, TimeCode_First);
+        Fill(Stream_Video, 0, Video_Delay, TimeCode_FirstFrame_ms);
+        if (TimeCode_FirstFrame.size()==11)
+            Fill(Stream_Video, 0, Video_Delay_DropFrame, TimeCode_FirstFrame[8]==';'?"Yes":"No");
         Fill(Stream_Video, 0, Video_Delay_Source, "Stream");
+        Fill(Stream_Video, 0, Video_TimeCode_FirstFrame, TimeCode_FirstFrame.c_str());
+        Fill(Stream_Video, 0, Video_TimeCode_Source, "Subcode time code");
         for (size_t Pos=0; Pos<Count_Get(Stream_Audio); Pos++)
         {
-            Fill(Stream_Audio, Pos, Audio_Delay, TimeCode_First);
+            Fill(Stream_Audio, Pos, Audio_Delay, TimeCode_FirstFrame_ms);
             Fill(Stream_Audio, Pos, Audio_Delay_Source, "Stream");
         }
     }
 
     #if defined(MEDIAINFO_EIA608_YES)
         for (size_t Pos=0; Pos<CC_Parsers.size(); Pos++)
-            if (CC_Parsers[Pos] && CC_Parsers[Pos]->Status[IsFilled])
+            if (CC_Parsers[Pos] && CC_Parsers[Pos]->Status[IsAccepted])
             {
-                CC_Parsers[Pos]->Finish();
-                Merge(*CC_Parsers[Pos]);
-                Fill(Stream_Text, StreamPos_Last, Text_ID, Pos+1, 10, true);
+                Finish(CC_Parsers[Pos]);
+                for (size_t Pos2=0; Pos2<CC_Parsers[Pos]->Count_Get(Stream_Text); Pos2++)
+                {
+                    Stream_Prepare(Stream_Text);
+                    Merge(*CC_Parsers[Pos], Stream_Text, Pos2, StreamPos_Last);
+                    Fill(Stream_Text, StreamPos_Last, Text_ID, CC_Parsers[Pos]->Retrieve(Stream_Text, Pos2, Text_ID), true);
+                }
             }
-    #endif
+    #endif //defined(MEDIAINFO_EIA608_YES)
 }
 
 //---------------------------------------------------------------------------
@@ -493,7 +543,7 @@ void File_DvDif::Streams_Finish()
         Ztring Recorded_Date(Recorded_Date_Date);
         if (Recorded_Date_Time.size()>4)
         {
-            Recorded_Date+=_T(" ");
+            Recorded_Date+=__T(" ");
             Recorded_Date+=Recorded_Date_Time;
         }
         if (Count_Get(Stream_General)==0)
@@ -631,7 +681,7 @@ bool File_DvDif::Synched_Test()
                         break;
             case 1 : //Subcode
                         if (!((DBN==0 && SCT_Old==0) || (DBN!=0 && SCT_Old==1))
-                         || Number!=DBN && !(Number==2 && DBN==0))
+                         || (Number!=DBN && !(Number==2 && DBN==0)))
                         {
                             if (!Status[IsAccepted])
                                 Trusted_IsNot("Wrong order");
@@ -641,7 +691,7 @@ bool File_DvDif::Synched_Test()
                         break;
             case 2 : //VAUX
                         if (!((DBN==0 && SCT_Old==1) || (DBN!=0 && SCT_Old==2))
-                         || Number!=DBN && !(Number==3 && DBN==0))
+                         || (Number!=DBN && !(Number==3 && DBN==0)))
                         {
                             if (!Status[IsAccepted])
                                 Trusted_IsNot("Wrong order");
@@ -651,7 +701,7 @@ bool File_DvDif::Synched_Test()
                         break;
             case 3 : //Audio
                         if (!((DBN==0 && SCT_Old==2) || (DBN!=0 && SCT_Old==4))
-                         || Number!=DBN && !(Number==9 && DBN==0))
+                         || (Number!=DBN && !(Number==9 && DBN==0)))
                         {
                             if (!Status[IsAccepted])
                                 Trusted_IsNot("Wrong order");
@@ -661,7 +711,7 @@ bool File_DvDif::Synched_Test()
                         break;
             case 4 : //Video
                         if (!(SCT_Old==3 || SCT_Old==4)
-                         || Number!=DBN && !(Number==135 && DBN==0))
+                         || (Number!=DBN && !(Number==135 && DBN==0)))
                         {
                             if (!Status[IsAccepted])
                                 Trusted_IsNot("Wrong order");
@@ -756,7 +806,7 @@ bool File_DvDif::Demux_UnpacketizeContainer_Test()
         if (Demux_Offset+8*80>Buffer_Size && File_Offset+Buffer_Size!=File_Size)
             return false; //No complete frame
         if (Demux_Offset+8*80>Buffer_Size && File_Offset+Buffer_Size==File_Size)
-            Demux_Offset=File_Size-File_Offset; //Using the complete buffer (no next sync)
+            Demux_Offset=(size_t)(File_Size-File_Offset); //Using the complete buffer (no next sync)
 
         Demux_UnpacketizeContainer_Demux();
     }
@@ -773,15 +823,15 @@ bool File_DvDif::Demux_UnpacketizeContainer_Test()
 void File_DvDif::Read_Buffer_Unsynched()
 {
     Synched_Test_Reset();
-        if ((Frame_Count
+        if (!IsSub && File_GoTo!=(int64u)-1 && (Frame_Count
     #if MEDIAINFO_SEEK
           || Duration_Detected
-    #endif MEDIAINFO_SEEK
+    #endif //MEDIAINFO_SEEK
           ) && !FSP_WasNotSet)
     {
         int64u BytesPerFrame=12000*(DSF?12:10);
         if (FSC_WasSet)
-            BytesPerFrame*=2;    
+            BytesPerFrame*=2;
         Frame_Count_NotParsedIncluded=File_GoTo/BytesPerFrame;
         FrameInfo.PTS=FrameInfo.DTS=float64_int64s(Frame_Count_NotParsedIncluded/(DSF?25.000:(30.000*1000/1001))*1000000000);
     }
@@ -789,26 +839,26 @@ void File_DvDif::Read_Buffer_Unsynched()
 
 //---------------------------------------------------------------------------
 #if MEDIAINFO_SEEK
-size_t File_DvDif::Read_Buffer_Seek (size_t Method, int64u Value, int64u ID)
+size_t File_DvDif::Read_Buffer_Seek (size_t Method, int64u Value, int64u /*ID*/)
 {
     //Init
     if (!Duration_Detected)
     {
         MediaInfo_Internal MI;
-        MI.Option(_T("File_KeepInfo"), _T("1"));
-        Ztring ParseSpeed_Save=MI.Option(_T("ParseSpeed_Get"), _T(""));
-        Ztring Demux_Save=MI.Option(_T("Demux_Get"), _T(""));
-        MI.Option(_T("ParseSpeed"), _T("0"));
-        MI.Option(_T("Demux"), Ztring());
+        MI.Option(__T("File_KeepInfo"), __T("1"));
+        Ztring ParseSpeed_Save=MI.Option(__T("ParseSpeed_Get"), __T(""));
+        Ztring Demux_Save=MI.Option(__T("Demux_Get"), __T(""));
+        MI.Option(__T("ParseSpeed"), __T("0"));
+        MI.Option(__T("Demux"), Ztring());
         size_t MiOpenResult=MI.Open(File_Name);
-        MI.Option(_T("ParseSpeed"), ParseSpeed_Save); //This is a global value, need to reset it. TODO: local value
-        MI.Option(_T("Demux"), Demux_Save); //This is a global value, need to reset it. TODO: local value
-        if (!MiOpenResult || MI.Get(Stream_General, 0, General_Format)!=_T("DV"))
+        MI.Option(__T("ParseSpeed"), ParseSpeed_Save); //This is a global value, need to reset it. TODO: local value
+        MI.Option(__T("Demux"), Demux_Save); //This is a global value, need to reset it. TODO: local value
+        if (!MiOpenResult || MI.Get(Stream_General, 0, General_Format)!=__T("DV"))
             return 0;
-        
+
         TotalFrames=Ztring(MI.Get(Stream_Video, 0, Video_FrameCount)).To_int64u();
         int64u VideoBitRate=Ztring(MI.Get(Stream_Video, 0, Video_BitRate)).To_int64u();
-        if (VideoBitRate>=50000000)
+        if (VideoBitRate==0 || VideoBitRate>=50000000)
         {
             FSC_WasSet=true;
             FSP_WasNotSet=true;
@@ -837,7 +887,7 @@ size_t File_DvDif::Read_Buffer_Seek (size_t Method, int64u Value, int64u ID)
         case 2  :   //Timestamp
                     {
                         //We transform TimeStamp to a frame number
-                        Value=((float64)Value)*(DSF?25.000:(30.000*1000/1001))/1000000000;
+                        Value=float64_int64s(((float64)Value)*(DSF?25.000:(30.000*1000/1001))/1000000000);
                     }
                     //No break;
         case 3  :   //FrameNumber
@@ -845,7 +895,7 @@ size_t File_DvDif::Read_Buffer_Seek (size_t Method, int64u Value, int64u ID)
                     {
                         int64u BytesPerFrame=12000*(DSF?12:10);
                         if (FSC_WasSet)
-                            BytesPerFrame*=2;    
+                            BytesPerFrame*=2;
                         GoTo(BytesPerFrame*Value);
                         Open_Buffer_Unsynch();
                         Frame_Count_NotParsedIncluded=Value;
@@ -894,7 +944,7 @@ void File_DvDif::Header_Parse()
     //Parsing
     BS_Begin();
     //0
-    Get_S1 (3, SCT,                                             "SCT - Section Type"); Param_Info(Dv_sct[SCT]);
+    Get_S1 (3, SCT,                                             "SCT - Section Type"); Param_Info1(Dv_sct[SCT]);
     Skip_SB(                                                    "Res - Reserved");
     Skip_S1(4,                                                  "Arb - Arbitrary bits");
     //1
@@ -934,7 +984,7 @@ void File_DvDif::Header_Parse()
         Header_Fill_Size(80);
         return;
     }
-    
+
     //Parsing
     SCT =(Buffer[Buffer_Offset  ]&0xE0)>>5;
     Dseq=(Buffer[Buffer_Offset+1]&0xF0)>>4;
@@ -973,7 +1023,7 @@ void File_DvDif::Data_Parse()
         return;
     }
 
-    Element_Info(DBN);
+    Element_Info1(DBN);
 
     switch (SCT)
     {
@@ -1023,6 +1073,13 @@ void File_DvDif::Header()
     BS_End();
     Skip_XX(72,                                                 "Reserved"); //Should be filled with 0xFF
 
+    if (Config->File_DvDif_IgnoreTransmittingFlags_Get())
+    {
+        TF1=false;
+        TF2=false;
+        TF3=false;
+    }
+
     FILLING_BEGIN();
         DSF_IsValid=true;
         Dseq_Old=DSF?11:9;
@@ -1051,6 +1108,13 @@ void File_DvDif::Header()
     TF2=(Buffer[Buffer_Offset+3]&0x80)?true:false;
     TF3=(Buffer[Buffer_Offset+4]&0x80)?true:false;
 
+    if (Config->File_DvDif_IgnoreTransmittingFlags_Get())
+    {
+        TF1=false;
+        TF2=false;
+        TF3=false;
+    }
+
     FILLING_BEGIN();
         DSF_IsValid=true;
         Dseq_Old=DSF?11:9;
@@ -1096,7 +1160,7 @@ void File_DvDif::Subcode()
 //---------------------------------------------------------------------------
 void File_DvDif::Subcode_Ssyb(int8u syb_num)
 {
-    Element_Begin("ssyb");
+    Element_Begin1("ssyb");
 
     //Parsing
     BS_Begin();
@@ -1121,7 +1185,7 @@ void File_DvDif::Subcode_Ssyb(int8u syb_num)
     //PC0-PC4
     Element();
 
-    Element_End();
+    Element_End0();
 }
 
 //---------------------------------------------------------------------------
@@ -1205,7 +1269,7 @@ void File_DvDif::Video()
 //---------------------------------------------------------------------------
 void File_DvDif::Element()
 {
-    Element_Begin();
+    Element_Begin0();
     int8u PackType;
     if (AuxToAnalyze==0x00)
         Get_B1 (PackType,                                       "Pack Type");
@@ -1215,6 +1279,7 @@ void File_DvDif::Element()
     switch(PackType)
     {
         case 0x13 : timecode(); break;
+        case 0x14 : binary_group(); break;
         case 0x50 : audio_source(); break;
         case 0x51 : audio_sourcecontrol(); break;
         case 0x52 : audio_recdate(); break;
@@ -1231,9 +1296,38 @@ void File_DvDif::Element()
         default   : Element_Name(Ztring().From_Number(PackType, 16));
                     Skip_B4(                                    "Unknown");
     }
-    Element_End();
+    Element_End0();
 }
 
+//---------------------------------------------------------------------------
+void File_DvDif::binary_group()
+{
+    Element_Name("binary_group");
+
+    if (Buffer[Buffer_Offset+(size_t)Element_Offset  ]==0xFF
+     && Buffer[Buffer_Offset+(size_t)Element_Offset+1]==0xFF
+     && Buffer[Buffer_Offset+(size_t)Element_Offset+2]==0xFF
+     && Buffer[Buffer_Offset+(size_t)Element_Offset+3]==0xFF
+    )
+    {
+        Skip_XX(4,                                              "All one");
+        return;
+    }
+
+    //Parsing
+    BS_Begin();
+    Skip_S1(4,                                                  "Binary group 2");
+    Skip_S1(4,                                                  "Binary group 1");
+    Skip_S1(4,                                                  "Binary group 4");
+    Skip_S1(4,                                                  "Binary group 3");
+    Skip_S1(4,                                                  "Binary group 6");
+    Skip_S1(4,                                                  "Binary group 5");
+    Skip_S1(4,                                                  "Binary group 8");
+    Skip_S1(4,                                                  "Binary group 7");
+    BS_End();
+}
+
+
 //---------------------------------------------------------------------------
 void File_DvDif::timecode()
 {
@@ -1249,8 +1343,8 @@ void File_DvDif::timecode()
         return;
     }
 
-    //PArsing
-    int8u Temp;
+    //Parsing
+    int8u Frames_Units, Frames_Tens, Seconds_Units, Seconds_Tens, Minutes_Units, Minutes_Tens, Hours_Units, Hours_Tens;
     int64u MilliSeconds=0;
     int8u  Frames=0;
     bool   DropFrame=false;
@@ -1262,10 +1356,10 @@ void File_DvDif::timecode()
         Skip_SB(                                                "Arbitrary bit");
     else        //525/60
         Get_SB (DropFrame,                                      "DP - Drop frame"); //525/60
-    Get_S1 (2, Temp,                                            "Frames (Tens)");
-    Frames+=Temp*10;
-    Get_S1 (4, Temp,                                            "Frames (Units)");
-    Frames+=Temp;
+    Get_S1 (2, Frames_Tens,                                     "Frames (Tens)");
+    Frames+=Frames_Tens*10;
+    Get_S1 (4, Frames_Units,                                    "Frames (Units)");
+    Frames+=Frames_Units;
 
     if (!DSF_IsValid)
         Skip_SB(                                                "BGF0 or PC");
@@ -1273,10 +1367,10 @@ void File_DvDif::timecode()
         Skip_SB(                                                "BGF0 - Binary group flag");
     else        //525/60
         Skip_SB(                                                "PC - Biphase mark polarity correction"); //0=even; 1=odd
-    Get_S1 (3, Temp,                                            "Seconds (Tens)");
-    MilliSeconds+=Temp*10*1000;
-    Get_S1 (4, Temp,                                            "Seconds (Units)");
-    MilliSeconds+=Temp*1000;
+    Get_S1 (3, Seconds_Tens,                                    "Seconds (Tens)");
+    MilliSeconds+=Seconds_Tens*10*1000;
+    Get_S1 (4, Seconds_Units,                                   "Seconds (Units)");
+    MilliSeconds+=Seconds_Units*1000;
 
     if (!DSF_IsValid)
         Skip_SB(                                                "BGF2 or BGF0");
@@ -1284,10 +1378,10 @@ void File_DvDif::timecode()
         Skip_SB(                                                "BGF2 - Binary group flag");
     else        //525/60
         Skip_SB(                                                "BGF0 - Binary group flag");
-    Get_S1 (3, Temp,                                            "Minutes (Tens)");
-    MilliSeconds+=Temp*10*60*1000;
-    Get_S1 (4, Temp,                                            "Minutes (Units)");
-    MilliSeconds+=Temp*60*1000;
+    Get_S1 (3, Minutes_Tens,                                    "Minutes (Tens)");
+    MilliSeconds+=Minutes_Tens*10*60*1000;
+    Get_S1 (4, Minutes_Units,                                   "Minutes (Units)");
+    MilliSeconds+=Minutes_Units*60*1000;
 
     if (!DSF_IsValid)
         Skip_SB(                                                "PC or BGF1");
@@ -1296,18 +1390,31 @@ void File_DvDif::timecode()
     else        //525/60
         Skip_SB(                                                "BGF1 - Binary group flag");
     Skip_SB(                                                    "BGF2 - Binary group flag");
-    Get_S1 (2, Temp,                                            "Hours (Tens)");
-    MilliSeconds+=Temp*10*60*60*1000;
-    Get_S1 (4, Temp,                                            "Hours (Units)");
-    MilliSeconds+=Temp*60*60*1000;
-    Element_Info(Ztring().Duration_From_Milliseconds(MilliSeconds+((DSF_IsValid && Frames!=45)?((int64u)(Frames/(DSF?25.000:29.970)*1000)):0)));
+    Get_S1 (2, Hours_Tens,                                      "Hours (Tens)");
+    MilliSeconds+=Hours_Tens*10*60*60*1000;
+    Get_S1 (4, Hours_Units,                                     "Hours (Units)");
+    MilliSeconds+=Hours_Units*60*60*1000;
+    Element_Info1(Ztring().Duration_From_Milliseconds(MilliSeconds+((DSF_IsValid && Frames!=45)?((int64u)(Frames/(DSF?25.000:29.970)*1000)):0)));
     BS_End();
 
-    if (TimeCode_First==(int64u)-1 && MilliSeconds!=167185000) //if all bits are set to 1, this is not a valid timestamp
+    if (TimeCode_FirstFrame_ms==(int64u)-1 && MilliSeconds!=167185000) //if all bits are set to 1, this is not a valid timestamp
     {
-        TimeCode_First=MilliSeconds;
+        TimeCode_FirstFrame_ms=MilliSeconds;
         if (DSF_IsValid && Frames!=45) //all bits are set to 1
-            TimeCode_First+=(int64u)(Frames/(DSF?25.000:29.970)*1000);
+
+        TimeCode_FirstFrame_ms+=(int64u)(Frames/(DSF?25.000:29.970)*1000);
+
+        TimeCode_FirstFrame+=('0'+Hours_Tens);
+        TimeCode_FirstFrame+=('0'+Hours_Units);
+        TimeCode_FirstFrame+=':';
+        TimeCode_FirstFrame+=('0'+Minutes_Tens);
+        TimeCode_FirstFrame+=('0'+Minutes_Units);
+        TimeCode_FirstFrame+=':';
+        TimeCode_FirstFrame+=('0'+Seconds_Tens);
+        TimeCode_FirstFrame+=('0'+Seconds_Units);
+        TimeCode_FirstFrame+=DropFrame?';':':';
+        TimeCode_FirstFrame+=('0'+Frames_Tens);
+        TimeCode_FirstFrame+=('0'+Frames_Units);
     }
 }
 
@@ -1330,24 +1437,24 @@ void File_DvDif::audio_source()
     Skip_S1(6,                                                  "AF - Samples in this frame");
 
     //PC2
-    Info_S1(1, StereoMode,                                      "SM - Stereo mode"); Param_Info(Dv_StereoMode[StereoMode]);
-    Info_S1(2, ChannelsPerBlock,                                "CHN - Channels per block"); Param_Info(Dv_ChannelsPerBlock[ChannelsPerBlock]);
-    Info_S1(1, Pair,                                            "PA - Pair"); Param_Info(Dv_Pair[Pair]);
+    Info_S1(1, StereoMode,                                      "SM - Stereo mode"); Param_Info1(Dv_StereoMode[StereoMode]);
+    Info_S1(2, ChannelsPerBlock,                                "CHN - Channels per block"); Param_Info1(Dv_ChannelsPerBlock[ChannelsPerBlock]);
+    Info_S1(1, Pair,                                            "PA - Pair"); Param_Info1(Dv_Pair[Pair]);
     Skip_S1(4,                                                  "AM - Audio mode");
 
     Skip_SB(                                                    "Reserved");
     Skip_SB(                                                    "ML - Multi-language");
     Skip_SB(                                                    "50/60");
-    Get_S1 (5, audio_source_stype,                              "STYPE - audio blocks per video frame"); Param_Info(audio_source_stype==0?"2 channels":(audio_source_stype==2?"4 channels":"Unknown")); //0=25 Mbps, 2=50 Mbps
+    Get_S1 (5, audio_source_stype,                              "STYPE - audio blocks per video frame"); Param_Info1(audio_source_stype==0?"2 channels":(audio_source_stype==2?"4 channels":"Unknown")); //0=25 Mbps, 2=50 Mbps
 
     Skip_SB(                                                    "EF - Emphasis off");
     Skip_SB(                                                    "TC - Time constant of emphasis");
-    Get_S1 (3, SamplingRate,                                    "SMP - Sampling rate"); Param_Info(Dv_Audio_SamplingRate[SamplingRate]);
-    Get_S1 (3, Resolution,                                      "QU - Resolution"); Param_Info(Dv_Audio_BitDepth[Resolution]);
+    Get_S1 (3, SamplingRate,                                    "SMP - Sampling rate"); Param_Info1(Dv_Audio_SamplingRate[SamplingRate]);
+    Get_S1 (3, Resolution,                                      "QU - Resolution"); Param_Info1(Dv_Audio_BitDepth[Resolution]);
     BS_End();
 
     FILLING_BEGIN();
-        if (!IgnoreAudio && (Frame_Count==1 || AuxToAnalyze)) //Only the first time
+        if (!IgnoreAudio && Streams_Audio.empty() && Dv_Audio_SamplingRate[SamplingRate] && Dv_Audio_BitDepth[Resolution])
         {
             //Calculating the count of audio
             size_t Audio_Count=1;
@@ -1364,9 +1471,9 @@ void File_DvDif::audio_source()
                 if (Streams_Audio[Pos]==NULL)
                     Streams_Audio[Pos]=new stream;
                 Streams_Audio[Pos]->Infos["ID"].From_Number(Pos);
-                Streams_Audio[Pos]->Infos["Format"]=_T("PCM");
-                Streams_Audio[Pos]->Infos["Codec"]=_T("PCM");
-                Streams_Audio[Pos]->Infos["BitRate_Mode"]=_T("CBR");
+                Streams_Audio[Pos]->Infos["Format"]=__T("PCM");
+                Streams_Audio[Pos]->Infos["Codec"]=__T("PCM");
+                Streams_Audio[Pos]->Infos["BitRate_Mode"]=__T("CBR");
                 Streams_Audio[Pos]->Infos["Channel(s)"].From_Number(audio_source_stype==3?1:2);
                 Streams_Audio[Pos]->Infos["SamplingRate"].From_Number(Dv_Audio_SamplingRate[SamplingRate]);
                 Streams_Audio[Pos]->Infos["BitDepth"].From_Number(Dv_Audio_BitDepth[Resolution]);
@@ -1390,10 +1497,10 @@ void File_DvDif::audio_sourcecontrol()
     BS_Begin();
 
     //PC1
-    Info_S1(2, CopyGenerationManagementSystem,                  "CGMS - Copy generation management system"); Param_Info(Dv_CopyGenerationManagementSystem[CopyGenerationManagementSystem]);
-    Info_S1(2, InputType,                                       "ISR - Input type"); Param_Info(Dv_InputType[InputType]);
-    Info_S1(2, CompressionTimes,                                "CMP - Compression times"); Param_Info(Dv_CompressionTimes[CompressionTimes]);
-    Info_S1(2, Emphasis,                                        "EFC - Emphasis"); Param_Info(Dv_Emphasis[Emphasis]);
+    Info_S1(2, CopyGenerationManagementSystem,                  "CGMS - Copy generation management system"); Param_Info1(Dv_CopyGenerationManagementSystem[CopyGenerationManagementSystem]);
+    Info_S1(2, InputType,                                       "ISR - Input type"); Param_Info1(Dv_InputType[InputType]);
+    Info_S1(2, CompressionTimes,                                "CMP - Compression times"); Param_Info1(Dv_CompressionTimes[CompressionTimes]);
+    Info_S1(2, Emphasis,                                        "EFC - Emphasis"); Param_Info1(Dv_Emphasis[Emphasis]);
 
     //PC2
     Skip_SB(                                                    "REC S Non-recording start point");
@@ -1503,7 +1610,7 @@ void File_DvDif::video_sourcecontrol()
 
     BS_Begin();
     //PC1
-    Info_S1(2, CopyGenerationManagementSystem,                  "CGMS - Copy generation management system"); Param_Info(Dv_CopyGenerationManagementSystem[CopyGenerationManagementSystem]);
+    Info_S1(2, CopyGenerationManagementSystem,                  "CGMS - Copy generation management system"); Param_Info1(Dv_CopyGenerationManagementSystem[CopyGenerationManagementSystem]);
     Skip_S1(2,                                                  "ISR");
     Skip_S1(2,                                                  "CMP");
     Skip_S2(2,                                                  "SS");
@@ -1513,11 +1620,11 @@ void File_DvDif::video_sourcecontrol()
     Skip_SB(                                                    "Reserved");
     Skip_S1(2,                                                  "REC M");
     Skip_SB(                                                    "Reserved");
-    Get_S1 (3, aspect,                                          "DISP - Aspect ratio"); Param_Info(Dv_Disp[aspect]);
+    Get_S1 (3, aspect,                                          "DISP - Aspect ratio"); Param_Info1(Dv_Disp[aspect]);
 
     //PC3
-    Skip_SB(                                                    "FF - Frame/Field"); //1=Frame, 0=Field
-    Skip_SB(                                                    "FS - First/second field"); //0=Field 2, 1=Field 1, if FF=0 x is output twice, if FF=1, Field x fisrst, other second
+    Get_SB (   FieldOrder_FF,                                   "FF - Frame/Field"); //1=Frame, 0=Field
+    Get_SB (   FieldOrder_FS,                                   "FS - First/second field"); //0=Field 2, 1=Field 1, if FF=0 x is output twice, if FF=1, Field x fisrst, other second
     Skip_SB(                                                    "FC - Frame Change"); //0=Same picture as before
     Get_SB (   Interlaced,                                      "IL - Interlaced"); //1=Interlaced
     Skip_SB(                                                    "SF");
@@ -1576,8 +1683,11 @@ void File_DvDif::closed_captions()
         if (CC_Parsers.empty())
         {
             CC_Parsers.resize(2);
-            for (size_t Pos=0; Pos<2; Pos++)
+            for (int8u Pos=0; Pos<2; Pos++)
+            {
                 CC_Parsers[Pos]=new File_Eia608();
+                ((File_Eia608*)CC_Parsers[Pos])->cc_type=Pos;
+            }
             Frame_Count_Valid*=10; //More frames
         }
         if (Dseq==0) //CC are duplicated for each DIF sequence!
@@ -1601,23 +1711,24 @@ void File_DvDif::consumer_camera_1()
 
     //Parsing
     BS_Begin();
+    int8u ae_mode, wb_mode, white_balance, fcm;
     Mark_1_NoTrustError();
     Mark_1_NoTrustError();
     Skip_S1(6,                                                  "iris");
-    Info_S1(4, ae_mode,                                         "ae mode"); Param_Info(Dv_consumer_camera_1_ae_mode[ae_mode]);
+    Get_S1 (4, ae_mode,                                         "ae mode"); Param_Info1(Dv_consumer_camera_1_ae_mode[ae_mode]);
     Skip_S1(4,                                                  "agc(Automatic Gain Control)");
-    Info_S1(3, wb_mode,                                         "wb mode (white balance mode)"); Param_Info(Dv_consumer_camera_1_wb_mode[wb_mode]);
-    Info_S1(5, white_balance,                                   "white balance"); Param_Info(Dv_consumer_camera_1_white_balance(white_balance));
-    Info_S1(1, fcm,                                             "fcm (Focus mode)"); Param_Info(Dv_consumer_camera_1_fcm[fcm]);
+    Get_S1 (3, wb_mode,                                         "wb mode (white balance mode)"); Param_Info1(Dv_consumer_camera_1_wb_mode[wb_mode]);
+    Get_S1 (5, white_balance,                                   "white balance"); Param_Info1(Dv_consumer_camera_1_white_balance(white_balance));
+    Get_S1 (1, fcm,                                             "fcm (Focus mode)"); Param_Info1(Dv_consumer_camera_1_fcm[fcm]);
     Skip_S1(7,                                                  "focus (focal point)");
     BS_End();
 
     if (Encoded_Library_Settings.empty())
     {
-        if (ae_mode<0x0F) Encoded_Library_Settings+=_T("ae mode=")+Ztring(Dv_consumer_camera_1_ae_mode[ae_mode])+_T(" / ");
-        if (wb_mode<0x08) Encoded_Library_Settings+=_T("wb mode=")+Ztring(Dv_consumer_camera_1_wb_mode[wb_mode])+_T(" / ");
-        if (wb_mode<0x1F) Encoded_Library_Settings+=_T("white balance=")+Ztring(Dv_consumer_camera_1_white_balance(white_balance))+_T(" / ");
-                          Encoded_Library_Settings+=_T("fcm=")+Ztring(Dv_consumer_camera_1_fcm[fcm]);
+        if (ae_mode<0x0F) Encoded_Library_Settings+=__T("ae mode=")+Ztring(Dv_consumer_camera_1_ae_mode[ae_mode])+__T(" / ");
+        if (wb_mode<0x08) Encoded_Library_Settings+=__T("wb mode=")+Ztring(Dv_consumer_camera_1_wb_mode[wb_mode])+__T(" / ");
+        if (wb_mode<0x1F) Encoded_Library_Settings+=__T("white balance=")+Ztring(Dv_consumer_camera_1_white_balance(white_balance))+__T(" / ");
+                          Encoded_Library_Settings+=__T("fcm=")+Ztring(Dv_consumer_camera_1_fcm[fcm]);
     }
 }
 
@@ -1638,7 +1749,7 @@ void File_DvDif::consumer_camera_2()
     Skip_S1(8,                                                  "focal length");
     Skip_S1(1,                                                  "zen");
     Info_S1(3, zoom_U,                                          "units of e-zoom");
-    Info_S1(4, zoom_D,                                          "1/10 of e-zoom"); if (zoom_D!=0xF) Param_Info(_T("zoom=")+Ztring().From_Number(zoom_U+((float32)zoom_U)/10, 2));
+    Info_S1(4, zoom_D,                                          "1/10 of e-zoom"); /*if (zoom_D!=0xF)*/ Param_Info1(__T("zoom=")+Ztring().From_Number(zoom_U+((float32)zoom_U)/10, 2));
     BS_End();
 }
 
@@ -1673,7 +1784,7 @@ Ztring File_DvDif::recdate()
     Get_S1 (4, Temp,                                            "Year (Units)");
     Year+=Temp;
     Year+=Year<25?2000:1900;
-    Element_Info(Ztring::ToZtring(Year)+_T("-")+Ztring::ToZtring(Month)+_T("-")+Ztring::ToZtring(Day));
+    Element_Info1(Ztring::ToZtring(Year)+__T("-")+Ztring::ToZtring(Month)+__T("-")+Ztring::ToZtring(Day));
 
     BS_End();
 
@@ -1681,13 +1792,13 @@ Ztring File_DvDif::recdate()
         return Ztring(); //If all bits are set to 1, this is invalid
     Ztring MonthString;
     if (Month<10)
-        MonthString=_T("0");
+        MonthString=__T("0");
     MonthString+=Ztring::ToZtring(Month);
     Ztring DayString;
     if (Day<10)
-        DayString=_T("0");
+        DayString=__T("0");
     DayString+=Ztring::ToZtring(Day);
-    return Ztring::ToZtring(Year)+_T("-")+MonthString+_T("-")+DayString;
+    return Ztring::ToZtring(Year)+__T("-")+MonthString+__T("-")+DayString;
 }
 
 //---------------------------------------------------------------------------
@@ -1738,7 +1849,7 @@ Ztring File_DvDif::rectime()
     Time+=Temp*10*60*60*1000;
     Get_S1 (4, Temp,                                            "Hours (Units)");
     Time+=Temp*60*60*1000;
-    Element_Info(Ztring().Duration_From_Milliseconds(Time));
+    Element_Info1(Ztring().Duration_From_Milliseconds(Time));
 
     BS_End();
 
