@@ -1,22 +1,22 @@
 // File_Mk - Info for Ibi files
-// Copyright (C) 2011-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2011-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
- 
+
 //---------------------------------------------------------------------------
 // Pre-compilation
 #include "MediaInfo/PreComp.h"
@@ -30,7 +30,7 @@
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
-#if defined(MEDIAINFO_IBI_YES)
+#if MEDIAINFO_IBI
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
@@ -75,7 +75,7 @@ File_Ibi::~File_Ibi()
 //***************************************************************************
 
 //---------------------------------------------------------------------------
-const Ztring &File_Ibi::Get (stream_t StreamKind, size_t StreamNumber, const Ztring &Parameter, info_t KindOfInfo, info_t KindOfSearch)
+const Ztring &File_Ibi::Get (stream_t /*StreamKind*/, size_t /*StreamNumber*/, const Ztring &Parameter, info_t /*KindOfInfo*/, info_t /*KindOfSearch*/)
 {
     ibi::streams::iterator IbiStream_Temp=Ibi->Streams.begin(); //TODO: management of multiple streams
     if (IbiStream_Temp!=Ibi->Streams.end() && !IbiStream_Temp->second->Infos.empty())
@@ -85,10 +85,10 @@ const Ztring &File_Ibi::Get (stream_t StreamKind, size_t StreamNumber, const Ztr
         for (size_t Pos=0; Pos<IbiStream_Temp->second->Infos.size()-1; Pos++)
             if (IbiStream_Temp->second->Infos[Pos].FrameNumber==FrameNumber || IbiStream_Temp->second->Infos[Pos+1].FrameNumber>FrameNumber)
             {
-                Get_Temp=_T("StreamOffset=")+Ztring::ToZtring(IbiStream_Temp->second->Infos[Pos].StreamOffset)
-                       + _T(" / FrameNumber=")+Ztring::ToZtring(IbiStream_Temp->second->Infos[Pos].FrameNumber)
-                       + _T(" / Dts=")+Ztring::ToZtring(IbiStream_Temp->second->Infos[Pos].Dts);
-                return Get_Temp; 
+                Get_Temp=__T("StreamOffset=")+Ztring::ToZtring(IbiStream_Temp->second->Infos[Pos].StreamOffset)
+                       + __T(" / FrameNumber=")+Ztring::ToZtring(IbiStream_Temp->second->Infos[Pos].FrameNumber)
+                       + __T(" / Dts=")+Ztring::ToZtring(IbiStream_Temp->second->Infos[Pos].Dts);
+                return Get_Temp;
             }
     }
 
@@ -118,7 +118,7 @@ void File_Ibi::Streams_Finish()
 {
     Config->File_KeepInfo_Set(true); //In order to let Get() available
 
-    for (ibi::streams::iterator IbiStream_Temp=Ibi->Streams.begin(); IbiStream_Temp!=Ibi->Streams.end(); IbiStream_Temp++)
+    for (ibi::streams::iterator IbiStream_Temp=Ibi->Streams.begin(); IbiStream_Temp!=Ibi->Streams.end(); ++IbiStream_Temp)
     {
         Stream_Prepare(Stream_Video);
         Fill(Stream_Video, StreamPos_Last, General_ID, IbiStream_Temp->first);
@@ -317,20 +317,18 @@ void File_Ibi::Ebml_DocType()
 
     //Parsing
     Ztring Data;
-    Get_Local(Element_Size, Data,                               "Data"); Element_Info(Data);
+    Get_Local(Element_Size, Data,                               "Data"); Element_Info1(Data);
 
     //Filling
     FILLING_BEGIN();
 
-        if (Data==_T("MediaInfo Index"))
+        if (Data==__T("MediaInfo Index"))
             Accept("Ibi");
         else
         {
             Reject("Ibi");
             return;
         }
-
-        Buffer_MaximumSize=8*1024*1024;
     FILLING_END();
 }
 
@@ -391,15 +389,15 @@ void File_Ibi::Stream_ByteOffset()
         int64u Item;
         Get_EB (Item,                                           "Item");
         Offset+=Item;
-        Param_Info(Pos);
-        Param_Info(Ztring::ToZtring(Offset)+_T(" (0x")+Ztring::ToZtring(Offset, 16)+_T(')'));
+        Param_Info1(Pos);
+        Param_Info1(Ztring::ToZtring(Offset)+__T(" (0x")+Ztring::ToZtring(Offset, 16)+__T(')'));
 
         FILLING_BEGIN();
             if (Ibi)
             {
                 //Filling information for ID after data
                 if (Ibi->Streams[ID_Current]==NULL)
-                    Ibi->Streams[ID_Current]=new ibi::stream();    
+                    Ibi->Streams[ID_Current]=new ibi::stream();
                 if (Pos>=Ibi->Streams[ID_Current]->Infos.size())
                 {
                     Ibi->Streams[ID_Current]->Infos.push_back(ibi::stream::info());
@@ -424,15 +422,15 @@ void File_Ibi::Stream_FrameNumber()
         int64u Item;
         Get_EB (Item,                                           "Item");
         Offset+=Item;
-        Param_Info(Pos);
-        Param_Info(Ztring::ToZtring(Offset)+_T(" (0x")+Ztring::ToZtring(Offset, 16)+_T(')'));
+        Param_Info1(Pos);
+        Param_Info1(Ztring::ToZtring(Offset)+__T(" (0x")+Ztring::ToZtring(Offset, 16)+__T(')'));
 
         FILLING_BEGIN();
             if (Ibi)
             {
                 //Filling information for ID after data
                 if (Ibi->Streams[ID_Current]==NULL)
-                    Ibi->Streams[ID_Current]=new ibi::stream();    
+                    Ibi->Streams[ID_Current]=new ibi::stream();
                 if (Pos>=Ibi->Streams[ID_Current]->Infos.size())
                 {
                     Ibi->Streams[ID_Current]->Infos.push_back(ibi::stream::info());
@@ -475,15 +473,15 @@ void File_Ibi::Stream_Dts()
         int64u Item;
         Get_EB (Item,                                           "Item");
         Offset+=Item;
-        Param_Info(Pos);
-        Param_Info(Ztring::ToZtring(Offset)+_T(" (0x")+Ztring::ToZtring(Offset, 16)+_T(')'));
+        Param_Info1(Pos);
+        Param_Info1(Ztring::ToZtring(Offset)+__T(" (0x")+Ztring::ToZtring(Offset, 16)+__T(')'));
 
         FILLING_BEGIN();
             if (Ibi)
             {
                 //Filling information for ID after data
                 if (Ibi->Streams[ID_Current]==NULL)
-                    Ibi->Streams[ID_Current]=new ibi::stream();    
+                    Ibi->Streams[ID_Current]=new ibi::stream();
                 if (Pos>=Ibi->Streams[ID_Current]->Infos.size())
                 {
                     Ibi->Streams[ID_Current]->Infos.push_back(ibi::stream::info());
@@ -538,7 +536,7 @@ void File_Ibi::CompressedIndex()
     while(Element_Level)
     {
         Element_Sizes_Sav.push_back(Element_TotalSize_Get());
-        Element_End();
+        Element_End0();
     }
 
     //Configuring file size
@@ -556,7 +554,7 @@ void File_Ibi::CompressedIndex()
     //Resetting file size
     File_Size=File_Size_Sav;
     while(Element_Level)
-        Element_End();
+        Element_End0();
     Element_Level++;
     Header_Fill_Size(File_Size);
     Element_Level--;
@@ -564,10 +562,10 @@ void File_Ibi::CompressedIndex()
     //Configuring level
     while(Element_Level<Element_Level_Sav)
     {
-        Element_Begin();
-        Element_Begin();
+        Element_Begin0();
+        Element_Begin0();
         Header_Fill_Size(Element_Sizes_Sav[0]);
-        Element_End();
+        Element_End0();
     }
 
     //Resetting buffer
@@ -590,47 +588,47 @@ void File_Ibi::UInteger_Info()
     {
         case 1 :
                 {
-                    Info_B1(Data,                               "Data"); Element_Info(Data);
+                    Info_B1(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 2 :
                 {
-                    Info_B2(Data,                               "Data"); Element_Info(Data);
+                    Info_B2(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 3 :
                 {
-                    Info_B3(Data,                               "Data"); Element_Info(Data);
+                    Info_B3(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 4 :
                 {
-                    Info_B4(Data,                               "Data"); Element_Info(Data);
+                    Info_B4(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 5 :
                 {
-                    Info_B5(Data,                               "Data"); Element_Info(Data);
+                    Info_B5(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 6 :
                 {
-                    Info_B6(Data,                               "Data"); Element_Info(Data);
+                    Info_B6(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 7 :
                 {
-                    Info_B7(Data,                               "Data"); Element_Info(Data);
+                    Info_B7(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 8 :
                 {
-                    Info_B8(Data,                               "Data"); Element_Info(Data);
+                    Info_B8(Data,                               "Data"); Element_Info1(Data);
                     return;
                 }
         case 16:
                 {
-                    Info_B16(Data,                              "Data"); Element_Info(Data);
+                    Info_B16(Data,                              "Data"); Element_Info1(Data);
                     return;
                 }
         default : Skip_XX(Element_Size,                         "Data");
@@ -645,49 +643,49 @@ int64u File_Ibi::UInteger_Get()
         case 1 :
                 {
                     int8u Data;
-                    Get_B1 (Data,                               "Data"); Element_Info(Data);
+                    Get_B1 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 2 :
                 {
                     int16u Data;
-                    Get_B2 (Data,                               "Data"); Element_Info(Data);
+                    Get_B2 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 3 :
                 {
                     int32u Data;
-                    Get_B3 (Data,                               "Data"); Element_Info(Data);
+                    Get_B3 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 4 :
                 {
                     int32u Data;
-                    Get_B4 (Data,                               "Data"); Element_Info(Data);
+                    Get_B4 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 5 :
                 {
                     int64u Data;
-                    Get_B5 (Data,                               "Data"); Element_Info(Data);
+                    Get_B5 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 6 :
                 {
                     int64u Data;
-                    Get_B6 (Data,                               "Data"); Element_Info(Data);
+                    Get_B6 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 7 :
                 {
                     int64u Data;
-                    Get_B7 (Data,                               "Data"); Element_Info(Data);
+                    Get_B7 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 8 :
                 {
                     int64u Data;
-                    Get_B8 (Data,                               "Data"); Element_Info(Data);
+                    Get_B8 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         default :   Skip_XX(Element_Size,                       "Data");
@@ -703,55 +701,55 @@ int128u File_Ibi::UInteger16_Get()
         case 1 :
                 {
                     int8u Data;
-                    Get_B1 (Data,                               "Data"); Element_Info(Data);
+                    Get_B1 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 2 :
                 {
                     int16u Data;
-                    Get_B2 (Data,                               "Data"); Element_Info(Data);
+                    Get_B2 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 3 :
                 {
                     int32u Data;
-                    Get_B3 (Data,                               "Data"); Element_Info(Data);
+                    Get_B3 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 4 :
                 {
                     int32u Data;
-                    Get_B4 (Data,                               "Data"); Element_Info(Data);
+                    Get_B4 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 5 :
                 {
                     int64u Data;
-                    Get_B5 (Data,                               "Data"); Element_Info(Data);
+                    Get_B5 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 6 :
                 {
                     int64u Data;
-                    Get_B6 (Data,                               "Data"); Element_Info(Data);
+                    Get_B6 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 7 :
                 {
                     int64u Data;
-                    Get_B7 (Data,                               "Data"); Element_Info(Data);
+                    Get_B7 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 8 :
                 {
                     int64u Data;
-                    Get_B8 (Data,                               "Data"); Element_Info(Data);
+                    Get_B8 (Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         case 16:
                 {
                     int128u Data;
-                    Get_B16(Data,                               "Data"); Element_Info(Data);
+                    Get_B16(Data,                               "Data"); Element_Info1(Data);
                     return Data;
                 }
         default :   Skip_XX(Element_Size,                       "Data");
