@@ -1,17 +1,17 @@
 // File_Mpeg4 - Info for MPEG-4 files
-// Copyright (C) 2005-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2005-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -77,7 +77,7 @@
     #include "MediaInfo/Audio/File_Mpega.h"
 #endif
 #if MEDIAINFO_DEMUX
-    #include "ZenLib/Base64/base64.h"
+    #include "base64.h"
 #endif //MEDIAINFO_DEMUX
 //---------------------------------------------------------------------------
 
@@ -182,12 +182,9 @@ const char* Mpeg4_Descriptors_StreamType(int8u ID)
 }
 
 //---------------------------------------------------------------------------
-const char* Mpeg4_Descriptors_ODProfileLevelIndication(int8u ID)
+const char* Mpeg4_Descriptors_ODProfileLevelIndication(int8u /*ID*/)
 {
-    switch (ID)
-    {
-        default   : return "";
-    }
+    return "";
 }
 
 //---------------------------------------------------------------------------
@@ -295,6 +292,14 @@ const char* Mpeg4_Descriptors_GraphicsProfileLevelIndication(int8u ID)
 File_Mpeg4_Descriptors::File_Mpeg4_Descriptors()
 :File__Analyze()
 {
+    //Configuration
+    ParserName=__T("MPEG-4 Descriptor");
+    #if MEDIAINFO_EVENTS
+        ParserIDs[0]=MediaInfo_Parser_Mpeg4_Desc;
+        StreamIDs_Width[0]=0;
+    #endif //MEDIAINFO_EVENTS
+    IsRawStream=true;
+
     //In
     KindOfStream=Stream_Max;
     Parser_DoNotFreeIt=false;
@@ -438,11 +443,11 @@ void File_Mpeg4_Descriptors::Descriptor_01()
     }
     if (Element_Code==0x02 || Element_Code==0x10)
     {
-        Info_B1(ODProfileLevel,                                 "ODProfileLevelIndication"); Param_Info(Mpeg4_Descriptors_ODProfileLevelIndication(ODProfileLevel));
-        Info_B1(SceneProfileLevel,                              "sceneProfileLevelIndication"); Param_Info(Mpeg4_Descriptors_SceneProfileLevelIndication(SceneProfileLevel));
-        Info_B1(AudioProfileLevel,                              "audioProfileLevelIndication"); Param_Info(Mpeg4_Descriptors_AudioProfileLevelIndication(AudioProfileLevel));
-        Info_B1(VisualProfileLevel,                             "visualProfileLevelIndication"); Param_Info(Mpeg4v_Profile_Level(VisualProfileLevel));
-        Info_B1(GraphicsProfileLevel,                           "graphicsProfileLevelIndication"); Param_Info(Mpeg4_Descriptors_GraphicsProfileLevelIndication(GraphicsProfileLevel));
+        Info_B1(ODProfileLevel,                                 "ODProfileLevelIndication"); Param_Info1(Mpeg4_Descriptors_ODProfileLevelIndication(ODProfileLevel));
+        Info_B1(SceneProfileLevel,                              "sceneProfileLevelIndication"); Param_Info1(Mpeg4_Descriptors_SceneProfileLevelIndication(SceneProfileLevel));
+        Info_B1(AudioProfileLevel,                              "audioProfileLevelIndication"); Param_Info1(Mpeg4_Descriptors_AudioProfileLevelIndication(AudioProfileLevel));
+        Info_B1(VisualProfileLevel,                             "visualProfileLevelIndication"); Param_Info1(Mpeg4v_Profile_Level(VisualProfileLevel));
+        Info_B1(GraphicsProfileLevel,                           "graphicsProfileLevelIndication"); Param_Info1(Mpeg4_Descriptors_GraphicsProfileLevelIndication(GraphicsProfileLevel));
     }
 
     FILLING_BEGIN();
@@ -484,9 +489,9 @@ void File_Mpeg4_Descriptors::Descriptor_04()
     //Parsing
     int32u bufferSizeDB, MaxBitrate, AvgBitrate;
     int8u  streamType;
-    Get_B1 (ObjectTypeId,                                       "objectTypeIndication"); Param_Info(Mpeg4_Descriptors_ObjectTypeIndication(ObjectTypeId));
+    Get_B1 (ObjectTypeId,                                       "objectTypeIndication"); Param_Info1(Mpeg4_Descriptors_ObjectTypeIndication(ObjectTypeId));
     BS_Begin();
-    Get_S1 (6, streamType,                                      "streamType"); Param_Info(Mpeg4_Descriptors_StreamType(streamType));
+    Get_S1 (6, streamType,                                      "streamType"); Param_Info1(Mpeg4_Descriptors_StreamType(streamType));
     Skip_SB(                                                    "upStream");
     Skip_SB(                                                    "reserved");
     BS_End();
@@ -615,7 +620,7 @@ void File_Mpeg4_Descriptors::Descriptor_04()
             case 0xA5 : Fill(Stream_Audio   , StreamPos_Last, Audio_Codec, "AC3", Error, false, true); break;
             case 0xA6 : Fill(Stream_Audio   , StreamPos_Last, Audio_Codec, "AC3+", Error, false, true); break;
             case 0xA9 : Fill(Stream_Audio   , StreamPos_Last, Audio_Codec, "DTS", Error, false, true); break;
-            case 0xAA : 
+            case 0xAA :
             case 0xAB : Fill(Stream_Audio   , StreamPos_Last, Audio_Codec, "DTS-HD", Error, false, true); break;
             case 0xAC : Fill(Stream_Audio   , StreamPos_Last, Audio_Codec, "DTS Express", Error, false, true); break;
             case 0xD1 : Fill(Stream_Audio   , StreamPos_Last, Audio_Codec, "EVRC", Error, false, true); break;
@@ -709,7 +714,7 @@ void File_Mpeg4_Descriptors::Descriptor_04()
                         break;
             case 0xA3 : //VC-1
                         #if defined(MEDIAINFO_VC1_YES)
-                            Parser=new File_Vc1; 
+                            Parser=new File_Vc1;
                         #endif
                         break;
             case 0xA4 : //Dirac
@@ -744,6 +749,7 @@ void File_Mpeg4_Descriptors::Descriptor_04()
             default: ;
         }
 
+        Element_Code=(int64u)-1;
         Open_Buffer_Init(Parser);
 
         Element_ThisIsAList();
@@ -771,6 +777,8 @@ void File_Mpeg4_Descriptors::Descriptor_05()
                                 break;
             default: ;
         }
+
+        Element_Code=(int64u)-1;
         Open_Buffer_Init(Parser);
     }
 
@@ -802,21 +810,22 @@ void File_Mpeg4_Descriptors::Descriptor_05()
 
     //Demux
     #if MEDIAINFO_DEMUX
-        switch (Config->Demux_InitData_Get())
-        {
-            case 0 :    //In demux event
-                        Demux_Level=2; //Container
-                        Demux(Buffer+Buffer_Offset, (size_t)Element_Size, ContentType_Header);
-                        break;
-            case 1 :    //In field
-                        {
-                        std::string Data_Raw((const char*)(Buffer+Buffer_Offset), (size_t)Element_Size);
-                        std::string Data_Base64(Base64::encode(Data_Raw));
-                        Parser->Fill(KindOfStream, 0, "Demux_InitBytes", Data_Base64);
-                        }
-                        break;
-            default :   ;
-        }
+        if (ObjectTypeId!=0x21 || !Config->Demux_Avc_Transcode_Iso14496_15_to_Iso14496_10_Get()) //0x21 is AVC
+            switch (Config->Demux_InitData_Get())
+            {
+                case 0 :    //In demux event
+                            Demux_Level=2; //Container
+                            Demux(Buffer+Buffer_Offset, (size_t)Element_Size, ContentType_Header);
+                            break;
+                case 1 :    //In field
+                            {
+                            std::string Data_Raw((const char*)(Buffer+Buffer_Offset), (size_t)Element_Size);
+                            std::string Data_Base64(Base64::encode(Data_Raw));
+                            Parser->Fill(KindOfStream, 0, "Demux_InitBytes", Data_Base64);
+                            }
+                            break;
+                default :   ;
+            }
     #endif //MEDIAINFO_DEMUX
 
     //Parser configuration after the parsing
@@ -847,7 +856,7 @@ void File_Mpeg4_Descriptors::Descriptor_06()
 
     //Parsing
     int8u predefined;
-    Get_B1 (predefined,                                         "predefined"); Param_Info(Mpeg4_Descriptors_Predefined(predefined));
+    Get_B1 (predefined,                                         "predefined"); Param_Info1(Mpeg4_Descriptors_Predefined(predefined));
     switch (predefined)
     {
         case 0x00 :
