@@ -1,17 +1,17 @@
 // File_Mxf - Info for MXF files
-// Copyright (C) 2006-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2006-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -33,6 +33,7 @@
 #endif //defined(MEDIAINFO_ANCILLARY_YES)
 #include "MediaInfo/MediaInfo_Internal.h"
 #include <vector>
+#include <set>
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -79,8 +80,8 @@ protected :
     void Streams_Finish_Track (int128u TrackUID);
     void Streams_Finish_Essence (int32u EssenceUID, int128u TrackUID);
     void Streams_Finish_Descriptor (int128u DescriptorUID, int128u PackageUID);
-    void Streams_Finish_Locator (int128u LocatorUID);
-    void Streams_Finish_Component (int128u ComponentUID, float64 EditRate);
+    void Streams_Finish_Locator (int128u DescriptorUID, int128u LocatorUID);
+    void Streams_Finish_Component (int128u ComponentUID, float64 EditRate, int32u TrackID);
     void Streams_Finish_Identification (int128u IdentificationUID);
     void Streams_Finish_CommercialNames ();
 
@@ -129,12 +130,13 @@ protected :
     void Track();
     void DMSegment();
     void GenericSoundEssenceDescriptor();
+    void GenericDataEssenceDescriptor();
     void MultipleDescriptor();
     void AES3PCMDescriptor();
     void WaveAudioDescriptor();
     void MPEG2VideoDescriptor();
     void JPEG2000PictureSubDescriptor();
-    void Unknown1();
+    void VbiPacketsDescriptor();
     void AncPacketsDescriptor();
     void OpenIncompleteHeaderPartition();
     void ClosedIncompleteHeaderPartition();
@@ -158,6 +160,7 @@ protected :
     void SDTI_DataMetadataSet();
     void SDTI_ControlMetadataSet();
     void SystemScheme1();
+    void DMScheme1();
     void Omneon_010201010100();
     void Omneon_010201020100();
 
@@ -193,6 +196,7 @@ protected :
     void ContentStorage_Packages();                             //1901
     void ContentStorage_EssenceContainerData();                 //1902
     void DMSegment_DMFramework();                               //6101
+    void DMSegment_TrackIDs();                                  //6102
     void EssenceContainerData_LinkedPackageUID();               //2701
     void EssenceContainerData_IndexSID();                       //3F06
     void EssenceContainerData_BodySID();                        //3F07
@@ -243,6 +247,7 @@ protected :
     void GenericSoundEssenceDescriptor_SoundEssenceCompression(); //3D06
     void GenericSoundEssenceDescriptor_ChannelCount();          //3D07
     void GenericSoundEssenceDescriptor_DialNorm();              //3D0C
+    void GenericDataEssenceDescriptor_DataEssenceCoding();      //3E01
     void GenericTrack_TrackID();                                //4801
     void GenericTrack_TrackName();                              //4802
     void GenericTrack_Sequence();                               //4803
@@ -279,6 +284,9 @@ protected :
     void JPEG2000PictureSubDescriptor_Csiz();                   //800A
     void JPEG2000PictureSubDescriptor_PictureComponentSizing(); //800B
     void MultipleDescriptor_SubDescriptorUIDs();                //3F01
+    void DMScheme1_PrimaryExtendedSpokenLanguage();             //
+    void DMScheme1_SecondaryExtendedSpokenLanguage();           //
+    void DMScheme1_OriginalExtendedSpokenLanguage();            //
     void MPEG2VideoDescriptor_SingleSequence();                 //
     void MPEG2VideoDescriptor_ConstantBFrames();                //
     void MPEG2VideoDescriptor_CodedContentType();               //
@@ -374,7 +382,7 @@ protected :
     };
     std::vector<randomindexmetadata> RandomIndexMetadatas;
     bool                             RandomIndexMetadatas_AlreadyParsed;
-    int64u                           RandomIndexMetadatas_ByteOffsetParsed;
+    std::set<int64u>                 PartitionPack_AlreadyParsed;
     size_t Streams_Count;
     int128u Code;
     int128u OperationalPattern;
@@ -388,7 +396,11 @@ protected :
     int64u IsParsingMiddle_MaxOffset;
     bool   Track_Number_IsAvailable;
     bool   IsParsingEnd;
+    bool   IsCheckingRandomAccessTable;
+    bool   IsCheckingFooterPartitionAddress;
+    bool   FooterPartitionAddress_Jumped;
     bool   PartitionPack_Parsed;
+    size_t IdIsAlwaysSame_Offset;
 
     //Primer
     std::map<int16u, int128u> Primer_Values;
@@ -472,12 +484,13 @@ protected :
     tracks Tracks;
 
     //Essence
+    typedef std::vector<File__Analyze*> parsers;
     struct essence
     {
         stream_t StreamKind;
         size_t   StreamPos;
         size_t   StreamPos_Initial;
-        File__Analyze* Parser;
+        parsers Parsers;
         std::map<std::string, Ztring> Infos;
         int64u Stream_Size;
         int32u TrackID;
@@ -494,7 +507,6 @@ protected :
             StreamKind=Stream_Max;
             StreamPos=(size_t)-1;
             StreamPos_Initial=(size_t)-1;
-            Parser=NULL;
             Stream_Size=(int64u)-1;
             TrackID=(int32u)-1;
             TrackID_WasLookedFor=false;
@@ -508,7 +520,8 @@ protected :
 
         ~essence()
         {
-            delete Parser; //Parser=NULL;
+            for (size_t Pos=0; Pos<Parsers.size(); Pos++)
+                delete Parsers[Pos];
         }
     };
     typedef std::map<int32u, essence> essences; //Key is TrackNumber
@@ -520,9 +533,11 @@ protected :
         std::vector<int128u> SubDescriptors;
         std::vector<int128u> Locators;
 
+        Ztring  ScanType;
         stream_t StreamKind;
         size_t   StreamPos;
         float64 SampleRate;
+        float64 DisplayAspectRatio;
         int128u InstanceUID;
         int128u EssenceContainer;
         int128u EssenceCompression;
@@ -541,6 +556,8 @@ protected :
         int32u QuantizationBits;
         int64u Duration;
         int8u  ActiveFormat;
+        int8u  FieldTopness;
+        int8u  FieldDominance;
         enum type
         {
             Type_Unknown,
@@ -553,15 +570,15 @@ protected :
             Type_AncPackets,
         };
         type Type;
-        #if MEDIAINFO_DEMUX || MEDIAINFO_SEEK
-            int32u ByteRate;
-        #endif //MEDIAINFO_DEMUX || MEDIAINFO_SEEK
+        bool HasBFrames;
+        int32u ByteRate;
 
         descriptor()
         {
             StreamKind=Stream_Max;
             StreamPos=(size_t)-1;
             SampleRate=0;
+            DisplayAspectRatio=0;
             InstanceUID.hi=(int64u)-1;
             InstanceUID.lo=(int64u)-1;
             EssenceContainer.hi=(int64u)-1;
@@ -582,10 +599,11 @@ protected :
             QuantizationBits=(int8u)-1;
             Duration=(int64u)-1;
             ActiveFormat=(int8u)-1;
+            FieldTopness=(int8u)-1; //Field x is upper field
+            FieldDominance=1; //Default is field 1 temporaly first
             Type=Type_Unknown;
-            #if MEDIAINFO_DEMUX || MEDIAINFO_SEEK
-                ByteRate=(int32u)-1;
-            #endif //MEDIAINFO_DEMUX || MEDIAINFO_SEEK
+            HasBFrames=false;
+            ByteRate=(int32u)-1;
         }
     };
     typedef std::map<int128u, descriptor> descriptors; //Key is InstanceUID of Descriptor
@@ -597,12 +615,14 @@ protected :
         Ztring      EssenceLocator;
         stream_t    StreamKind;
         size_t      StreamPos;
+        int32u      LinkedTrackID;
         bool        IsTextLocator;
 
         locator()
         {
             StreamKind=Stream_Max;
             StreamPos=(size_t)-1;
+            LinkedTrackID=(int32u)-1;
             IsTextLocator=false;
         }
 
@@ -625,15 +645,70 @@ protected :
         int32u  SourceTrackID;
         std::vector<int128u> StructuralComponents; //Sequence from MaterialPackage only
 
+        //Time code component
+        int16u  TimeCode_RoundedTimecodeBase;
+        int64u  TimeCode_StartTimecode;
+        bool    TimeCode_DropFrame;
+
         component()
         {
             Duration=(int64u)-1;
-            SourceTrackID=0;
+            SourceTrackID=(int32u)-1;
+
+            //Time code component
+            TimeCode_RoundedTimecodeBase=(int16u)-1;
+            TimeCode_StartTimecode=(int64u)-1;
+            TimeCode_DropFrame=false;
+        }
+
+        void Update (struct component &New)
+        {
+            if (New.Duration!=(int64u)-1)
+                Duration=New.Duration;
+            if (New.SourcePackageID.hi || New.SourcePackageID.lo)
+                SourcePackageID=New.SourcePackageID;
+            if (New.SourceTrackID!=(int32u)-1)
+                SourceTrackID=New.SourceTrackID;
+            if (!New.StructuralComponents.empty())
+                StructuralComponents=New.StructuralComponents;
         }
     };
     typedef std::map<int128u, component> components; //Key is InstanceUID of the component
     components Components;
 
+    //Descriptive Metadata - DMSegments
+    struct dmsegment
+    {
+        int128u     Framework;
+        std::vector<int32u> TrackIDs;
+
+        dmsegment()
+        {
+        }
+
+        ~dmsegment()
+        {
+        }
+    };
+    typedef std::map<int128u, dmsegment> dmsegments; //Key is InstanceUID of the DMSegment
+    dmsegments DMSegments;
+
+    //Descriptive Metadata - DMScheme1
+    struct dmscheme1
+    {
+        Ztring      PrimaryExtendedSpokenLanguage;
+
+        dmscheme1()
+        {
+        }
+
+        ~dmscheme1()
+        {
+        }
+    };
+    typedef std::map<int128u, dmscheme1> dmscheme1s; //Key is InstanceUID of the DMScheme1
+    dmscheme1s DMScheme1s;
+
     //Parsers
     void           ChooseParser__FromEssence(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__Aaf(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
@@ -647,23 +722,24 @@ protected :
     void           ChooseParser__Aaf_GC_Compound(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__Avid(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
     void           ChooseParser__Avid_Picture(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser__FromEssenceContainer(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Avc(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_DV(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Mpeg4v(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Mpegv(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Raw(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_RV24(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Vc3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Aac(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Ac3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Aes3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Alaw(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_ChannelGrouping(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Mpega(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Pcm(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
-    File__Analyze* ChooseParser_Jpeg2000(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser__FromEssenceContainer(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Avc(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_DV(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Mpeg4v(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Mpegv(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Raw(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_RV24(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Vc3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Aac(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Ac3(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Alaw(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_ChannelGrouping(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Mpega(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Pcm(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_SmpteSt0331(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_SmpteSt0337(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
+    void           ChooseParser_Jpeg2000(const essences::iterator &Essence, const descriptors::iterator &Descriptor);
 
     //Helpers
     void Subsampling_Compute(descriptors::iterator Descriptor);
@@ -680,13 +756,15 @@ protected :
     bool   TimeCode_DropFrame;
     float64 DTS_Delay; //In seconds
     bool   StreamPos_StartAtOne; //information about the base of StreamPos (0 or 1, 1 is found in 1 file)
-    int64u SDTI_TimeCode_StartTimecode;
+    string SDTI_TimeCode_StartTimecode;
+    int64u SDTI_TimeCode_StartTimecode_ms;
     int64u SDTI_SizePerFrame;
     bool   SDTI_IsPresent; //Used to test if SDTI packet is used for Index StreamOffset calculation
     bool   SDTI_IsInIndexStreamOffset; //Used to test if SDTI packet is used for Index StreamOffset calculation
-    int64u SystemScheme1_TimeCodeArray_StartTimecode;
+    string SystemScheme1_TimeCodeArray_StartTimecode;
+    int64u SystemScheme1_TimeCodeArray_StartTimecode_ms;
     int64u SystemScheme1_FrameRateFromDescriptor;
-    bool   Essences_FirstEssence_Parsed; 
+    bool   Essences_FirstEssence_Parsed;
     int32u IndexTable_NSL;
     int32u IndexTable_NPE;
     #if defined(MEDIAINFO_ANCILLARY_YES)
@@ -701,18 +779,20 @@ protected :
     struct partition
     {
         int64u StreamOffset; //From file, not MXF one
-        int64u BodyOffset;
         int64u PartitionPackByteCount; //Fill included
+        int64u FooterPartition;
         int64u HeaderByteCount;
         int64u IndexByteCount;
+        int64u BodyOffset;
 
         partition()
         {
             StreamOffset=0;
-            BodyOffset=0;
             PartitionPackByteCount=(int64u)-1;
+            FooterPartition=0;
             HeaderByteCount=0;
             IndexByteCount=0;
+            BodyOffset=0;
         }
 
         bool operator < (const partition& lhs) const
@@ -729,7 +809,6 @@ protected :
 
     #if MEDIAINFO_DEMUX || MEDIAINFO_SEEK
         bool Demux_HeaderParsed;
-        bool Demux_Interleave;
         size_t CountOfLocatorsToParse;
         float64 Demux_Rate;
 
@@ -765,13 +844,15 @@ protected :
         typedef std::vector<indextable> indextables;
         indextables                     IndexTables;
         size_t                          IndexTables_Pos;
-        
+
         //Other
         int64u  Clip_Header_Size;
         int64u  Clip_Begin;
         int64u  Clip_End;
         int128u Clip_Code;
+        int64u  OverallBitrate_IsCbrForSure;
         bool    Duration_Detected;
+        bool    DetectDuration();
     #endif //MEDIAINFO_DEMUX || MEDIAINFO_SEEK
 };
 
