@@ -1,17 +1,17 @@
 // File_Riff - Info for RIFF files
-// Copyright (C) 2002-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2002-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -53,6 +53,7 @@
 #if MEDIAINFO_EVENTS
     #include "MediaInfo/MediaInfo_Events.h"
 #endif //MEDIAINFO_EVENTS
+#include <ZenLib/File.h>
 #include <ZenLib/Utils.h>
 using namespace ZenLib;
 //---------------------------------------------------------------------------
@@ -96,11 +97,10 @@ File_Riff::File_Riff()
     //Configuration
     #if MEDIAINFO_EVENTS
         ParserIDs[0]=MediaInfo_Parser_Riff;
-        StreamIDs_Width[0]=17;
+        StreamIDs_Width[0]=4;
     #endif //MEDIAINFO_EVENTS
     #if MEDIAINFO_DEMUX
         Demux_Level=2; //Container
-        Demux_EventWasSent_Accept_Specific=true;
     #endif //MEDIAINFO_DEMUX
     DataMustAlwaysBeComplete=false;
 
@@ -138,9 +138,14 @@ File_Riff::File_Riff()
     IsWave64=false;
     IsRIFF64=false;
     IsWaveBroken=false;
+    IsNotWordAligned=false;
+    IsNotWordAligned_Tested=false;
     SecondPass=false;
     DV_FromHeader=NULL;
     Kind=Kind_None;
+    #if MEDIAINFO_DEMUX
+    Demux_Parser=NULL;
+    #endif //MEDIAINFO_DEMUX
 
     //Pointers
     Stream_Structure_Temp=Stream_Structure.end();
@@ -162,13 +167,16 @@ File_Riff::~File_Riff()
 void File_Riff::Streams_Finish ()
 {
     //Ancillary specific
-    if (Ancillary && (*Ancillary))
-    {
-        Finish(*Ancillary);
-        Merge(**Ancillary);
-        return;
-    }
-    
+    #if defined(MEDIAINFO_ANCILLARY_YES)
+        if (Ancillary && (*Ancillary))
+        {
+            Clear();
+            Finish(*Ancillary);
+            Merge(**Ancillary);
+            return;
+        }
+    #endif //defined(MEDIAINFO_ANCILLARY_YES)
+
     //Global
     if (IsRIFF64)
         Fill(Stream_General, 0, General_Format_Profile, "RF64");
@@ -185,6 +193,19 @@ void File_Riff::Streams_Finish ()
         if (Temp->second.StreamSize>0)
             Fill(StreamKind_Last, StreamPos_Last, Fill_Parameter(StreamKind_Last, Generic_StreamSize), Temp->second.StreamSize);
 
+        //When there are few frames, difficult to detect PCM
+        if (Temp->second.IsPcm && !Temp->second.Parsers.empty() && !Temp->second.Parsers[0]->Status[IsAccepted])
+        {
+            for (size_t Pos=0; Pos<Temp->second.Parsers.size()-1; Pos++)
+                delete Temp->second.Parsers[Pos];
+            Temp->second.Parsers.erase(Temp->second.Parsers.begin(), Temp->second.Parsers.begin()+Temp->second.Parsers.size()-1);
+            Temp->second.Parsers[0]->Accept();
+        }
+
+        Ztring ID;
+        if (Temp->first!=(int32u)-1)
+            ID.From_Number(((Temp->first>>24)-'0')*10+(((Temp->first>>16)&0xFF)-'0'));
+
         //Parser specific
         if (Temp->second.Parsers.size()==1)
         {
@@ -198,14 +219,36 @@ void File_Riff::Streams_Finish ()
             StreamSize=Retrieve(StreamKind_Last, StreamPos_Last, Fill_Parameter(StreamKind_Last, Generic_StreamSize)); //We want to keep the 4CC of AVI
 
             //Merging
-            if (Config_ParseSpeed<=1.0)
+            if (Config->ParseSpeed<=1.0)
             {
                 Fill(Temp->second.Parsers[0]);
                 Temp->second.Parsers[0]->Open_Buffer_Unsynch();
             }
             Finish(Temp->second.Parsers[0]);
-            Merge(*Temp->second.Parsers[0], StreamKind_Last, 0, StreamPos_Last);
-            Fill(StreamKind_Last, StreamPos_Last, General_ID, ((Temp->first>>24)-'0')*10+(((Temp->first>>16)&0xFF)-'0'));
+            if (!Temp->second.Parsers.empty() && Temp->second.Parsers[0]->Count_Get(StreamKind_Last))
+                for (size_t Pos=0; Pos<Temp->second.Parsers[0]->Count_Get(StreamKind_Last); Pos++)
+                {
+                    Ztring Temp_ID=ID;
+                    Ztring Temp_ID_String=ID;
+                    Merge(*Temp->second.Parsers[0], StreamKind_Last, Pos, StreamPos_Last+Pos);
+                    if (!Retrieve(StreamKind_Last, StreamPos_Last, General_ID).empty())
+                    {
+                        if (!Temp_ID.empty())
+                        {
+                            Temp_ID+=__T('-');
+                            Temp_ID_String+=__T('-');
+                        }
+                        Temp_ID+=Retrieve(StreamKind_Last, StreamPos_Last, General_ID);
+                        Temp_ID_String+=Retrieve(StreamKind_Last, StreamPos_Last, General_ID);
+                    }
+                    Fill(StreamKind_Last, StreamPos_Last, General_ID, Temp_ID, true);
+                    Fill(StreamKind_Last, StreamPos_Last, General_StreamOrder, Temp_ID_String, true);
+                }
+            else
+            {
+                Fill(StreamKind_Last, StreamPos_Last, General_ID, ID, true);
+                Fill(StreamKind_Last, StreamPos_Last, General_StreamOrder, ID, true);
+            }
 
             //Hacks - After
             Fill(StreamKind_Last, StreamPos_Last, Fill_Parameter(StreamKind_Last, Generic_StreamSize), StreamSize, true);
@@ -218,18 +261,22 @@ void File_Riff::Streams_Finish ()
                 const Ztring &FrameRate=Retrieve(Stream_Video, StreamPos_Last, Video_FrameRate);
                 if (FrameRate.To_int32u()==120)
                 {
-                    Fill(Stream_Video, StreamPos_Last, Video_FrameRate_String, MediaInfoLib::Config.Language_Get(FrameRate+_T(" (24/30)"), _T(" fps")), true);
+                    Fill(Stream_Video, StreamPos_Last, Video_FrameRate_String, MediaInfoLib::Config.Language_Get(FrameRate+__T(" (24/30)"), __T(" fps")), true);
                     Fill(Stream_Video, StreamPos_Last, Video_FrameRate_Minimum, 24, 10, true);
                     Fill(Stream_Video, StreamPos_Last, Video_FrameRate_Maximum, 30, 10, true);
                     Fill(Stream_Video, StreamPos_Last, Video_FrameRate_Mode, "VFR");
                 }
             }
 
+            //Hack - SMPTE ST 337, RIFF channels count is wrong
+            if (StreamKind_Last==Stream_Audio && Retrieve(Stream_Audio, StreamPos_Last, Audio_MuxingMode)==__T("AES3") && Temp->second.Parsers[0]->Get(Stream_Audio, 0, Audio_Channel_s_).empty())
+                Clear(Stream_Audio, StreamPos_Last, Audio_Channel_s_);
+
             //Alignment
             if (StreamKind_Last==Stream_Audio && Count_Get(Stream_Video)>0) //Only if this is not a WAV file
             {
                 Fill(Stream_Audio, StreamPos_Last, Audio_Alignment, Temp->second.ChunksAreComplete?"Aligned":"Split");
-                Fill(Stream_Audio, StreamPos_Last, Audio_Alignment_String, MediaInfoLib::Config.Language_Get(Temp->second.ChunksAreComplete?_T("Alignment_Aligned"):_T("Alignment_Split")));
+                Fill(Stream_Audio, StreamPos_Last, Audio_Alignment_String, MediaInfoLib::Config.Language_Get(Temp->second.ChunksAreComplete?__T("Alignment_Aligned"):__T("Alignment_Split")));
             }
 
             //Delay
@@ -263,7 +310,7 @@ void File_Riff::Streams_Finish ()
                 {
                     Delay+=((float)Temp->second.Start)*1000/Temp->second.Rate;
                     Fill(Stream_Audio, StreamPos_Last, Audio_Delay, Delay, 0, true);
-                    Fill(Stream_Audio, StreamPos_Last, Audio_Delay_Source, "Container");
+                    Fill(Stream_Audio, StreamPos_Last, Audio_Delay_Source, "Stream", Unlimited, true, true);
                     for (size_t StreamPos=0; StreamPos<Count_Get(Stream_Video); StreamPos++)
                         Fill(Stream_Video, StreamPos, Video_Delay, 0, 10, true);
                 }
@@ -271,16 +318,16 @@ void File_Riff::Streams_Finish ()
 
             //Special case: AAC
             if (StreamKind_Last==Stream_Audio
-             && (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==_T("AAC")
-              || Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==_T("MPEG Audio")
-              || Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==_T("Vorbis")))
+             && (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==__T("AAC")
+              || Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==__T("MPEG Audio")
+              || Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==__T("Vorbis")))
                 Clear(Stream_Audio, StreamPos_Last, Audio_BitDepth); //Resolution is not valid for AAC / MPEG Audio / Vorbis
 
             //Format specific
             #if defined(MEDIAINFO_DVDIF_YES)
-                if (StreamKind_Last==Stream_Video && (MediaInfoLib::Config.Codec_Get(Ztring().From_CC4(Temp->second.Compression), InfoCodec_KindofCodec).find(_T("DV"))==0
-                                                   || Retrieve(Stream_Video, StreamPos_Last, Video_Format)==_T("DV")
-                                                   || Retrieve(Stream_Video, StreamPos_Last, Video_Codec)==_T("DV")))
+                if (StreamKind_Last==Stream_Video && (MediaInfoLib::Config.Codec_Get(Ztring().From_CC4(Temp->second.Compression), InfoCodec_KindofCodec).find(__T("DV"))==0
+                                                   || Retrieve(Stream_Video, StreamPos_Last, Video_Format)==__T("DV")
+                                                   || Retrieve(Stream_Video, StreamPos_Last, Video_Codec)==__T("DV")))
                 {
                     if (Retrieve(Stream_General, 0, General_Recorded_Date).empty())
                         Fill(Stream_General, 0, General_Recorded_Date, Temp->second.Parsers[0]->Retrieve(Stream_General, 0, General_Recorded_Date));
@@ -295,10 +342,10 @@ void File_Riff::Streams_Finish ()
                         Merge(*Temp->second.Parsers[0], Stream_Audio, Audio_Pos, StreamPos_Last);
                         Fill(Stream_Audio, Pos, Audio_MuxingMode, "DV");
                         Fill(Stream_Audio, Pos, Audio_Duration, Retrieve(Stream_Video, Temp->second.StreamPos, Video_Duration));
-                        Fill(Stream_Audio, Pos, "MuxingMode_MoreInfo", _T("Muxed in Video #")+Ztring().From_Number(Temp->second.StreamPos+1));
-                        Fill(Stream_Audio, Pos, Audio_StreamSize, "0"); //Included in the DV stream size
+                        Fill(Stream_Audio, Pos, "MuxingMode_MoreInfo", __T("Muxed in Video #")+Ztring().From_Number(Temp->second.StreamPos+1));
+                        Fill(Stream_Audio, Pos, Audio_StreamSize_Encoded, 0); //Included in the DV stream size
                         Ztring ID=Retrieve(Stream_Audio, Pos, Audio_ID);
-                        Fill(Stream_Audio, Pos, Audio_ID, Retrieve(Stream_Video, Temp->second.StreamPos, Video_ID)+_T("-")+ID, true);
+                        Fill(Stream_Audio, Pos, Audio_ID, Retrieve(Stream_Video, Temp->second.StreamPos, Video_ID)+__T("-")+ID, true);
                     }
 
                     StreamKind_Last=Stream_Video;
@@ -307,7 +354,7 @@ void File_Riff::Streams_Finish ()
             #endif
         }
         else if (StreamKind_Last!=Stream_General)
-            Fill(StreamKind_Last, StreamPos_Last, General_ID, ((Temp->first>>24)-'0')*10+(((Temp->first>>16)&0xFF)-'0'));
+            Fill(StreamKind_Last, StreamPos_Last, General_ID, ID);
 
         //Duration
         if (Temp->second.PacketCount>0)
@@ -325,31 +372,31 @@ void File_Riff::Streams_Finish ()
                 //Duration in case it is missing from header (malformed header...)
                 int64u SamplingCount=0;
                 #if defined(MEDIAINFO_MPEGA_YES)
-                if (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==_T("MPEG Audio"))
+                if (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==__T("MPEG Audio"))
                 {
                     if (Temp->second.Parsers[0] && Temp->second.PacketPos==((File_Mpega*)Temp->second.Parsers[0])->Frame_Count_Valid) //Only for stream with one frame per chunk
                     {
                         Ztring Version=Retrieve(Stream_Audio, StreamPos_Last, Audio_Format_Version);
                         Ztring Layer=Retrieve(Stream_Audio, StreamPos_Last, Audio_Format_Profile);
-                        if (Version==_T("Version 1") && Layer==_T("Layer 1"))
+                        if (Version==__T("Version 1") && Layer==__T("Layer 1"))
                             SamplingCount=Temp->second.PacketCount*384;  //MPEG-1 Layer 1
-                        else if ((Version==_T("Version 2") || Version==_T("Version 2.5")) && Layer==_T("Layer 1"))
+                        else if ((Version==__T("Version 2") || Version==__T("Version 2.5")) && Layer==__T("Layer 1"))
                             SamplingCount=Temp->second.PacketCount*192;  //MPEG-2 or 2.5 Layer 1
-                        else if ((Version==_T("Version 2") || Version==_T("Version 2.5")) && Layer==_T("Layer 3"))
+                        else if ((Version==__T("Version 2") || Version==__T("Version 2.5")) && Layer==__T("Layer 3"))
                             SamplingCount=Temp->second.PacketCount*576;  //MPEG-2 or 2.5 Layer 3
                         else
                             SamplingCount=Temp->second.PacketCount*1152;
                     }
                 }
                 #endif
-                if (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==_T("PCM"))
+                if (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==__T("PCM"))
                 {
                     int64u Resolution=Retrieve(Stream_Audio, StreamPos_Last, Audio_BitDepth).To_int64u();
                     int64u Channels=Retrieve(Stream_Audio, StreamPos_Last, Audio_Channel_s_).To_int64u();
                     if (Resolution>0 && Channels>0)
                         SamplingCount=Temp->second.StreamSize*8/Resolution/Channels;
                 }
-                if (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==_T("ADPCM"))
+                if (Retrieve(Stream_Audio, StreamPos_Last, Audio_Format)==__T("ADPCM"))
                 {
                     int64u Resolution=Retrieve(Stream_Audio, StreamPos_Last, Audio_BitDepth).To_int64u();
                     int64u Channels=Retrieve(Stream_Audio, StreamPos_Last, Audio_Channel_s_).To_int64u();
@@ -371,6 +418,8 @@ void File_Riff::Streams_Finish ()
                     Fill(Stream_Audio, StreamPos_Last, Audio_Duration, SamplingCount*1000/SamplingRate, 0, true);
                 else if (Temp->second.indx_Duration && Temp->second.Rate)
                     Fill(Stream_Audio, StreamPos_Last, Audio_Duration, ((float64)Temp->second.indx_Duration)*1000*Temp->second.Scale/Temp->second.Rate, 0, true);
+                else if (Temp->second.Rate && Temp->second.Scale!=1) //Note: some files with Scale==1 are buggy
+                    Fill(Stream_Audio, StreamPos_Last, Audio_Duration, ((float64)Temp->second.Length)*1000*Temp->second.Scale/Temp->second.Rate, 0, true);
 
                 //Interleave
                 if (Stream[0x30300000].PacketCount && Temp->second.PacketCount)
@@ -381,18 +430,18 @@ void File_Riff::Streams_Finish ()
                         Fill(Stream_Audio, StreamPos_Last, "Interleave_Duration", (float)Stream[0x30300000].PacketCount/Temp->second.PacketCount*1000/Retrieve(Stream_Video, 0, Video_FrameRate).To_float32(), 0);
                         Ztring Interleave_Duration_String;
                         Interleave_Duration_String+=Retrieve(Stream_Audio, StreamPos_Last, "Interleave_Duration");
-                        Interleave_Duration_String+=_T(" ");
-                        Interleave_Duration_String+=MediaInfoLib::Config.Language_Get(_T("ms"));
+                        Interleave_Duration_String+=__T(" ");
+                        Interleave_Duration_String+=MediaInfoLib::Config.Language_Get(__T("ms"));
                         if (!Retrieve(Stream_Audio, StreamPos_Last, "Interleave_VideoFrames").empty())
                         {
-                            Interleave_Duration_String+=_T(" (");
-                            Interleave_Duration_String+=MediaInfoLib::Config.Language_Get(Retrieve(Stream_Audio, StreamPos_Last, "Interleave_VideoFrames"), _T(" video frames"));
-                            Interleave_Duration_String+=_T(")");
+                            Interleave_Duration_String+=__T(" (");
+                            Interleave_Duration_String+=MediaInfoLib::Config.Language_Get(Retrieve(Stream_Audio, StreamPos_Last, "Interleave_VideoFrames"), __T(" video frames"));
+                            Interleave_Duration_String+=__T(")");
                         }
                         Fill(Stream_Audio, StreamPos_Last, "Interleave_Duration/String", Interleave_Duration_String);
                     }
                     int64u Audio_FirstBytes=0;
-                    for (std::map<int64u, stream_structure>::iterator Stream_Structure_Temp=Stream_Structure.begin(); Stream_Structure_Temp!=Stream_Structure.end(); Stream_Structure_Temp++)
+                    for (std::map<int64u, stream_structure>::iterator Stream_Structure_Temp=Stream_Structure.begin(); Stream_Structure_Temp!=Stream_Structure.end(); ++Stream_Structure_Temp)
                     {
                         if (Stream_Structure_Temp->second.Name==0x30300000)
                             break;
@@ -402,13 +451,30 @@ void File_Riff::Streams_Finish ()
                     if (Audio_FirstBytes && Retrieve(Stream_Audio, StreamPos_Last, Audio_BitRate).To_int32u())
                     {
                         Fill(Stream_Audio, StreamPos_Last, "Interleave_Preload", Audio_FirstBytes*1000/Temp->second.AvgBytesPerSec);
-                        Fill(Stream_Audio, StreamPos_Last, "Interleave_Preload/String", Retrieve(Stream_Audio, StreamPos_Last, "Interleave_Preload")+_T(" ")+MediaInfoLib::Config.Language_Get(_T("ms")));
+                        Fill(Stream_Audio, StreamPos_Last, "Interleave_Preload/String", Retrieve(Stream_Audio, StreamPos_Last, "Interleave_Preload")+__T(" ")+MediaInfoLib::Config.Language_Get(__T("ms")));
                     }
                 }
             }
+
+            //Source duration
+            if (Temp->second.PacketCount && Temp->second.Length!=Temp->second.PacketCount)
+            {
+                if (StreamKind_Last==Stream_Video && Temp->second.Rate)
+                    Fill(Stream_Video, StreamPos_Last, "Source_Duration", ((float64)Temp->second.PacketCount)*1000*Temp->second.Scale/Temp->second.Rate, 0);
+                if (StreamKind_Last==Stream_Audio && Temp->second.Rate)
+                {
+                    float64 Duration_Source=((float64)Temp->second.StreamSize)*1000/Temp->second.AvgBytesPerSec;
+                    float64 Duration_Header=Retrieve(Stream_Audio, StreamPos_Last, Audio_Duration).To_float64();
+                    float64 Difference=Duration_Source-Duration_Header;
+                    if (Temp->second.Scale!=1 && float64_int64s(Duration_Header/Duration_Source)==Temp->second.Scale)
+                        Fill(Stream_Audio, StreamPos_Last, Audio_Duration, Duration_Source, 0, true); //Found 1 stream with Scale not being right
+                    else if (Difference<-2 || Difference>2) //+/- 2 ms
+                        Fill(Stream_Audio, StreamPos_Last, "Source_Duration", Duration_Source, 0);
+                }
+            }
         }
 
-        Temp++;
+        ++Temp;
     }
 
     //Some work on the first video stream
@@ -426,8 +492,23 @@ void File_Riff::Streams_Finish ()
 
     //Interleaved
     if (Interleaved0_1 && Interleaved0_10 && Interleaved1_1 && Interleaved1_10)
-        Fill(Stream_General, 0, General_Interleaved, (Interleaved0_1<Interleaved1_1 && Interleaved0_10>Interleaved1_1
-                                                   || Interleaved1_1<Interleaved0_1 && Interleaved1_10>Interleaved0_1)?"Yes":"No");
+        Fill(Stream_General, 0, General_Interleaved, ((Interleaved0_1<Interleaved1_1 && Interleaved0_10>Interleaved1_1)
+                                                   || (Interleaved1_1<Interleaved0_1 && Interleaved1_10>Interleaved0_1))?"Yes":"No");
+
+    //Time codes
+    TimeCode_Fill(__T("ISMP"), INFO_ISMP);
+    TimeCode_Fill(__T("tc_A"), Tdat_tc_A);
+    TimeCode_Fill(__T("tc_O"), Tdat_tc_O);
+
+    //MD5
+    size_t Pos=0;
+    for (size_t StreamKind=Stream_General+1; StreamKind<Stream_Max; StreamKind++)
+        for (size_t StreamPos=0; StreamPos<Count_Get((stream_t)StreamKind); StreamPos++)
+            if (Pos<MD5s.size())
+            {
+                Fill((stream_t)StreamKind, StreamPos, "MD5", MD5s[Pos]);
+                Pos++;
+            }
 
     //Purge what is not needed anymore
     if (!File_Name.empty()) //Only if this is not a buffer, with buffer we can have more data
@@ -444,24 +525,24 @@ void File_Riff::Streams_Finish ()
              if (!Retrieve(Stream_Video, 0, Video_Format_Commercial_IfAny).empty())
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, Retrieve(Stream_Video, 0, Video_Format_Commercial_IfAny));
-            Fill(Stream_General, 0, General_Format_Commercial, _T("AVI ")+Retrieve(Stream_Video, 0, Video_Format_Commercial_IfAny));
+            Fill(Stream_General, 0, General_Format_Commercial, __T("AVI ")+Retrieve(Stream_Video, 0, Video_Format_Commercial_IfAny));
         }
-        else if (Retrieve(Stream_Video, 0, Video_Format)==_T("DV"))
+        else if (Retrieve(Stream_Video, 0, Video_Format)==__T("DV"))
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, "DV");
             Fill(Stream_General, 0, General_Format_Commercial, "AVI DV");
         }
-        else if (Retrieve(Stream_Video, 0, Video_Format)==_T("MPEG Video") && Retrieve(Stream_Video, 0, Video_Format_Settings_GOP)==_T("N=1") && Retrieve(Stream_Video, 0, Video_Colorimetry)==_T("4:2:2") && Retrieve(Stream_Video, 0, Video_BitRate)==_T("30000000"))
+        else if (Retrieve(Stream_Video, 0, Video_Format)==__T("MPEG Video") && Retrieve(Stream_Video, 0, Video_Format_Settings_GOP)==__T("N=1") && Retrieve(Stream_Video, 0, Video_Colorimetry)==__T("4:2:2") && Retrieve(Stream_Video, 0, Video_BitRate)==__T("30000000"))
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, "MPEG IMX 30");
             Fill(Stream_Video, 0, Video_Format_Commercial_IfAny, "MPEG IMX 30");
         }
-        else if (Retrieve(Stream_Video, 0, Video_Format)==_T("MPEG Video") && Retrieve(Stream_Video, 0, Video_Format_Settings_GOP)==_T("N=1") && Retrieve(Stream_Video, 0, Video_Colorimetry)==_T("4:2:2") && Retrieve(Stream_Video, 0, Video_BitRate)==_T("40000000"))
+        else if (Retrieve(Stream_Video, 0, Video_Format)==__T("MPEG Video") && Retrieve(Stream_Video, 0, Video_Format_Settings_GOP)==__T("N=1") && Retrieve(Stream_Video, 0, Video_Colorimetry)==__T("4:2:2") && Retrieve(Stream_Video, 0, Video_BitRate)==__T("40000000"))
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, "MPEG IMX 40");
             Fill(Stream_Video, 0, Video_Format_Commercial_IfAny, "MPEG IMX 40");
         }
-        else if (Retrieve(Stream_Video, 0, Video_Format)==_T("MPEG Video") && Retrieve(Stream_Video, 0, Video_Format_Settings_GOP)==_T("N=1") && Retrieve(Stream_Video, 0, Video_Colorimetry)==_T("4:2:2") && Retrieve(Stream_Video, 0, Video_BitRate)==_T("50000000"))
+        else if (Retrieve(Stream_Video, 0, Video_Format)==__T("MPEG Video") && Retrieve(Stream_Video, 0, Video_Format_Settings_GOP)==__T("N=1") && Retrieve(Stream_Video, 0, Video_Colorimetry)==__T("4:2:2") && Retrieve(Stream_Video, 0, Video_BitRate)==__T("50000000"))
         {
             Fill(Stream_General, 0, General_Format_Commercial_IfAny, "MPEG IMX 50");
             Fill(Stream_Video, 0, Video_Format_Commercial_IfAny, "MPEG IMX 50");
@@ -486,7 +567,7 @@ void File_Riff::Read_Buffer_Init()
 
 //---------------------------------------------------------------------------
 #if MEDIAINFO_SEEK
-size_t File_Riff::Read_Buffer_Seek (size_t Method, int64u Value, int64u ID)
+size_t File_Riff::Read_Buffer_Seek (size_t Method, int64u Value, int64u /*ID*/)
 {
     //Only Wave and AIFF
     switch (Kind)
@@ -547,14 +628,30 @@ void File_Riff::Read_Buffer_Unsynched()
     if (IsSub)
     {
         while(Element_Level)
-            Element_End();
+            Element_End0();
 
-        //Ancillary specific
-        if (Ancillary && (*Ancillary))
-            (*Ancillary)->Open_Buffer_Unsynch();
+        #if defined(MEDIAINFO_ANCILLARY_YES)
+            //Ancillary specific
+            if (Ancillary && (*Ancillary))
+                (*Ancillary)->Open_Buffer_Unsynch();
+        #endif //defined(MEDIAINFO_ANCILLARY_YES)
     }
 }
 
+//---------------------------------------------------------------------------
+#if MEDIAINFO_DEMUX
+void File_Riff::Read_Buffer_Continue()
+{
+    if (Demux_Parser)
+    {
+        Open_Buffer_Continue(Demux_Parser, Buffer+Buffer_Offset, 0, false);
+        Demux_Parser=NULL;
+        //if (Config->Demux_EventWasSent)
+        //    return;
+    }
+}
+#endif //MEDIAINFO_DEMUX
+
 //***************************************************************************
 // Buffer
 //***************************************************************************
@@ -582,7 +679,7 @@ bool File_Riff::Header_Begin()
                     return false;
             }
             else
-        #endif MEDIAINFO_DEMUX
+        #endif //MEDIAINFO_DEMUX
         if (File_Offset+Buffer_Size<=Buffer_DataToParse_End)
             Element_Size=Buffer_Size; //All the buffer is used
         else
@@ -593,22 +690,26 @@ bool File_Riff::Header_Begin()
 
         if (Buffer_Offset+(size_t)Element_Size>Buffer_Size)
             return false;
-        
-        Element_Begin();
+
+        // Fake header
+        Element_Begin0();
+        Element_Begin0();
+        Header_Fill_Size(Buffer_DataToParse_End-(File_Offset+Buffer_Offset));
+        Element_End();
+
         switch (Kind)
         {
             case Kind_Wave : WAVE_data_Continue(); break;
             case Kind_Aiff : AIFF_SSND_Continue(); break;
+            case Kind_Rmp3 : RMP3_data_Continue(); break;
             default        : AVI__movi_xxxx();
         }
 
-        if (Config_ParseSpeed<1.0 && File_Offset+Buffer_Offset+Element_Offset-Buffer_DataToParse_Begin>=0x10000)
+        if (Config->ParseSpeed<1.0 && File_Offset+Buffer_Offset+Element_Offset-Buffer_DataToParse_Begin>=0x10000)
         {
-            Buffer_Offset=Buffer_DataToParse_End-File_Offset;
-            if (Buffer_Offset<Buffer_Size)
-                Element_Size=Buffer_Size-Buffer_Offset;
-            else
-                Element_Size=0;
+            File_GoTo=Buffer_DataToParse_End;
+            Buffer_Offset=Buffer_Size;
+            Element_Size=0;
         }
         else
         {
@@ -616,7 +717,14 @@ bool File_Riff::Header_Begin()
             Element_Size-=Element_Offset;
         }
         Element_Offset=0;
-        Element_End();
+        Element_End0();
+
+        if (Status[IsFinished] || (File_GoTo!=(int64u)-1 && (File_GoTo<=Buffer_DataToParse_Begin || File_GoTo>=Buffer_DataToParse_End)))
+        {
+            Buffer_DataToParse_Begin=(int64u)-1;
+            Buffer_DataToParse_End=0;
+            return false;
+        }
 
         if (Buffer_Offset>=Buffer_Size)
             return false;
@@ -705,8 +813,29 @@ void File_Riff::Header_Parse()
     if (IsBigEndian)
         Get_B4 (Size,                                           "Size");
     else
+    {
         Get_L4 (Size,                                           "Size");
 
+        //Testing malformed (not word aligned)
+        if (!IsNotWordAligned_Tested && Size%2)
+        {
+            if (File_Offset+Buffer_Offset+8+Size==File_Size)
+                IsNotWordAligned=true;
+            else if (!File_Name.empty())
+            {
+                File F(File_Name);
+                F.GoTo(File_Offset+Buffer_Offset+8+Size);
+                int8u Temp;
+                if (F.Read(&Temp, 1))
+                {
+                    if (!((Temp<'A' || Temp>'z') && Temp!=' '))
+                        IsNotWordAligned=true;
+                }
+            }
+            IsNotWordAligned_Tested=true;
+        }
+    }
+
     //RF64
     int64u Size_Complete=Size;
     if (Size==0 && Name==Elements::RIFF)
@@ -721,17 +850,26 @@ void File_Riff::Header_Parse()
         if (Name==Elements::RF64 && CC4(Buffer+Buffer_Offset+0x0C)==Elements::WAVE_ds64)
         {
             Size_Complete=LittleEndian2int64u(Buffer+Buffer_Offset+0x14);
-            Param_Info(Size_Complete);
+            Param_Info1(Size_Complete);
         }
         else if (Name==Elements::WAVE_data)
         {
             Size_Complete=WAVE_data_Size;
-            Param_Info(Size_Complete);
+            Param_Info1(Size_Complete);
         }
     }
 
+    //Coherency
+    if (Stream_Structure_Temp!=Stream_Structure.end() && Stream_Structure_Temp->second.Size==0)
+    {
+        Name=(int32u)-1;
+        Size_Complete=0; //Hack in some indexes with Size==0 (why?), ignoring content of header
+    }
+    if (File_Offset+Buffer_Offset+8+Size_Complete>File_Size)
+        Size_Complete=File_Size-(File_Offset+Buffer_Offset+8);
+
     //Alignment
-    if (Size_Complete%2)
+    if (Size_Complete%2 && !IsNotWordAligned)
     {
         Size_Complete++; //Always 2-byte aligned
         Alignement_ExtraByte=1;
@@ -765,25 +903,28 @@ void File_Riff::Header_Parse()
     if (Name==Elements::ON2f)
         Name=Elements::AVI_;
 
-    //Filling
-    if (movi_Size && Size_Complete>movi_Size/2 && 8+Size_Complete>1024*1024 && !((Name&0xFFFF0000)==0x69780000 || (Name&0x0000FFFF)==0x00006978) && Element_Level==4+(rec__Present?1:0) && Buffer_Offset+8+Size_Complete>Buffer_Size)
+    //Tests
+    if (Element_Level==2 && Name==Elements::WAVE && !IsRIFF64 && File_Size>0xFFFFFFFF)
+        IsWaveBroken=true; //Non standard big files detection
+    if (IsWaveBroken && (Name==Elements::WAVE || Name==Elements::WAVE_data))
+        Size_Complete=File_Size-(File_Offset+Buffer_Offset+8); //Non standard big files detection
+    if (movi_Size && Size_Complete>movi_Size/2 && 8+Size_Complete>1024*1024 && !((Name&0xFFFF0000)==0x69780000 || (Name&0x0000FFFF)==0x00006978) && Element_Level==(rec__Present?(size_t)5:(size_t)4) && Buffer_Offset+8+Size_Complete>Buffer_Size)
     {
         Buffer_DataToParse_End=File_Offset+Buffer_Offset+8+Size_Complete;
         Size_Complete=Buffer_Size-(Buffer_Offset+8);
     }
-    if ((Name==Elements::WAVE_data || Name==Elements::AIFF_SSND) && AvgBytesPerSec && Demux_Rate && BlockAlign)
+    if ((Name==Elements::WAVE_data || Name==Elements::AIFF_SSND))
     {
         Buffer_DataToParse_Begin=File_Offset+Buffer_Offset+8;
-        Buffer_DataToParse_End=File_Offset+Buffer_Offset+8+Size_Complete;
+        if (Size_Complete)
+            Buffer_DataToParse_End=File_Offset+Buffer_Offset+8+Size_Complete;
+        else
+            Buffer_DataToParse_End=File_Size; //Found one file with 0 as size of data part
         Size_Complete=0;
     }
 
     //Filling
     Header_Fill_Code(Name, Ztring().From_CC4(Name));
-    if (Element_Level==2 && Name==Elements::WAVE && !IsRIFF64 && File_Size>0xFFFFFFFF)
-        IsWaveBroken=true; //Non standard big files detection
-    if (IsWaveBroken && (Name==Elements::WAVE || Name==Elements::WAVE_data))
-        Size_Complete=File_Size-(File_Offset+Buffer_Offset+8); //Non standard big files detection
     Header_Fill_Size(Size_Complete+8);
 }
 
@@ -803,6 +944,39 @@ bool File_Riff::BookMark_Needed()
     return true;
 }
 
+//***************************************************************************
+// Helpers
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+void File_Riff::TimeCode_Fill(const Ztring &Name, const Ztring &Value)
+{
+    float64 FrameRate=Retrieve(Stream_Video, 0, Video_FrameRate).To_float64();
+    if (Value.size()==11 && FrameRate)
+    {
+        for (size_t StreamKind=Stream_General+1; StreamKind<Stream_Max; StreamKind++)
+            for (size_t StreamPos=0; StreamPos<Count_Get((stream_t)StreamKind); StreamPos++)
+            {
+                int64u Delay=0;
+                Delay+=(Value[0]-__T('0'))*10*60*60*1000;
+                Delay+=(Value[1]-__T('0'))   *60*60*1000;
+                Delay+=(Value[3]-__T('0'))   *10*60*1000;
+                Delay+=(Value[4]-__T('0'))      *60*1000;
+                Delay+=(Value[6]-__T('0'))      *10*1000;
+                Delay+=(Value[7]-__T('0'))         *1000;
+
+                int64u Frames=0;
+                Frames+=(Value[ 9]-__T('0'))*10;
+                Frames+=(Value[10]-__T('0'));
+                Delay+=float64_int64s((1000/FrameRate)*Frames);
+
+                Fill((stream_t)StreamKind, StreamPos, Fill_Parameter((stream_t)StreamKind, Generic_Delay), Delay);
+                Fill((stream_t)StreamKind, StreamPos, Fill_Parameter((stream_t)StreamKind, Generic_Delay_Source), __T("Container (")+Name+__T(")"));
+                Fill((stream_t)StreamKind, StreamPos, Fill_Parameter((stream_t)StreamKind, Generic_Delay_String4), Value);
+            }
+    }
+}
+
 //***************************************************************************
 // C++
 //***************************************************************************
