@@ -1,17 +1,17 @@
 // File_Id3 - Info for ID3v2 tagged files
-// Copyright (C) 2005-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2005-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -89,7 +89,7 @@ extern const char* Id3v2_PictureType(int8u Type)
 #include "MediaInfo/Tag/File_Id3v2.h"
 #include "ZenLib/ZtringListList.h"
 #include "ZenLib/Utils.h"
-#include "ZenLib/Base64/base64.h"
+#include "base64.h"
 #include <cstring>
 using namespace ZenLib;
 //---------------------------------------------------------------------------
@@ -312,9 +312,6 @@ namespace Elements
 File_Id3v2::File_Id3v2()
 :File__Analyze()
 {
-    //Configuration
-    Buffer_MaximumSize=8*1024*1024;
-    
     //Temp
     Id3v2_Size=0;
 }
@@ -353,19 +350,19 @@ void File_Id3v2::Streams_Fill()
         Ztring Recorded_Date=Year;
         if (!Month.empty())
         {
-            Recorded_Date+=_T('-');
+            Recorded_Date+=__T('-');
             Recorded_Date+=Month;
             if (!Day.empty())
             {
-                Recorded_Date+=_T('-');
+                Recorded_Date+=__T('-');
                 Recorded_Date+=Day;
                 if (!Hour.empty())
                 {
-                    Recorded_Date+=_T(' ');
+                    Recorded_Date+=__T(' ');
                     Recorded_Date+=Hour;
                     if (Minute.empty())
                     {
-                        Recorded_Date+=_T(':');
+                        Recorded_Date+=__T(':');
                         Recorded_Date+=Minute;
                     }
                 }
@@ -398,14 +395,14 @@ void File_Id3v2::FileHeader_Parse()
              | ((Size>>1)&0x3F80)
              | ((Size>>2)&0x1FC000)
              | ((Size>>3)&0x0FE00000);
-    Param_Info(Id3v2_Size);
+    Param_Info1(Id3v2_Size);
     if (ExtendedHeader)
     {
-        Element_Begin("Extended header");
+        Element_Begin1("Extended header");
         int32u Size_Extended;
         Get_B4 (Size_Extended,                                  "Size");
         Skip_XX(Size_Extended,                                  "Extended header");
-        Element_End();
+        Element_End0();
     }
 
     FILLING_BEGIN();
@@ -479,7 +476,7 @@ void File_Id3v2::Header_Parse()
                | ((Size>>1)&0x3F80)
                | ((Size>>2)&0x1FC000)
                | ((Size>>3)&0x0FE00000);
-            Param_Info(Size, " bytes");
+            Param_Info2(Size, " bytes");
         }
         Get_B2 (Flags,                                          "Flags");
         if (Id3v2_Version==3)
@@ -504,6 +501,26 @@ void File_Id3v2::Header_Parse()
         }
     }
 
+    //Hanlding Unsynchronisation
+    if (Unsynchronisation_Global || Unsynchronisation_Frame)
+    {
+        if (Buffer_Offset+(size_t)Element_Offset+Size>Buffer_Size)
+        {
+            Element_WaitForMoreData();
+            return;
+        }
+        for (size_t Element_Offset_Unsynch=0; Element_Offset_Unsynch+2<Element_Offset+Size; Element_Offset_Unsynch++)
+            if (CC2(Buffer+Buffer_Offset+Element_Offset_Unsynch)==0xFF00)
+            {
+                Size++;
+                if (Buffer_Offset+(size_t)Element_Offset+Size>Buffer_Size)
+                {
+                    Element_WaitForMoreData();
+                    return;
+                }
+            }
+    }
+
     //Filling
     Ztring ToShow;
     if (Id3v2_Version==2)
@@ -527,7 +544,7 @@ void File_Id3v2::Data_Parse()
                  | ((DataLength>>1)&0x3F80)
                  | ((DataLength>>2)&0x1FC000)
                  | ((DataLength>>3)&0x0FE00000);
-        Param_Info(DataLength, " bytes");
+        Param_Info2(DataLength, " bytes");
     }
 
     //Unsynchronisation
@@ -572,7 +589,7 @@ void File_Id3v2::Data_Parse()
     }
 
     #define CASE_INFO(_NAME, _DETAIL) \
-        case Elements::_NAME : Element_Info(_DETAIL); _NAME(); break;
+        case Elements::_NAME : Element_Info1(_DETAIL); _NAME(); break;
 
     //Parsing
     Element_Value.clear();
@@ -779,8 +796,8 @@ void File_Id3v2::T___()
     //Exceptions
     if (Element_Code==Elements::TCMP || Element_Code==Elements::TCP)
     {
-        if (Element_Value==_T("0")) Element_Value.clear(); //This is usually set to 0 even if the user did not explicitely indicated something (default)
-        if (Element_Value==_T("1")) Element_Value=_T("Yes");
+        if (Element_Value==__T("0")) Element_Value.clear(); //This is usually set to 0 even if the user did not explicitely indicated something (default)
+        if (Element_Value==__T("1")) Element_Value=__T("Yes");
     }
 
     //Filling
@@ -906,8 +923,8 @@ void File_Id3v2::APIC()
         Get_C3(Image_format,                                    "Image_format");
         switch (Image_format)
         {
-            case 0x504E47 : Mime="image/png";
-            case 0x4A5047 : Mime="image/jpeg";
+            case 0x504E47 : Mime="image/png"; break;
+            case 0x4A5047 : Mime="image/jpeg"; break;
             default       : ;
         }
     }
@@ -917,7 +934,7 @@ void File_Id3v2::APIC()
         Get_ISO_8859_1(Element_Size-Element_Offset, Mime,       "MIME_type");
         Element_Offset=Element_Offset_Real+Mime.size()+1;
     }
-    Get_B1 (PictureType,                                        "Picture_type"); Element_Info(Id3v2_PictureType(PictureType));
+    Get_B1 (PictureType,                                        "Picture_type"); Element_Info1(Id3v2_PictureType(PictureType));
     int64u Element_Offset_Real=Element_Offset;
     switch (Encoding)
     {
@@ -955,24 +972,24 @@ void File_Id3v2::COMM()
     T__X();
 
     //Testing
-         if (Element_Values(0)==_T("iTunes_CDDB_IDs")) return;
-    else if (Element_Values(0)==_T("iTunNORM")) return;
-    else if (Element_Values(0)==_T("iTunSMPB")) return;
-    else if (Element_Values(0)==_T("Songs-DB_Tempo")) return;
-    else if (Element_Values(0)==_T("Songs-DB_Preference")) return;
-    else if (Element_Values(0)==_T("MusicMatch_Tempo")) return;
-    else if (Element_Values(0)==_T("MusicMatch_Mood"))
+         if (Element_Values(0)==__T("iTunes_CDDB_IDs")) return;
+    else if (Element_Values(0)==__T("iTunNORM")) return;
+    else if (Element_Values(0)==__T("iTunSMPB")) return;
+    else if (Element_Values(0)==__T("Songs-DB_Tempo")) return;
+    else if (Element_Values(0)==__T("Songs-DB_Preference")) return;
+    else if (Element_Values(0)==__T("MusicMatch_Tempo")) return;
+    else if (Element_Values(0)==__T("MusicMatch_Mood"))
     {
         if (Retrieve(Stream_General, 0, General_Mood).empty())
-            Element_Values(0)==_T("Mood");
+            Element_Values(0)==__T("Mood");
         else
             return;
     }
-    else if (Element_Values(0)==_T("MusicMatch_Preference")) return;
+    else if (Element_Values(0)==__T("MusicMatch_Preference")) return;
 
     //Filling
     if (Element_Values(0).empty())
-        Element_Values(0)=_T("Comment");
+        Element_Values(0)=__T("Comment");
     Fill_Name();
 }
 
@@ -984,17 +1001,17 @@ void File_Id3v2::RGAD()
     Get_BF4 (Peak_Amplitude,                                    "Peak Amplitude");
     while (Element_Offset+2<=Element_Size)
     {
-        Element_Begin("Gain Adjustement", 2);
+        Element_Begin1("Gain Adjustement");
         int16u Replay_Gain_Adjustment;
         int8u  Name_code;
         bool   Sign_bit;
         BS_Begin();
-        Get_S1 (3, Name_code,                                   "Name code"); Param_Info(Id3v2_RGAD_Name_code[Name_code]);
-        Info_S1(3, Originator_code,                             "Originator code"); Param_Info(Id3v2_RGAD_Originator_code[Originator_code]);
+        Get_S1 (3, Name_code,                                   "Name code"); Param_Info1(Id3v2_RGAD_Name_code[Name_code]);
+        Info_S1(3, Originator_code,                             "Originator code"); Param_Info1(Id3v2_RGAD_Originator_code[Originator_code]);
         Get_SB (Sign_bit,                                       "Sign bit");
-        Get_S2 (9, Replay_Gain_Adjustment,                      "Replay Gain Adjustment"); Param_Info ((Sign_bit?-1:1)*(float)Replay_Gain_Adjustment/10, 1, " dB");
+        Get_S2 (9, Replay_Gain_Adjustment,                      "Replay Gain Adjustment"); Param_Info3 ((Sign_bit?-1:1)*(float)Replay_Gain_Adjustment/10, 1, " dB");
         BS_End();
-        Element_End();
+        Element_End0();
 
         FILLING_BEGIN();
             switch (Name_code)
@@ -1030,8 +1047,8 @@ void File_Id3v2::USLT()
 
     //Filling
     if (!Element_Values(0).empty())
-        Element_Values(1)=Element_Values(0)+MediaInfoLib::Config.Language_Get(_T(": "))+Element_Values(1);
-    Element_Values(0)=_T("Lyrics");
+        Element_Values(1)=Element_Values(0)+MediaInfoLib::Config.Language_Get(__T(": "))+Element_Values(1);
+    Element_Values(0)=__T("Lyrics");
 
     Fill_Name();
 }
@@ -1043,13 +1060,19 @@ void File_Id3v2::TXXX()
 
     //Filling
     if (Element_Values(0).empty())
-        Element_Values(0)=_T("Comment");
+        Element_Values(0)=__T("Comment");
     Fill_Name();
 }
 
 //---------------------------------------------------------------------------
 void File_Id3v2::SYLT()
 {
+    if (Element_Size<6)
+    {
+        Skip_XX(Element_Size,                                   "(Problem)");
+        return;
+    }
+
     int8u Encoding;
     Get_B1 (Encoding,                                           "Text encoding");
     Skip_C3(                                                    "Language");
@@ -1077,7 +1100,7 @@ void File_Id3v2::WXXX()
     if (Element_Values(1).empty())
         return;
     if (Element_Values(0).empty())
-        Element_Values(0)=_T("URL");
+        Element_Values(0)=__T("URL");
     Fill_Name();
 }
 
@@ -1089,9 +1112,9 @@ void File_Id3v2::WXXX()
 void File_Id3v2::Fill_Name()
 {
     Ztring Value=Ztring().From_CC4((int32u)Element_Code);
-    if (MediaInfoLib::Config.CustomMapping_IsPresent(_T("Id3v2"), Value))
+    if (MediaInfoLib::Config.CustomMapping_IsPresent(__T("Id3v2"), Value))
     {
-        Fill(Stream_General, 0, MediaInfoLib::Config.CustomMapping_Get(_T("Id3v2"), Value).To_Local().c_str(), Element_Value);
+        Fill(Stream_General, 0, MediaInfoLib::Config.CustomMapping_Get(__T("Id3v2"), Value).To_Local().c_str(), Element_Value);
         return;
     }
 
@@ -1130,9 +1153,9 @@ void File_Id3v2::Fill_Name()
         case Elements::TCOM : Fill(Stream_General, 0, General_Composer, Element_Value); break;
         case Elements::TCON :
                               {
-                                if (Element_Value.find(_T("("))==0)
-                                    Element_Value=Element_Value.SubString(_T("("), _T(")")); //Replace (nn) by nn
-                                if (Element_Value==_T("0") || Element_Value==_T("255"))
+                                if (Element_Value.find(__T("("))==0)
+                                    Element_Value=Element_Value.SubString(__T("("), __T(")")); //Replace (nn) by nn
+                                if (Element_Value==__T("0") || Element_Value==__T("255"))
                                     Element_Value.clear();
                                 Fill(Stream_General, 0, General_Genre, Element_Value);
                               }
@@ -1150,7 +1173,7 @@ void File_Id3v2::Fill_Name()
         case Elements::TDRC : Normalize_Date(Element_Value); Fill(Stream_General, 0, General_Recorded_Date, Element_Value); break;
         case Elements::TDRL : Normalize_Date(Element_Value); Fill(Stream_General, 0, General_Released_Date, Element_Value); break;
         case Elements::TDTG : Normalize_Date(Element_Value); Fill(Stream_General, 0, General_Tagged_Date, Element_Value); break;
-        case Elements::TENC : Fill(Stream_General, 0, General_Encoded_Library, Element_Value); break;
+        case Elements::TENC : Fill(Stream_General, 0, General_EncodedBy, Element_Value); break;
         case Elements::TEXT : Fill(Stream_General, 0, General_Lyricist, Element_Value); break;
         case Elements::TFLT : Fill(Stream_General, 0, "File type", Element_Value); break;
         case Elements::TIM  :
@@ -1181,7 +1204,7 @@ void File_Id3v2::Fill_Name()
         case Elements::TPE4 : Fill(Stream_General, 0, General_RemixedBy, Element_Value); break;
         case Elements::TPOS :
                               {
-                                ZtringList List; List.Separator_Set(0, _T("/")); List.Write(Element_Value);
+                                ZtringList List; List.Separator_Set(0, __T("/")); List.Write(Element_Value);
                                 if (!List(0).empty())
                                     Fill(Stream_General, 0, General_Part_Position, List(0));
                                 if (!List(1).empty())
@@ -1192,7 +1215,7 @@ void File_Id3v2::Fill_Name()
         case Elements::TPUB : Fill(Stream_General, 0, General_Publisher, Element_Value); break;
         case Elements::TRCK :
                               {
-                                ZtringList List; List.Separator_Set(0, _T("/")); List.Write(Element_Value);
+                                ZtringList List; List.Separator_Set(0, __T("/")); List.Write(Element_Value);
                                 if (!List(0).empty())
                                     Fill(Stream_General, 0, General_Track_Position, List(0));
                                 if (!List(1).empty())
@@ -1209,22 +1232,22 @@ void File_Id3v2::Fill_Name()
         case Elements::TSOP : Fill(Stream_General, 0, General_Performer_Sort, Element_Value); break;
         case Elements::TSOT : Fill(Stream_General, 0, General_Track_Sort, Element_Value); break;
         case Elements::TSRC : Fill(Stream_General, 0, General_ISRC, Element_Value); break;
-        case Elements::TSSE : Fill(Stream_General, 0, General_Encoded_Library_Settings, Element_Value); break;
+        case Elements::TSSE : Fill(Stream_General, 0, General_Encoded_Library, Element_Value); break;
         case Elements::TSST : Fill(Stream_General, 0, "Set subtitle", Element_Value); break;
-        case Elements::TXXX : if (Element_Values(0)==_T("AccurateRipResult"))      ;
-                         else if (Element_Values(0)==_T("AccurateRipDiscID"))      ;
-                         else if (Element_Values(0)==_T("CT_GAPLESS_DATA"))        ;
-                         else if (Element_Values(0)==_T("DISCNUMBER"))             Fill(Stream_General, 0, General_Part_Position,           Element_Values(1), true);
-                         else if (Element_Values(0)==_T("DISCTOTAL"))              Fill(Stream_General, 0, General_Part_Position_Total,     Element_Values(1), true);
-                         else if (Element_Values(0)==_T("first_played_timestamp")) Fill(Stream_General, 0, General_Played_First_Date,       Ztring().Date_From_Milliseconds_1601(Element_Values(1).To_int64u()/10000));
-                         else if (Element_Values(0)==_T("last_played_timestamp"))  Fill(Stream_General, 0, General_Played_Last_Date,        Ztring().Date_From_Milliseconds_1601(Element_Values(1).To_int64u()/10000));
-                         else if (Element_Values(0)==_T("play_count"))             Fill(Stream_General, 0, General_Played_Count,            Element_Values(1).To_int64u());
-                         else if (Element_Values(0)==_T("added_timestamp"))        Fill(Stream_General, 0, General_Added_Date,              Ztring().Date_From_Milliseconds_1601(Element_Values(1).To_int64u()/10000));
-                         else if (Element_Values(0)==_T("replaygain_album_gain"))  Fill(Stream_General, 0, General_Album_ReplayGain_Gain,   Element_Values(1).To_float64(), 2, true);
-                         else if (Element_Values(0)==_T("replaygain_album_peak"))  Fill(Stream_General, 0, General_Album_ReplayGain_Peak,   Element_Values(1).To_float64(), 6, true);
-                         else if (Element_Values(0)==_T("replaygain_track_gain"))  Fill(Stream_Audio,   0, Audio_ReplayGain_Gain,           Element_Values(1).To_float64(), 2, true);
-                         else if (Element_Values(0)==_T("replaygain_track_peak"))  Fill(Stream_Audio,   0, Audio_ReplayGain_Peak,           Element_Values(1).To_float64(), 6, true);
-                         else if (Element_Values(0)==_T("TRACKTOTAL"))             Fill(Stream_General, 0, General_Track_Position_Total,    Element_Values(1), true);
+        case Elements::TXXX : if (Element_Values(0)==__T("AccurateRipResult"))      ;
+                         else if (Element_Values(0)==__T("AccurateRipDiscID"))      ;
+                         else if (Element_Values(0)==__T("CT_GAPLESS_DATA"))        ;
+                         else if (Element_Values(0)==__T("DISCNUMBER"))             Fill(Stream_General, 0, General_Part_Position,           Element_Values(1), true);
+                         else if (Element_Values(0)==__T("DISCTOTAL"))              Fill(Stream_General, 0, General_Part_Position_Total,     Element_Values(1), true);
+                         else if (Element_Values(0)==__T("first_played_timestamp")) Fill(Stream_General, 0, General_Played_First_Date,       Ztring().Date_From_Milliseconds_1601(Element_Values(1).To_int64u()/10000));
+                         else if (Element_Values(0)==__T("last_played_timestamp"))  Fill(Stream_General, 0, General_Played_Last_Date,        Ztring().Date_From_Milliseconds_1601(Element_Values(1).To_int64u()/10000));
+                         else if (Element_Values(0)==__T("play_count"))             Fill(Stream_General, 0, General_Played_Count,            Element_Values(1).To_int64u());
+                         else if (Element_Values(0)==__T("added_timestamp"))        Fill(Stream_General, 0, General_Added_Date,              Ztring().Date_From_Milliseconds_1601(Element_Values(1).To_int64u()/10000));
+                         else if (Element_Values(0)==__T("replaygain_album_gain"))  Fill(Stream_General, 0, General_Album_ReplayGain_Gain,   Element_Values(1).To_float64(), 2, true);
+                         else if (Element_Values(0)==__T("replaygain_album_peak"))  Fill(Stream_General, 0, General_Album_ReplayGain_Peak,   Element_Values(1).To_float64(), 6, true);
+                         else if (Element_Values(0)==__T("replaygain_track_gain"))  Fill(Stream_Audio,   0, Audio_ReplayGain_Gain,           Element_Values(1).To_float64(), 2, true);
+                         else if (Element_Values(0)==__T("replaygain_track_peak"))  Fill(Stream_Audio,   0, Audio_ReplayGain_Peak,           Element_Values(1).To_float64(), 6, true);
+                         else if (Element_Values(0)==__T("TRACKTOTAL"))             Fill(Stream_General, 0, General_Track_Position_Total,    Element_Values(1), true);
                          else
                             Fill(Stream_General, 0, Element_Values(0).To_UTF8().c_str(), Element_Values(1));
                          break;
@@ -1243,7 +1266,7 @@ void File_Id3v2::Fill_Name()
         case Elements::WXXX : Fill(Stream_General, 0, Element_Values(0).To_UTF8().c_str(), Element_Values(1)); break;
         case Elements::BUF  : break;
         case Elements::CNT  : break;
-        case Elements::COM  : Fill(Stream_General, 0, "Comment", Element_Value); break;
+        case Elements::COM  : Fill(Stream_General, 0, Element_Values(0).To_UTF8().c_str(), Element_Values(1)); break;
         case Elements::CRA  : break;
         case Elements::CRM  : break;
         case Elements::EQU  : break;
@@ -1264,9 +1287,9 @@ void File_Id3v2::Fill_Name()
         case Elements::TCM  : Fill(Stream_General, 0, "Composer", Element_Value); break;
         case Elements::TCO  :
                               {
-                                if (Element_Value.find(_T("("))==0)
-                                    Element_Value=Element_Value.SubString(_T("("), _T(")")); //Replace (nn) by nn
-                                if (Element_Value==_T("0") || Element_Value==_T("255"))
+                                if (Element_Value.find(__T("("))==0)
+                                    Element_Value=Element_Value.SubString(__T("("), __T(")")); //Replace (nn) by nn
+                                if (Element_Value==__T("0") || Element_Value==__T("255"))
                                     Element_Value.clear();
                                 if (!Element_Value.empty())
                                     Fill(Stream_General, 0, General_Genre, Element_Value);
@@ -1291,7 +1314,7 @@ void File_Id3v2::Fill_Name()
         case Elements::TP4  : Fill(Stream_General, 0, "RemixedBy", Element_Value); break;
         case Elements::TPA  :
                               {
-                                ZtringList List; List.Separator_Set(0, _T("/")); List.Write(Element_Value);
+                                ZtringList List; List.Separator_Set(0, __T("/")); List.Write(Element_Value);
                                 if (!List(0).empty())
                                     Fill(Stream_General, 0, General_Part_Position, List(0));
                                 if (!List(1).empty())
@@ -1303,7 +1326,7 @@ void File_Id3v2::Fill_Name()
         case Elements::TRD  : Normalize_Date(Element_Value); Fill(Stream_General, 0, "Recorded_Date", Element_Value); break;
         case Elements::TRK  :
                               {
-                                ZtringList List; List.Separator_Set(0, _T("/")); List.Write(Element_Value);
+                                ZtringList List; List.Separator_Set(0, __T("/")); List.Write(Element_Value);
                                 if (!List(0).empty())
                                     Fill(Stream_General, 0, General_Track_Position, List(0));
                                 if (!List(1).empty())
@@ -1319,7 +1342,7 @@ void File_Id3v2::Fill_Name()
         case Elements::TXX  : Fill(Stream_General, 0, Element_Values(0).To_UTF8().c_str(), Element_Values(1)); break;
         case Elements::TYE  : Year=Element_Value; break;
         case Elements::UFI  : Fill(Stream_Audio,   0, "UID", Element_Value); break;
-        case Elements::ULT  : break;
+        case Elements::ULT  : Fill(Stream_General, 0, Element_Values(0).To_UTF8().c_str(), Element_Values(1)); break;
         case Elements::WAF  : break;
         case Elements::WAR  : Fill(Stream_General, 0, General_Service_Url, Element_Value); break;
         case Elements::WAS  : Fill(Stream_General, 0, "Official audio source webpage", Element_Value); break;
@@ -1336,8 +1359,8 @@ void File_Id3v2::Normalize_Date(Ztring& Date)
 {
     if (Date.size()<=8)
         return; //Format unknown
-    Date[8]=_T(' '); //could be "T"
-    Date=Ztring(_T("UTC "))+Date; //Id3v2 specify a UTC date
+    Date[8]=__T(' '); //could be "T"
+    Date=Ztring(__T("UTC "))+Date; //Id3v2 specify a UTC date
 }
 
 //***************************************************************************
