@@ -1,17 +1,17 @@
 // File_Eia708 - Info for EIA-708 files
-// Copyright (C) 2009-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2009-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -51,6 +51,7 @@ File_Eia708::File_Eia708()
 {
     //Config
     PTS_DTS_Needed=true;
+    MustSynchronize=true;
 
     //In
     cc_type=(int8u)-1;
@@ -78,18 +79,50 @@ File_Eia708::~File_Eia708()
 //---------------------------------------------------------------------------
 void File_Eia708::Streams_Fill()
 {
-    Stream_Prepare(Stream_Text);
-    Fill(Stream_Text, 0, Text_ID, 1); //TODO: fill with the exact service
-    Fill(Stream_Text, 0, Text_Format, "EIA-708");
-    Fill(Stream_Text, 0, Text_StreamSize, 0);
-    Fill(Stream_Text, 0, Text_BitRate_Mode, "CBR");
+    if (Config->File_Eia708_DisplayEmptyStream_Get() && Streams.size()<2)
+        Streams.resize(2);
+
+    for (size_t Pos=0; Pos<Streams.size(); Pos++)
+        if (Streams[Pos] || (Pos && Pos<2 && Config->File_Eia708_DisplayEmptyStream_Get()))
+        {
+            Stream_Prepare(Stream_Text);
+            Fill(Stream_Text, StreamPos_Last, Text_ID, Pos);
+            Fill(Stream_Text, StreamPos_Last, Text_Format, "EIA-708");
+            Fill(Stream_Text, StreamPos_Last, Text_StreamSize, 0);
+            Fill(Stream_Text, StreamPos_Last, Text_BitRate_Mode, "CBR");
+        }
 }
 
 //---------------------------------------------------------------------------
 void File_Eia708::Streams_Finish()
 {
-    if (!HasContent)
-        Fill(Stream_Text, 0, "ContentInfo", "No content");
+}
+
+//***************************************************************************
+// Buffer - Synchro
+//***************************************************************************
+
+//---------------------------------------------------------------------------
+bool File_Eia708::Synchronize()
+{
+    if (cc_type!=3)
+        return false; //Waiting for sync from underlying layer
+
+    if (!Status[IsAccepted])
+        Accept("EIA-708");
+
+    //Synched is OK
+    return true;
+}
+
+//---------------------------------------------------------------------------
+bool File_Eia708::Synched_Test()
+{
+    if (cc_type==4) //Magic value saying that the buffer must be kept (this is only a point of synchro from the undelying layer)
+        Buffer_Offset=Buffer_Size; //Sync point
+
+    //We continue
+    return true;
 }
 
 //***************************************************************************
@@ -105,29 +138,11 @@ void File_Eia708::Read_Buffer_Init()
 void File_Eia708::Read_Buffer_Continue()
 {
     FrameInfo.PTS=FrameInfo.DTS;
-
-    if (!Status[IsAccepted])
-    {
-        if (cc_type!=3)
-        {
-            Skip_B2(                                                "Waiting for header");
-            return;
-        }
-
-        Accept("EIA-708");
-
-        //Forcing detection even if this is empty caption (option)
-        if (Config->File_Eia708_DisplayEmptyStream_Get()) //TODO: separate services
-            Fill("EIA-708");
-    }
 }
 
 //---------------------------------------------------------------------------
 void File_Eia708::Read_Buffer_Unsynched()
 {
-    if (cc_type==4) //Magic value saying that the buffer must be kept (this is only a point of synchro from the undelying layer)
-        return;
-
     for (int8u service_number=1; service_number<Streams.size(); service_number++)
         if (Streams[service_number])
         {
@@ -140,7 +155,7 @@ void File_Eia708::Read_Buffer_Unsynched()
                     {
                         for (size_t Pos_X=0; Pos_X<Window->Minimal.CC[Pos_Y].size(); Pos_X++)
                         {
-                            Window->Minimal.CC[Pos_Y][Pos_X].Value=L' ';;
+                            Window->Minimal.CC[Pos_Y][Pos_X].Value=L' ';
                             Window->Minimal.CC[Pos_Y][Pos_X].Attribute=0;
                         }
                     }
@@ -172,7 +187,7 @@ void File_Eia708::Header_Parse()
     Get_S1(6, packet_size,                                      "packet_size_code");
     BS_End();
 
-    Header_Fill_Code(0, _T("DTVCC packet"));
+    Header_Fill_Code(0, __T("DTVCC packet"));
     Header_Fill_Size(packet_size==0?128:(packet_size*2));
 }
 
@@ -195,9 +210,9 @@ void File_Eia708::Data_Parse()
 
         if (service_number)
         {
-            Element_Begin("Service Block Packet", block_size);
+            Element_Begin1("Service Block Packet");
             Service();
-            Element_End();
+            Element_End0();
         }
     }
 }
@@ -229,15 +244,15 @@ void File_Eia708::Service()
             case 0x0C : FF(); break; //Form Feed
             case 0x0D : CR(); break; //Carriage Return
             case 0x0E : HCR(); break; //Horizontal Carriage Return
-            case 0x01 : 
-            case 0x02 : 
-            case 0x04 : 
-            case 0x05 : 
-            case 0x06 : 
-            case 0x07 : 
-            case 0x09 : 
-            case 0x0A : 
-            case 0x0B : 
+            case 0x01 :
+            case 0x02 :
+            case 0x04 :
+            case 0x05 :
+            case 0x06 :
+            case 0x07 :
+            case 0x09 :
+            case 0x0A :
+            case 0x0B :
             case 0x0F : break; //1-byte long undefined code
             case 0x10 : //EXT1
                         {
@@ -447,7 +462,7 @@ void File_Eia708::Service()
             case 0x7E : Character_Fill(L'~'     ); break;
             case 0x7F : Character_Fill(L'\x266A'); break;
             //CEA-708-D, Section 7.1.5 (C1)
-            case 0x80 : //CW0 
+            case 0x80 : //CW0
             case 0x81 : //CW1
             case 0x82 : //CW2
             case 0x83 : //CW3
@@ -467,7 +482,7 @@ void File_Eia708::Service()
             case 0x90 : SPA(); Pos+=2; break; //SetPenAttributes
             case 0x91 : SPC(); Pos+=3; break; //SetPenColor
             case 0x92 : SPL(); Pos+=2; break; //SetPenLocation
-            case 0x97 : SWA(); Pos+=2; break; //SetWindowAttributes
+            case 0x97 : SWA(); Pos+=5; break; //SetWindowAttributes
             case 0x98 : //DF0
             case 0x99 : //DF1
             case 0x9A : //DF2
@@ -599,7 +614,7 @@ void File_Eia708::ETX()
 //Backspace
 void File_Eia708::BS()
 {
-    Param_Info("Backspace");
+    Param_Info1("Backspace");
 
     int8u WindowID=Streams[service_number]->WindowID;
     if (WindowID==(int8u)-1)
@@ -631,7 +646,7 @@ void File_Eia708::BS()
 //Form Feed
 void File_Eia708::FF()
 {
-    Param_Info("Form Feed");
+    Param_Info1("Form Feed");
 
     int8u WindowID=Streams[service_number]->WindowID;
     if (WindowID==(int8u)-1)
@@ -670,7 +685,7 @@ void File_Eia708::FF()
 //Carriage return
 void File_Eia708::CR()
 {
-    Param_Info("Carriage return");
+    Param_Info1("Carriage return");
 
     int8u WindowID=Streams[service_number]->WindowID;
     if (WindowID==(int8u)-1)
@@ -686,7 +701,7 @@ void File_Eia708::CR()
         //Rolling up window
         for (int8u Pos_Y=0; Pos_Y<Window->row_count-1; Pos_Y++)
             Window->Minimal.CC[Pos_Y]=Window->Minimal.CC[Pos_Y+1];
-        
+
         //SetPenLocation
         y=Window->row_count-1;
 
@@ -696,7 +711,7 @@ void File_Eia708::CR()
 
         if (Window->visible)
         {
-            //Updating global area 
+            //Updating global area
             for (int8u Pos_Y=0; Pos_Y<Window->row_count; Pos_Y++)
                 for (int8u Pos_X=0; Pos_X<Window->column_count; Pos_X++)
                     if (Window->Minimal.Window_y+Pos_Y<(int8u)Streams[service_number]->Minimal.CC.size() && Window->Minimal.Window_x+Pos_X<(int8u)Streams[service_number]->Minimal.CC[Window->Minimal.Window_y+Pos_Y].size())
@@ -717,7 +732,7 @@ void File_Eia708::CR()
 //Horizontal Carriage Return
 void File_Eia708::HCR()
 {
-    Param_Info("Horizontal Carriage Return");
+    Param_Info1("Horizontal Carriage Return");
 
     int8u WindowID=Streams[service_number]->WindowID;
     if (WindowID==(int8u)-1)
@@ -726,7 +741,7 @@ void File_Eia708::HCR()
     if (Window==NULL)
         return; //Must wait for the corresponding DFx
     int8u y=Window->Minimal.y;
-    
+
     for (int8u Pos_X=0; Pos_X<Window->column_count; Pos_X++)
     {
         //Clearing window
@@ -735,7 +750,7 @@ void File_Eia708::HCR()
         if (Window->visible)
         {
             //Clearing global area
-            if (Window->Minimal.Window_y+y<(int8u)Streams[service_number]->Minimal.CC.size() && Window->Minimal.Window_x+y<(int8u)Streams[service_number]->Minimal.CC[Window->Minimal.Window_y+Window->Minimal.y].size())
+            if (Window->Minimal.Window_y+y<(int8u)Streams[service_number]->Minimal.CC.size() && Window->Minimal.Window_x+Pos_X<(int8u)Streams[service_number]->Minimal.CC[Window->Minimal.Window_y+Window->Minimal.y].size())
                 Streams[service_number]->Minimal.CC[Window->Minimal.Window_y+y][Window->Minimal.Window_x+Pos_X]=character();
 
             //Has changed
@@ -752,7 +767,7 @@ void File_Eia708::HCR()
 //SetCurrentWindow
 void File_Eia708::CWx(int8u WindowID)
 {
-    Param_Info("SetCurrentWindow"); Param_Info(WindowID);
+    Param_Info1("SetCurrentWindow"); Param_Info1(WindowID);
 
     Streams[service_number]->WindowID=WindowID;
 }
@@ -761,13 +776,13 @@ void File_Eia708::CWx(int8u WindowID)
 //ClearWindows
 void File_Eia708::CLW()
 {
-    Param_Info("ClearWindows");
+    Param_Info1("ClearWindows");
 
     int8u Save_WindowID=Streams[service_number]->WindowID;
     bool  Save_StandAloneCommand=StandAloneCommand;
     StandAloneCommand=false;
 
-    Element_Begin("ClearWindows");
+    Element_Begin1("ClearWindows");
     BS_Begin();
     int8u WindowID=8;
     bool HasChanged_=false;
@@ -775,13 +790,13 @@ void File_Eia708::CLW()
     {
         WindowID--;
         bool IsSet;
-        Get_SB (   IsSet,                                       Ztring(_T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
+        Get_SB (   IsSet,                                       Ztring(__T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
 
         if (IsSet)
         {
             Streams[service_number]->WindowID=WindowID;
             window* Window=Streams[service_number]->Windows[WindowID];
-            
+
             //ClearWindow is like Form Feed
             FF();
 
@@ -795,11 +810,11 @@ void File_Eia708::CLW()
     }
     while (WindowID>0);
     BS_End();
-    Element_End();
+    Element_End0();
 
     Streams[service_number]->WindowID=Save_WindowID;
     StandAloneCommand=Save_StandAloneCommand;
-    
+
     if (HasChanged_)
     {
         //Has changed
@@ -811,13 +826,13 @@ void File_Eia708::CLW()
 //DisplayWindows
 void File_Eia708::DSW()
 {
-    Param_Info("DisplayWindows");
+    Param_Info1("DisplayWindows");
 
     int8u Save_WindowID=Streams[service_number]->WindowID;
     bool  Save_StandAloneCommand=StandAloneCommand;
     StandAloneCommand=false;
 
-    Element_Begin("DisplayWindows");
+    Element_Begin1("DisplayWindows");
     BS_Begin();
     int8u WindowID=8;
     bool HasChanged_=false;
@@ -825,12 +840,12 @@ void File_Eia708::DSW()
     {
         WindowID--;
         bool IsSet;
-        Get_SB (   IsSet,                                       Ztring(_T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
+        Get_SB (   IsSet,                                       Ztring(__T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
 
         if (IsSet)
         {
             window* Window=Streams[service_number]->Windows[WindowID];
-            
+
             if (Window && !Window->visible)
             {
                 Window->visible=true;
@@ -851,11 +866,11 @@ void File_Eia708::DSW()
     }
     while (WindowID>0);
     BS_End();
-    Element_End();
+    Element_End0();
 
     Streams[service_number]->WindowID=Save_WindowID;
     StandAloneCommand=Save_StandAloneCommand;
-    
+
     if (HasChanged_)
     {
         //Has changed
@@ -867,13 +882,13 @@ void File_Eia708::DSW()
 //HideWindows
 void File_Eia708::HDW()
 {
-    Param_Info("HideWindows");
+    Param_Info1("HideWindows");
 
     int8u Save_WindowID=Streams[service_number]->WindowID;
     bool  Save_StandAloneCommand=StandAloneCommand;
     StandAloneCommand=false;
 
-    Element_Begin("HideWindows");
+    Element_Begin1("HideWindows");
     BS_Begin();
     int8u WindowID=8;
     bool HasChanged_=false;
@@ -882,12 +897,12 @@ void File_Eia708::HDW()
         WindowID--;
 
         bool IsSet;
-        Get_SB (   IsSet,                                       Ztring(_T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
+        Get_SB (   IsSet,                                       Ztring(__T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
 
         if (IsSet)
         {
             window* Window=Streams[service_number]->Windows[WindowID];
-            
+
             if (Window && Window->visible)
             {
                 Window->visible=false;
@@ -911,11 +926,11 @@ void File_Eia708::HDW()
     }
     while (WindowID>0);
     BS_End();
-    Element_End();
+    Element_End0();
 
     Streams[service_number]->WindowID=Save_WindowID;
     StandAloneCommand=Save_StandAloneCommand;
-    
+
     if (HasChanged_)
     {
         //Has changed
@@ -927,13 +942,13 @@ void File_Eia708::HDW()
 //ToggleWindows
 void File_Eia708::TGW()
 {
-    Param_Info("ToggleWindows");
+    Param_Info1("ToggleWindows");
 
     int8u Save_WindowID=Streams[service_number]->WindowID;
     bool  Save_StandAloneCommand=StandAloneCommand;
     StandAloneCommand=false;
 
-    Element_Begin("ToggleWindows");
+    Element_Begin1("ToggleWindows");
     BS_Begin();
     int8u WindowID=8;
     bool HasChanged_=false;
@@ -941,12 +956,12 @@ void File_Eia708::TGW()
     {
         WindowID--;
         bool IsSet;
-        Get_SB (   IsSet,                                       Ztring(_T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
+        Get_SB (   IsSet,                                       Ztring(__T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
 
         if (IsSet)
         {
             window* Window=Streams[service_number]->Windows[WindowID];
-            
+
             if (Window)
             {
                 Window->visible=!Window->visible;
@@ -967,11 +982,11 @@ void File_Eia708::TGW()
     }
     while (WindowID>0);
     BS_End();
-    Element_End();
+    Element_End0();
 
     Streams[service_number]->WindowID=Save_WindowID;
     StandAloneCommand=Save_StandAloneCommand;
-    
+
     if (HasChanged_)
     {
         //Has changed
@@ -983,7 +998,7 @@ void File_Eia708::TGW()
 //DeleteWindows
 void File_Eia708::DLW()
 {
-    Param_Info("DeleteWindows");
+    Param_Info1("DeleteWindows");
 
     int8u Save_WindowID=Streams[service_number]->WindowID;
     bool  Save_StandAloneCommand=StandAloneCommand;
@@ -992,7 +1007,7 @@ void File_Eia708::DLW()
     //Bug in some files
     bool Bug_WindowOffset=false;
 
-    Element_Begin("DeleteWindows");
+    Element_Begin1("DeleteWindows");
     BS_Begin();
     int8u WindowID=8;
     bool HasChanged_=false;
@@ -1000,7 +1015,7 @@ void File_Eia708::DLW()
     {
         WindowID--;
         bool IsSet;
-        Get_SB (   IsSet,                                       Ztring(_T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
+        Get_SB (   IsSet,                                       Ztring(__T("window ")+Ztring::ToZtring(WindowID)).To_Local().c_str());
 
         //Bug in some files
         if (IsSet && WindowID==1 && Streams[service_number]->Windows[0]!=NULL && Streams[service_number]->Windows[1]==NULL) //Mix between Windows 0 and 1
@@ -1039,11 +1054,11 @@ void File_Eia708::DLW()
     }
     while (WindowID>0);
     BS_End();
-    Element_End();
+    Element_End0();
 
     Streams[service_number]->WindowID=Save_WindowID;
     StandAloneCommand=Save_StandAloneCommand;
-    
+
     if (HasChanged_)
     {
         //Has changed
@@ -1055,17 +1070,17 @@ void File_Eia708::DLW()
 //Delay
 void File_Eia708::DLY()
 {
-    Param_Info("Delay");
-    Element_Begin("Delay");
+    Param_Info1("Delay");
+    Element_Begin1("Delay");
     Skip_B1(                        "tenths of seconds");
-    Element_End();
+    Element_End0();
 }
 
 //---------------------------------------------------------------------------
 //Delay Cancel
 void File_Eia708::DLC()
 {
-    Param_Info("Delay Cancel");
+    Param_Info1("Delay Cancel");
 }
 
 //---------------------------------------------------------------------------
@@ -1073,15 +1088,15 @@ void File_Eia708::DLC()
 void File_Eia708::RST()
 {
     //TODO: Should clear all buffers
-    Param_Info("Reset");
+    Param_Info1("Reset");
 }
 
 //---------------------------------------------------------------------------
 //Set Pen Attributes
 void File_Eia708::SPA()
 {
-    Param_Info("Set Pen Attributes");
-    Element_Begin("Set Pen Attributes");
+    Param_Info1("Set Pen Attributes");
+    Element_Begin1("Set Pen Attributes");
     BS_Begin();
     Skip_S1(4,                                                  "text tag");
     Skip_S1(2,                                                  "offset");
@@ -1091,15 +1106,15 @@ void File_Eia708::SPA()
     Skip_S1(3,                                                  "edge type");
     Skip_S1(3,                                                  "font style");
     BS_End();
-    Element_End();
+    Element_End0();
 }
 
 //---------------------------------------------------------------------------
 //Set Pen Color
 void File_Eia708::SPC()
 {
-    Param_Info("Set Pen Color");
-    Element_Begin("Set Pen Color");
+    Param_Info1("Set Pen Color");
+    Element_Begin1("Set Pen Color");
     BS_Begin();
     Skip_S1(2,                                                  "foreground opacity");
     Skip_S1(2,                                                  "foreground red");
@@ -1115,15 +1130,15 @@ void File_Eia708::SPC()
     Skip_S1(2,                                                  "edge green");
     Skip_S1(2,                                                  "edge blue");
     BS_End();
-    Element_End();
+    Element_End0();
 }
 
 //---------------------------------------------------------------------------
 //SetPenLocation
 void File_Eia708::SPL()
 {
-    Param_Info("SetPenLocation");
-    Element_Begin("SetPenLocation");
+    Param_Info1("SetPenLocation");
+    Element_Begin1("SetPenLocation");
     int8u row, column;
     BS_Begin();
     Mark_0();
@@ -1135,7 +1150,7 @@ void File_Eia708::SPL()
     Mark_0();
     Get_S1 (6, column,                                          "column");
     BS_End();
-    Element_End();
+    Element_End0();
 
     int8u WindowID=Streams[service_number]->WindowID;
     if (WindowID==(int8u)-1)
@@ -1144,16 +1159,25 @@ void File_Eia708::SPL()
     if (Window==NULL)
         return; //Must wait for the corresponding DFx
 
-    Window->Minimal.x=column;
-    Window->Minimal.y=row;
+    if (row<Window->Minimal.CC.size() && column<Window->Minimal.CC[Window->Minimal.y].size())
+    {
+        Window->Minimal.x=column;
+        Window->Minimal.y=row;
+    }
+    else
+    {
+        // There is a problem, resetting pen location
+        Window->Minimal.x=0;
+        Window->Minimal.y=0;
+    }
 }
 
 //---------------------------------------------------------------------------
 //SetWindowAttributes
 void File_Eia708::SWA()
 {
-    Param_Info("SetWindowAttributes");
-    Element_Begin("SetWindowAttributes");
+    Param_Info1("SetWindowAttributes");
+    Element_Begin1("SetWindowAttributes");
     BS_Begin();
     Skip_S1(2,                                                  "fill opacity");
     Skip_S1(2,                                                  "fill red");
@@ -1171,21 +1195,21 @@ void File_Eia708::SWA()
     Skip_S1(4,                                                  "effect speed");
     Skip_S1(2,                                                  "effect direction");
     Skip_S1(2,                                                  "display effect");
-    Mark_0();
-    Mark_0();
+    Mark_0_NoTrustError();
+    Mark_0_NoTrustError();
     Skip_S1(2,                                                  "edge red");
     Skip_S1(2,                                                  "edge green");
     Skip_S1(2,                                                  "edge blue");
     BS_End();
-    Element_End();
+    Element_End0();
 }
 
 //---------------------------------------------------------------------------
 //DefineWindow
 void File_Eia708::DFx(int8u WindowID)
 {
-    Param_Info("DefineWindow"); Param_Info(WindowID);
-    Element_Begin("DefineWindow");
+    Param_Info1("DefineWindow"); Param_Info1(WindowID);
+    Element_Begin1("DefineWindow");
     int8u anchor_vertical, anchor_horizontal, anchor_point, row_count, column_count;
     bool visible, relative_positioning;
     BS_Begin();
@@ -1208,7 +1232,7 @@ void File_Eia708::DFx(int8u WindowID)
     Skip_S1(4,                                                  "window style");
     Skip_S1(2,                                                  "pen style ID");
     BS_End();
-    Element_End();
+    Element_End0();
 
     Streams[service_number]->WindowID=WindowID;
     if (Streams[service_number]->Windows[WindowID]==NULL)
@@ -1274,9 +1298,30 @@ void File_Eia708::DFx(int8u WindowID)
     Window->column_count=column_count+1;
     Window->Minimal.x=0;
     Window->Minimal.y=0;
-    Window->Minimal.CC.resize(row_count+1);
-    for (int8u Pos_Y=0; Pos_Y<row_count+1; Pos_Y++)
-        Window->Minimal.CC[Pos_Y].resize(column_count+1);
+
+    if (Window->row_count>12)
+    {
+        Window->row_count=12; //Limitation of specifications
+    }
+    if (AspectRatio && Window->column_count>(int8u)(24*AspectRatio))
+    {
+        Window->column_count=(int8u)(24*AspectRatio); //Limitation of specifications
+    }
+    Window->Minimal.CC.resize(Window->row_count);
+    for (int8u Pos_Y=0; Pos_Y<Window->row_count; Pos_Y++)
+        Window->Minimal.CC[Pos_Y].resize(Window->column_count);
+
+    if (Window->row_count>12)
+    {
+        Window->row_count=12; //Limitation of specifications
+    }
+    if (AspectRatio && Window->column_count>(int8u)(24*AspectRatio))
+    {
+        Window->column_count=(int8u)(24*AspectRatio); //Limitation of specifications
+    }
+    Window->Minimal.CC.resize(Window->row_count);
+    for (int8u Pos_Y=0; Pos_Y<Window->row_count; Pos_Y++)
+        Window->Minimal.CC[Pos_Y].resize(Window->column_count);
 }
 
 //***************************************************************************
@@ -1286,7 +1331,7 @@ void File_Eia708::DFx(int8u WindowID)
 //---------------------------------------------------------------------------
 void File_Eia708::Character_Fill(wchar_t Character)
 {
-    Param_Info(Ztring(1, Character));
+    Param_Info1(Ztring(1, Character));
 
     int8u WindowID=Streams[service_number]->WindowID;
     if (WindowID==(int8u)-1)
@@ -1319,15 +1364,9 @@ void File_Eia708::Character_Fill(wchar_t Character)
         x++;
         Window->Minimal.x=x;
     }
-    
+
     if (!HasContent)
         HasContent=true;
-    if (!Status[IsFilled]) //TODO: separate services
-    {
-        Fill("EIA-708");
-        if (MediaInfoLib::Config.ParseSpeed_Get()<1)
-            Finish("EIA-708");
-    }
 }
 
 //---------------------------------------------------------------------------
