@@ -1,17 +1,17 @@
 // File_Vc1 - Info for VC-1 files
-// Copyright (C) 2007-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2007-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// under the terms of the GNU Library General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
 // any later version.
 //
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Lesser General Public License for more details.
+// GNU Library General Public License for more details.
 //
-// You should have received a copy of the GNU Lesser General Public License
+// You should have received a copy of the GNU Library General Public License
 // along with this library. If not, see <http://www.gnu.org/licenses/>.
 //
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -48,6 +48,7 @@
 #if MEDIAINFO_EVENTS
     #include "MediaInfo/MediaInfo_Config_MediaInfo.h"
     #include "MediaInfo/MediaInfo_Events.h"
+    #include "MediaInfo/MediaInfo_Events_Internal.h"
 #endif //MEDIAINFO_EVENTS
 using namespace std;
 using namespace ZenLib;
@@ -64,10 +65,10 @@ namespace MediaInfoLib
 //---------------------------------------------------------------------------
 const char* Vc1_Profile[]=
 {
-    "SP",
-    "MP",
-    "CP",
-    "AP",
+    "Simple",
+    "Main",
+    "Complex",
+    "Advanced",
 };
 
 //---------------------------------------------------------------------------
@@ -160,37 +161,15 @@ const int8u Vc1_FieldTypeTable[][2]=
 };
 
 //---------------------------------------------------------------------------
-int32u Vc1_ptype(int8u Size, int32u Value)
+const File__Analyze::vlc Vc1_ptype[]=
 {
-    switch (Size)
-    {
-        case 1 :
-                    switch (Value)
-                    {
-                        case 0x0 : return 1;
-                        default  : return (int32u)-1;
-                    }
-        case 2 :
-                    switch (Value)
-                    {
-                        case 0x2 : return 2;
-                        default  : return (int32u)-1;
-                    }
-        case 3 :
-                    switch (Value)
-                    {
-                        case 0x6 : return 0;
-                        default  : return (int32u)-1;
-                    }
-        case 4 :
-                    switch (Value)
-                    {
-                        case 0xE : return 3;
-                        case 0xF : return 4;
-                        default  : return (int32u)-1;
-                    }
-        default: return (int32u)-1;
-    }
+    //                                  macroblock_address_increment
+    {   0   ,   1   ,   0   ,   0   ,   1   },
+    {   2   ,   1   ,   0   ,   0   ,   2   },
+    {   6   ,   1   ,   0   ,   0   ,   0   },
+    {   14  ,   1   ,   0   ,   0   ,   3   },
+    {   15  ,   0   ,   0   ,   0   ,   4   },
+    VLC_END
 };
 
 //---------------------------------------------------------------------------
@@ -288,9 +267,11 @@ void File_Vc1::Streams_Fill()
     Fill(Stream_Video, 0, Video_Codec, From_WMV3?"WMV3":"VC-1"); //For compatibility with the old reaction
     Fill(Stream_Video, 0, Video_BitDepth, 8);
 
-    Ztring Profile=Vc1_Profile[profile];
-    if (profile==3)
-        Profile+=_T("@L")+Ztring::ToZtring(level);
+    Ztring Profile;
+    if (profile!=(int8u)-1)
+        Profile=Vc1_Profile[profile];
+    if (profile==3 && level!=(int8u)-1)
+        Profile+=__T("@L")+Ztring::ToZtring(level);
     Fill(Stream_Video, 0, Video_Format_Profile, Profile);
     Fill(Stream_Video, 0, Video_Codec_Profile, Profile);
     Fill(Stream_Video, 0, Video_Colorimetry, Vc1_ColorimetryFormat[colordiff_format]);
@@ -323,7 +304,7 @@ void File_Vc1::Streams_Fill()
     if (Frame_Count>0 && interlace)
         Fill(Stream_Video, 0, Video_ScanOrder, Interlaced_Bottom?"BFF":"TFF");
     std::string TempRef;
-    for (std::map<int16u, temporalreference>::iterator Temp=TemporalReference.begin(); Temp!=TemporalReference.end(); Temp++)
+    for (std::map<int16u, temporalreference>::iterator Temp=TemporalReference.begin(); Temp!=TemporalReference.end(); ++Temp)
     {
         TempRef+=Temp->second.top_field_first?"T":"B";
         TempRef+=Temp->second.repeat_first_field?"3":"2";
@@ -408,22 +389,27 @@ bool File_Vc1::FileHeader_Begin()
 bool File_Vc1::Synched_Test()
 {
     //Must have enough buffer for having header
-    if (Buffer_Offset+3>Buffer_Size)
+    if (Buffer_Offset+4>Buffer_Size)
         return false;
 
     //Quick test of synchro
-    if (CC3(Buffer+Buffer_Offset)!=0x000001)
+    if (Buffer[Buffer_Offset  ]!=0x00
+     || Buffer[Buffer_Offset+1]!=0x00
+     || Buffer[Buffer_Offset+2]!=0x01)
+    {
         Synched=false;
+        return true;
+    }
 
     //Quick search
-    if (Synched && !Header_Parser_QuickSearch())
+    if (!Header_Parser_QuickSearch())
         return false;
 
     #if MEDIAINFO_IBI
         bool RandomAccess=Buffer[Buffer_Offset+3]==0x0F; //SequenceHeader
         if (RandomAccess)
             Ibi_Add();
-    #endif MEDIAINFO_IBI
+    #endif //MEDIAINFO_IBI
 
         //We continue
     return true;
@@ -445,8 +431,8 @@ void File_Vc1::Synched_Init()
     framerateexp=0;
     frameratecode_enr=0;
     frameratecode_dr=0;
-    profile=0;
-    level=0;
+    profile=(int8u)-1;
+    level=(int8u)-1;
     colordiff_format=0;
     AspectRatio=0;
     AspectRatioX=0;
@@ -502,7 +488,7 @@ bool File_Vc1::Demux_UnpacketizeContainer_Test()
                 Demux_Offset+=2;
                 while(Demux_Offset<Buffer_Size && Buffer[Buffer_Offset]!=0x00)
                     Demux_Offset+=2;
-                if (Demux_Offset<Buffer_Size && Buffer[Demux_Offset-1]==0x00 || Demux_Offset>=Buffer_Size)
+                if (Demux_Offset>=Buffer_Size || Buffer[Demux_Offset-1]==0x00)
                     Demux_Offset--;
             }
 
@@ -621,7 +607,7 @@ bool File_Vc1::Header_Parser_Fill_Size()
         Buffer_Offset_Temp+=2;
         while(Buffer_Offset_Temp<Buffer_Size && Buffer[Buffer_Offset_Temp]!=0x00)
             Buffer_Offset_Temp+=2;
-        if (Buffer_Offset_Temp<Buffer_Size && Buffer[Buffer_Offset_Temp-1]==0x00 || Buffer_Offset_Temp>=Buffer_Size)
+        if (Buffer_Offset_Temp>=Buffer_Size || Buffer[Buffer_Offset_Temp-1]==0x00)
             Buffer_Offset_Temp--;
 
         if (start_code==0x0D) //FrameHeader, we need only few bytes
@@ -671,16 +657,15 @@ bool File_Vc1::Header_Parser_QuickSearch()
         Synched=false;
         if (!Synchronize())
         {
-            if (File_Offset+Buffer_Size==File_Size)
-            {
-                Synched=true;
-                return true;
-            }
+            UnSynched_IsNotJunk=true;
             return false;
         }
 
         if (Buffer_Offset+4>Buffer_Size)
+        {
+            UnSynched_IsNotJunk=true;
             return false;
+        }
     }
 
     if (Buffer_Offset+3==Buffer_Size)
@@ -720,13 +705,11 @@ void File_Vc1::FrameHeader()
 {
     //Name
     Element_Name("FrameHeader");
-    Element_Info(Ztring(_T("Frame ")+Ztring::ToZtring(Frame_Count)));
+    Element_Info1(Ztring(__T("Frame ")+Ztring::ToZtring(Frame_Count)));
     if (FrameRate)
     {
-        if (FrameInfo.PTS!=(int64u)-1)
-            Element_Info(_T("PTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.PTS)/1000000+Frame_Count_InThisBlock*1000/FrameRate)));
-        if (FrameInfo.DTS!=(int64u)-1)
-            Element_Info(_T("DTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.DTS)/1000000)));
+        Element_Info1C((FrameInfo.PTS!=(int64u)-1), __T("PTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.PTS)/1000000+Frame_Count_InThisBlock*1000/FrameRate)));
+        Element_Info1C((FrameInfo.DTS!=(int64u)-1), __T("DTS ")+Ztring().Duration_From_Milliseconds(float64_int64s(((float64)FrameInfo.DTS)/1000000)));
     }
 
     //Counting
@@ -739,7 +722,7 @@ void File_Vc1::FrameHeader()
 
     //Parsing
     BS_Begin();
-    int8u ptype;
+    int8u ptype=(int8u)-1;
     if (profile==3) //Advanced
     {
         int8u PictureFormat=0; //Default=Progressive frame
@@ -754,7 +737,7 @@ void File_Vc1::FrameHeader()
                 PictureFormat=fcm_2?2:1; //Interlaced Field : Interlaced Frame
             }
         }
-        Param_Info(Vc1_PictureFormat[PictureFormat]);
+        Param_Info1(Vc1_PictureFormat[PictureFormat]);
         PictureFormat_Count[PictureFormat]++;
 
         if (PictureFormat==2) //Interlaced Field
@@ -763,8 +746,8 @@ void File_Vc1::FrameHeader()
             Get_S1 ( 3, ptype_,                                 "ptype");
             if (ptype_<5)
             {
-                Param_Info(Vc1_Type[Vc1_FieldTypeTable[ptype_][0]]); Element_Info(Vc1_Type[Vc1_FieldTypeTable[ptype_][0]]); //First field
-                Param_Info(Vc1_Type[Vc1_FieldTypeTable[ptype_][1]]); Element_Info(Vc1_Type[Vc1_FieldTypeTable[ptype_][1]]); //Second field
+                Param_Info1(Vc1_Type[Vc1_FieldTypeTable[ptype_][0]]); Element_Info1(Vc1_Type[Vc1_FieldTypeTable[ptype_][0]]); //First field
+                Param_Info1(Vc1_Type[Vc1_FieldTypeTable[ptype_][1]]); Element_Info1(Vc1_Type[Vc1_FieldTypeTable[ptype_][1]]); //Second field
                 ptype=Vc1_FieldTypeTable[ptype_][0]; //Saving the ptype from the first field
             }
             else
@@ -775,9 +758,9 @@ void File_Vc1::FrameHeader()
         }
         else
         {
-            int32u ptype_;
-            Get_VL (Vc1_ptype, ptype_,                          "ptype"); if (ptype_<5) {Param_Info(Vc1_Type[(size_t)ptype_]); Element_Info(Vc1_Type[(size_t)ptype_]);}
-            ptype=(int8u)ptype_;
+            size_t ptype_;
+            Get_VL (Vc1_ptype, ptype_,                          "ptype"); if (ptype_<5) {Param_Info1(Vc1_Type[Vc1_ptype[ptype_].mapped_to3]); Element_Info1(Vc1_Type[Vc1_ptype[ptype_].mapped_to3]);}
+            ptype=(int8u)Vc1_ptype[ptype_].mapped_to3;
         }
         if (RefFramesCount<2 && (ptype==0 || ptype==1))
             RefFramesCount++;
@@ -906,38 +889,27 @@ void File_Vc1::FrameHeader()
 
         #if MEDIAINFO_EVENTS
             {
-                struct MediaInfo_Event_Video_SliceInfo_0 Event;
-                Event.EventCode=MediaInfo_EventCode_Create(MediaInfo_Parser_None, MediaInfo_Event_Video_SliceInfo, 0);
-                Event.Stream_Offset=File_Offset+Buffer_Offset;
-                Event.PCR=FrameInfo.PCR;
-                Event.PTS=FrameInfo.PTS;
-                Event.DTS=FrameInfo.DTS;
-                Event.DUR=FrameInfo.DUR;
-                Event.StreamIDs_Size=StreamIDs_Size;
-                Event.StreamIDs=(MediaInfo_int64u*)StreamIDs;
-                Event.StreamIDs_Width=(MediaInfo_int8u*)StreamIDs_Width;
-                Event.ParserIDs=(MediaInfo_int8u* )ParserIDs;
-                Event.FramePosition=Frame_Count;
-                Event.FieldPosition=Field_Count;
-                Event.SlicePosition=0;
-                switch (ptype)
-                {
-                    case 0 :
-                                Event.SliceType=0; break;
-                    case 1 :
-                                Event.SliceType=1; break;
-                    case 2 :
-                    case 3 :
-                                Event.SliceType=2; break;
-                    case 4 :
-                                Event.SliceType=3; break;
-                    default:
-                                Event.SliceType=(int8u)-1;
-                }
-                Event.Flags=0;
-                Config->Event_Send((const int8u*)&Event, sizeof(MediaInfo_Event_Video_SliceInfo_0));
+                EVENT_BEGIN (Video, SliceInfo, 0)
+                    Event.FieldPosition=Field_Count;
+                    Event.SlicePosition=0;
+                    switch (ptype)
+                    {
+                        case 0 :
+                                    Event.SliceType=0; break;
+                        case 1 :
+                                    Event.SliceType=1; break;
+                        case 2 :
+                        case 3 :
+                                    Event.SliceType=2; break;
+                        case 4 :
+                                    Event.SliceType=3; break;
+                        default:
+                                    Event.SliceType=(int8u)-1;
+                    }
+                    Event.Flags=0;
+                EVENT_END   ()
             }
-        #endif MEDIAINFO_EVENTS
+        #endif //MEDIAINFO_EVENTS
     FILLING_END();
 
     Synched=false; //We do not have the complete FrameHeader
@@ -966,13 +938,13 @@ void File_Vc1::EntryPointHeader()
     if (hrd_param_flag)
         for (int8u Pos=0; Pos<hrd_num_leaky_buckets; Pos++)
         {
-            Element_Begin("leaky_bucket");
+            Element_Begin1("leaky_bucket");
             Skip_S1( 8,                                         "hrd_full");
-            Element_End();
+            Element_End0();
         }
     TEST_SB_SKIP(                                               "coded_size_flag");
-        Info_S2(12, coded_width,                                "coded_width"); Param_Info((coded_width+1)*2, " pixels");
-        Info_S2(12, coded_height,                               "coded_height"); Param_Info((coded_height+1)*2, " pixels");
+        Info_S2(12, coded_width,                                "coded_width"); Param_Info2((coded_width+1)*2, " pixels");
+        Info_S2(12, coded_height,                               "coded_height"); Param_Info2((coded_height+1)*2, " pixels");
     TEST_SB_END();
     if (extended_mv)
         Skip_SB(                                                "extended_dmv");
@@ -1008,7 +980,7 @@ void File_Vc1::SequenceHeader()
 
     //Parsing
     BS_Begin();
-    Get_S1 ( 2, profile,                                        "profile"); Param_Info(Vc1_Profile[profile]);
+    Get_S1 ( 2, profile,                                        "profile"); Param_Info1(Vc1_Profile[profile]);
     if (profile==0 || profile==1) //Simple or Main
     {
         Skip_S1( 2,                                             "res_sm");
@@ -1034,12 +1006,12 @@ void File_Vc1::SequenceHeader()
     else if (profile==3) //Advanced
     {
         Get_S1 ( 3, level,                                      "level");
-        Get_S1 ( 2, colordiff_format,                           "colordiff_format"); Param_Info(Vc1_ColorimetryFormat[colordiff_format]);
+        Get_S1 ( 2, colordiff_format,                           "colordiff_format"); Param_Info1(Vc1_ColorimetryFormat[colordiff_format]);
         Skip_S1( 3,                                             "frmrtq_postproc");
         Skip_S1( 5,                                             "bitrtq_postproc");
         Skip_SB(                                                "postprocflag");
-        Get_S2 (12, coded_width,                                "max_coded_width"); Param_Info((coded_width+1)*2, " pixels");
-        Get_S2 (12, coded_height,                               "max_coded_height"); Param_Info((coded_height+1)*2, " pixels");
+        Get_S2 (12, coded_width,                                "max_coded_width"); Param_Info2((coded_width+1)*2, " pixels");
+        Get_S2 (12, coded_height,                               "max_coded_height"); Param_Info2((coded_height+1)*2, " pixels");
         Get_SB (    pulldown,                                   "pulldown");
         Get_SB (    interlace,                                  "interlace");
         Get_SB (    tfcntrflag,                                 "tfcntrflag - frame counter");
@@ -1047,10 +1019,10 @@ void File_Vc1::SequenceHeader()
         Skip_SB(                                                "reserved");
         Get_SB (    psf,                                        "psf - progressive segmented frame");
         TEST_SB_SKIP(                                           "display_ext");
-            Info_S2(14, display_x,                              "display_horiz_size"); Param_Info(display_x+1, " pixels");
-            Info_S2(14, display_y,                              "display_vert_size"); Param_Info(display_y+1, " pixels");
+            Info_S2(14, display_x,                              "display_horiz_size"); Param_Info2(display_x+1, " pixels");
+            Info_S2(14, display_y,                              "display_vert_size"); Param_Info2(display_y+1, " pixels");
             TEST_SB_SKIP(                                       "aspectratio_flag");
-                Get_S1 ( 4, AspectRatio,                        "aspect_ratio"); Param_Info(Vc1_PixelAspectRatio[AspectRatio]);
+                Get_S1 ( 4, AspectRatio,                        "aspect_ratio"); Param_Info1(Vc1_PixelAspectRatio[AspectRatio]);
                 if (AspectRatio==0x0F)
                 {
                     Get_S1 ( 8, AspectRatioX,                   "aspect_horiz_size");
@@ -1059,10 +1031,10 @@ void File_Vc1::SequenceHeader()
             TEST_SB_END();
             TEST_SB_GET(framerate_present,                      "framerate_flag");
                 TESTELSE_SB_GET(framerate_form,                 "framerateind");
-                    Get_S2 (16, framerateexp,                   "framerateexp"); Param_Info((float32)((framerateexp+1)/32.0), 3, " fps");
+                    Get_S2 (16, framerateexp,                   "framerateexp"); Param_Info3((float32)((framerateexp+1)/32.0), 3, " fps");
                 TESTELSE_SB_ELSE(                               "framerateind");
-                    Get_S1 ( 8, frameratecode_enr,              "frameratenr"); Param_Info(Vc1_FrameRate_enr(frameratecode_enr));
-                    Get_S1 ( 4, frameratecode_dr,               "frameratedr"); Param_Info(Vc1_FrameRate_dr(frameratecode_dr));
+                    Get_S1 ( 8, frameratecode_enr,              "frameratenr"); Param_Info1(Vc1_FrameRate_enr(frameratecode_enr));
+                    Get_S1 ( 4, frameratecode_dr,               "frameratedr"); Param_Info1(Vc1_FrameRate_dr(frameratecode_dr));
                 TESTELSE_SB_END();
             TEST_SB_END();
             TEST_SB_SKIP(                                       "color_format_flag");
@@ -1079,12 +1051,12 @@ void File_Vc1::SequenceHeader()
             hrd_buffers.clear();
             for (int8u Pos=0; Pos<hrd_num_leaky_buckets; Pos++)
             {
-                Element_Begin("leaky_bucket");
+                Element_Begin1("leaky_bucket");
                 int16u hrd_buffer;
                 Skip_S2(16,                                     "hrd_rate");
                 Get_S2(16, hrd_buffer,                         "hrd_buffer");
-                int32u hrd_buffer_value=(int32u)((hrd_buffer+1)*pow(2.0, 1+buffer_size_exponent)); Param_Info(hrd_buffer_value, " bytes");
-                Element_End();
+                int32u hrd_buffer_value=(int32u)((hrd_buffer+1)*pow(2.0, 1+buffer_size_exponent)); Param_Info2(hrd_buffer_value, " bytes");
+                Element_End0();
 
                 //Filling
                 hrd_buffers.push_back(hrd_buffer_value);
