@@ -1,5 +1,5 @@
 // ZenLib::Ztring - More methods for std::(w)string
-// Copyright (C) 2002-2011 MediaArea.net SARL, Info@MediaArea.net
+// Copyright (C) 2002-2012 MediaArea.net SARL, Info@MediaArea.net
 //
 // This software is provided 'as-is', without any express or implied
 // warranty.  In no event will the authors be held liable for any damages
@@ -30,9 +30,7 @@
 //---------------------------------------------------------------------------
 
 //---------------------------------------------------------------------------
-#include "ZenLib/Conf.h"
 #include "ZenLib/Utils.h"
-#include "ZenLib/int128u.h"
 #include <string>
 #include <sstream>
 //---------------------------------------------------------------------------
@@ -196,10 +194,10 @@ public :
     Ztring& From_Number  (const float64,  int8u AfterComma=3, ztring_t Options=Ztring_Nothing);
         /// @brief convert number into Ztring
     Ztring& From_Number  (const float80,  int8u AfterComma=3, ztring_t Options=Ztring_Nothing);
-    #ifdef NEED_SIZET
+    #ifdef SIZE_T_IS_LONG
         /// @brief convert number into Ztring
     Ztring& From_Number  (const size_t,   int8u Radix=10);
-    #endif //NEED_SIZET
+    #endif //SIZE_T_IS_LONG
         /// @brief convert number (BCD coded) into Ztring
     Ztring& From_BCD     (const int8u);
         /// @brief convert count of milliseconds into a readable and sortable string
@@ -314,9 +312,9 @@ public :
     static Ztring ToZtring  (const float32  F, int8u AfterComma=3)               {return Ztring().From_Number(F, AfterComma);};
     static Ztring ToZtring  (const float64  F, int8u AfterComma=3)               {return Ztring().From_Number(F, AfterComma);};
     static Ztring ToZtring  (const float80  F, int8u AfterComma=3)               {return Ztring().From_Number(F, AfterComma);};
-    #ifdef NEED_SIZET
+    #ifdef SIZE_T_IS_LONG
     static Ztring ToZtring  (const size_t   I,  int8u Radix=10)                  {return Ztring().From_Number(I, Radix);};
-    #endif //NEED_SIZET
+    #endif //SIZE_T_IS_LONG
 
     //Edition
         /// @brief test if it is a number
@@ -326,13 +324,13 @@ public :
         /// @brief convert into uppercase
     Ztring &MakeUpperCase();
         /// @brief Remove leading whitespaces from a string
-    Ztring &TrimLeft(Char ToTrim=_T(' '));
+    Ztring &TrimLeft(Char ToTrim=__T(' '));
         /// @brief Remove trailing whitespaces from a string
-    Ztring &TrimRight(Char ToTrim=_T(' '));
+    Ztring &TrimRight(Char ToTrim=__T(' '));
         /// @brief Remove leading and trailing whitespaces from a string
-    Ztring &Trim(Char ToTrim=_T(' '));
+    Ztring &Trim(Char ToTrim=__T(' '));
         /// @brief Quotes a string
-    Ztring &Quote(Char ToTrim=_T('\"'));
+    Ztring &Quote(Char ToTrim=__T('\"'));
         /// @brief return a string between two strings
         /// @param Begin First string
         /// @param End Second string
@@ -363,10 +361,9 @@ public :
         /// @param Options Options for comaparing \n
         ///                Available : Ztring_CaseSensitive
         /// @return The result of comparasion
-    bool Compare (const Ztring &ToCompare, const Ztring &Comparator=_T("=="), ztring_t Options=Ztring_Nothing) const;
+    bool Compare (const Ztring &ToCompare, const Ztring &Comparator=__T("=="), ztring_t Options=Ztring_Nothing) const;
 };
 
 } //NameSpace
 
 #endif
-
